@@ -1,7 +1,7 @@
 /*
  * SiteRateModel.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,52 +25,15 @@
 
 package dr.app.beagle.evomodel.sitemodel;
 
-import dr.inference.model.Model;
-
 /**
  * SiteRateModel - Specifies how rates vary across sites. Unlike 'SiteModel'
  * this only deals with rates for site categories.
  *
  * @author Andrew Rambaut
  * @author Alexei Drummond
- *
  * @version $Id: SiteModel.java,v 1.77 2005/05/24 20:25:58 rambaut Exp $
  */
 
-public interface SiteRateModel extends Model {
-
-	/**
-	 * @return the number of categories of substitution processes
-	 */
-	int getCategoryCount();
-
-    /**
-     * Get an array of the relative rates of sites in each category. These
-     * may include the 'mu' parameter, an overall scaling of the siteRateModel.
-     * @return an array of the rates.
-     */
-    double[] getCategoryRates();
-
-	/**
-	 * Get an array of the expected proportion of sites in each category.
-	 * @return an array of the proportions.
-	 */
-	double[] getCategoryProportions();
-
-    /**
-     * Get the rate for a particular category. This may include the 'mu'
-     * parameter, an overall scaling of the siteRateModel.
-     * @param category the category number
-     * @return the rate.
-     */
-    double getRateForCategory(int category);
-
-    /**
-     * Get the expected proportion of sites in this category.
-     * @param category the category number
-     * @return the proportion.
-     */
-    double getProportionForCategory(int category);
-
-
+public interface SiteRateModel extends dr.evomodel.sitemodel.SiteRateModel {
+    // Interface description remains compatibility with non-BEAGLE classes, so delegated to SiteRateModel
 }
\ No newline at end of file
