@@ -1,7 +1,7 @@
 /*
  * CodonPartitionedRobustCounting.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -65,6 +65,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
     public static final String UNCONDITIONED_PREFIX = "u_";
     public static final String SITE_SPECIFIC_PREFIX = "c_";
     public static final String TOTAL_PREFIX = "total_";
+    public static final String UNCONDITIONED_TOTAL_PREFIX = "utotal_";
     public static final String BASE_TRAIT_PREFIX = "base_";
     public static final String COMPLETE_HISTORY_PREFIX = "all_";
     public static final String UNCONDITIONED_PER_BRANCH_PREFIX = "b_u_";
@@ -88,8 +89,29 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
                                           boolean includeInternalBranches,
                                           boolean doUnconditionalPerBranch,
                                           boolean saveCompleteHistory,
+                                          boolean tryNewNeutralModel,
                                           StratifiedTraitOutputFormat branchFormat,
-                                          StratifiedTraitOutputFormat logFormat) {
+                                          StratifiedTraitOutputFormat logFormat,
+                                          String prefix) {
+        this(name, tree, partition, codons, codonLabeling, useUniformization, includeExternalBranches,
+                includeInternalBranches, doUnconditionalPerBranch, saveCompleteHistory, false, tryNewNeutralModel,
+                branchFormat, logFormat, prefix);
+    }
+
+    public CodonPartitionedRobustCounting(String name, TreeModel tree,
+                                          AncestralStateBeagleTreeLikelihood[] partition,
+                                          Codons codons,
+                                          CodonLabeling codonLabeling,
+                                          boolean useUniformization,
+                                          boolean includeExternalBranches,
+                                          boolean includeInternalBranches,
+                                          boolean doUnconditionalPerBranch,
+                                          boolean saveCompleteHistory,
+                                          boolean forceUnconditionalAverageRate,
+                                          boolean tryNewNeutralModel,
+                                          StratifiedTraitOutputFormat branchFormat,
+                                          StratifiedTraitOutputFormat logFormat,
+                                          String prefix) {
         super(name);
         this.tree = tree;
         addModel(tree);
@@ -119,16 +141,33 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         this.saveCompleteHistory = saveCompleteHistory;
 
         productChainModel =
-                new ProductChainSubstitutionModel("codonLabeling", substModelsList, siteRateModelsList);
+                new ProductChainSubstitutionModel("codonLabeling", substModelsList, siteRateModelsList, false);
         addModel(productChainModel);
 
+        this.forceUnconditionalAverageRate = forceUnconditionalAverageRate;
+
+        if (forceUnconditionalAverageRate) {
+            averagedProductChainModel = new ProductChainSubstitutionModel("codonLabeling", substModelsList, siteRateModelsList, true);
+            addModel(averagedProductChainModel);
+        }
+
         this.useUniformization = useUniformization;
         if (useUniformization) {
             markovJumps = new UniformizedSubstitutionModel(productChainModel);
             ((UniformizedSubstitutionModel) markovJumps).setSaveCompleteHistory(saveCompleteHistory);
+
+            if (forceUnconditionalAverageRate) {
+                averagedMarkovJumps = new UniformizedSubstitutionModel(averagedProductChainModel);
+                ((UniformizedSubstitutionModel) averagedMarkovJumps).setSaveCompleteHistory(saveCompleteHistory);
+            }
         } else {
             markovJumps = new MarkovJumpsSubstitutionModel(productChainModel);
+
+            if (forceUnconditionalAverageRate) {
+                averagedMarkovJumps = new MarkovJumpsSubstitutionModel(averagedProductChainModel);
+            }
         }
+
         double[] synRegMatrix = CodonLabeling.getRegisterMatrix(codonLabeling, codons, true);
         markovJumps.setRegistration(synRegMatrix);
 
@@ -143,6 +182,12 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         this.includeInternalBranches = includeInternalBranches;
         this.doUnconditionedPerBranch = doUnconditionalPerBranch;
 
+        this.tryNewNeutralModel = tryNewNeutralModel;
+
+        //this.neutralSubstitutionModel = null; // new ComplexSubstitutionModel();
+
+        this.prefix = prefix;
+
         setupTraits();
     }
 
@@ -409,8 +454,10 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         };
 
         // This should be the default output in columns logs
+        String name = prefix != null ? prefix + TOTAL_PREFIX + codonLabeling.getText() :
+                TOTAL_PREFIX + codonLabeling.getText();
         TreeTrait sumOverSitesAndTreeTrait = new TreeTrait.SumOverTreeD(
-                TOTAL_PREFIX + codonLabeling.getText(),
+                name,
                 sumOverSitesTrait,
                 includeExternalBranches,
                 includeInternalBranches) {
@@ -461,6 +508,21 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
                 }
             };
 
+            String nameU = prefix != null ? prefix + UNCONDITIONED_TOTAL_PREFIX + codonLabeling.getText() :
+                    UNCONDITIONED_TOTAL_PREFIX + codonLabeling.getText();
+            TreeTrait sumUnconditionedOverSitesAndTreeTrait = new TreeTrait.SumOverTreeD(
+                    nameU,
+                    sumUnconditionedOverSitesTrait,
+                    includeExternalBranches,
+                    includeInternalBranches) {
+                public boolean getLoggable() {
+                    return true;
+                }
+            };
+
+            treeTraitLogger = new TreeTraitLogger(tree,
+                    new TreeTrait[]{sumOverSitesAndTreeTrait, sumUnconditionedOverSitesAndTreeTrait});
+
             treeTraits.addTrait(unconditionedBase);
             treeTraits.addTrait(sumUnconditionedOverSitesTrait);
         }
@@ -503,7 +565,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         if (unconditionedCountsPerBranch == null) {
             unconditionedCountsPerBranch = new double[tree.getNodeCount()][numCodons];
         }
-        double[] rootDistribution = productChainModel.getFrequencyModel().getFrequencies();
+        double[] rootDistribution = getUnconditionalRootDistribution();
         for (int i = 0; i < tree.getNodeCount(); i++) {
             NodeRef node = tree.getNode(i);
             if (!tree.isRoot(node)) {
@@ -518,7 +580,7 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
             unconditionedCounts = new double[numCodons];
         }
         final double treeLength = getExpectedTreeLength();
-        double[] rootDistribution = productChainModel.getFrequencyModel().getFrequencies();
+        double[] rootDistribution = getUnconditionalRootDistribution();
 //        final int stateCount = 64;
 //        double[] lambda = new double[stateCount * stateCount];
 //        productChainModel.getInfinitesimalMatrix(lambda);
@@ -536,10 +598,26 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
         fillInUnconditionalTraitValues(treeLength, rootDistribution, unconditionedCounts);
     }
 
+    private double[] getUnconditionalRootDistribution() {
+        if (forceUnconditionalAverageRate) {
+            return averagedProductChainModel.getFrequencyModel().getFrequencies();
+        } else {
+            return productChainModel.getFrequencyModel().getFrequencies();
+        }
+    }
+
+    private void fillInUnconditionalQMatrix(double[] lambda) {
+        if (forceUnconditionalAverageRate) {
+            averagedProductChainModel.getInfinitesimalMatrix(lambda);
+        } else {
+            productChainModel.getInfinitesimalMatrix(lambda);
+        }
+    }
+
     private void fillInUnconditionalTraitValues(double expectedLength, double[] freq, double[] out) {
         final int stateCount = 64;
         double[] lambda = new double[stateCount * stateCount];
-        productChainModel.getInfinitesimalMatrix(lambda);
+        fillInUnconditionalQMatrix(lambda);
         for (int i = 0; i < numCodons; i++) {
             final int startingState = MathUtils.randomChoicePDF(freq);
             StateHistory history = StateHistory.simulateUnconditionalOnEndingState(
@@ -563,14 +641,15 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
     public Double getUnconditionedTraitValue() {
         if (!TRIAL) {
-            return markovJumps.getMarginalRate() * getExpectedTreeLength();
+            throw new RuntimeException("Believed broken for neutral models");
+//            return markovJumps.getMarginalRate() * getExpectedTreeLength();
         } else {
             final double treeLength = getExpectedTreeLength();
-            double[] rootDistribution = productChainModel.getFrequencyModel().getFrequencies();
+            double[] rootDistribution = getUnconditionalRootDistribution();
             final int startingState = MathUtils.randomChoicePDF(rootDistribution);
             final int stateCount = 64;
             double[] lambda = new double[stateCount * stateCount];
-            productChainModel.getInfinitesimalMatrix(lambda);
+            fillInUnconditionalQMatrix(lambda);
             StateHistory history = StateHistory.simulateUnconditionalOnEndingState(
                     0.0,
                     startingState,
@@ -632,13 +711,20 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
     private final AncestralStateBeagleTreeLikelihood[] partition;
     private final MarkovJumpsSubstitutionModel markovJumps;
+    private MarkovJumpsSubstitutionModel averagedMarkovJumps = null;
+
+    private final boolean forceUnconditionalAverageRate;
+
     private final boolean useUniformization;
     private final BranchRateModel branchRateModel;
     private final ProductChainSubstitutionModel productChainModel;
+    private ProductChainSubstitutionModel averagedProductChainModel = null;
 
     private final CodonLabeling codonLabeling;
     private final Tree tree;
 
+    private final String prefix;
+
     private final StratifiedTraitOutputFormat branchFormat;
     private final StratifiedTraitOutputFormat logFormat;
 
@@ -666,4 +752,6 @@ public class CodonPartitionedRobustCounting extends AbstractModel implements Tre
 
     private boolean saveCompleteHistory = false;
 
+    private boolean tryNewNeutralModel = false;
+
 }
