@@ -1,7 +1,7 @@
 /*
  * DependentProductChainSubstitutionModel.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -28,18 +28,15 @@ package dr.app.beagle.evomodel.substmodel;
 import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
 import dr.inference.model.Parameter;
 import dr.math.KroneckerOperation;
-import dr.app.beagle.evomodel.substmodel.*;
-
 
 import java.util.List;
 
 /**
  * A derived class from ProductChainSubstitutionModel that introduces dependence between sites.
  *
- * @version 08/01/2010
- *
  * @author Marc A. Suchard
  * @author Yu-Nong Gong
+ * @version 08/01/2010
  */
 public class DependentProductChainSubstitutionModel extends ProductChainSubstitutionModel {
 
@@ -53,6 +50,8 @@ public class DependentProductChainSubstitutionModel extends ProductChainSubstitu
         super(name, baseModels, rateModels);
         this.dependenceParameter = dependenceParameter;
 
+        eigenSystemCES = new ColtEigenSystem(stateCount);
+
     }
 
     public EigenDecomposition getEigenDecomposition() {
@@ -65,9 +64,9 @@ public class DependentProductChainSubstitutionModel extends ProductChainSubstitu
     }
 
     public void getInfinitesimalMatrix(double[] out) {
-            getEigenDecomposition(); // Updates rate matrix if necessary
-            System.arraycopy(rateMatrix, 0, out, 0, stateCount * stateCount);
-        }
+        getEigenDecomposition(); // Updates rate matrix if necessary
+        System.arraycopy(rateMatrix, 0, out, 0, stateCount * stateCount);
+    }
 
     private void computeKroneckerSumsAndProducts() {
 
@@ -89,59 +88,59 @@ public class DependentProductChainSubstitutionModel extends ProductChainSubstitu
         double dependence = dependenceParameter.getParameterValue(0);
 
         /* put dependence parameter into anti-diagonal elements*/
-            for (int j = 0; j < stateCount; j++) {
-                currentRate[(j+1) * stateCount - (j+1)] = dependence;
-            }
+        for (int j = 0; j < stateCount; j++) {
+            currentRate[(j + 1) * stateCount - (j + 1)] = dependence;
+        }
 
         /* put dependence parameter into co-varying sites */
 
             /* take nucleotide for example, A, T, C, and G as 0, 1, 2, and 3 */
-            String[] letA = new String[(int) Math.sqrt(stateCount)];
-            String[] letB = new String[(int) Math.sqrt(stateCount)];
-            String[] letAB = new String[stateCount];
-            int count = 0;
-    
-            for(int letI=0;letI<letA.length;letI++){
-                letA[letI]=String.valueOf(count);
-                letB[letI]=String.valueOf(count);
-                count++;
-            }
+        String[] letA = new String[(int) Math.sqrt(stateCount)];
+        String[] letB = new String[(int) Math.sqrt(stateCount)];
+        String[] letAB = new String[stateCount];
+        int count = 0;
+
+        for (int letI = 0; letI < letA.length; letI++) {
+            letA[letI] = String.valueOf(count);
+            letB[letI] = String.valueOf(count);
+            count++;
+        }
 
             /* generate two-state array {(00), (01), (02), (03), (10), ..., (33)} */
-            int letABi=0;
-            for(int letAi=0;letAi<letA.length;letAi++){
-                for(int letBi=0;letBi<letB.length;letBi++){
-                letAB[letABi]=letA[letAi]+" "+letB[letBi];
+        int letABi = 0;
+        for (int letAi = 0; letAi < letA.length; letAi++) {
+            for (int letBi = 0; letBi < letB.length; letBi++) {
+                letAB[letABi] = letA[letAi] + " " + letB[letBi];
                 letABi++;
-                }
             }
+        }
 
             /* assign dependence parameter to co-varying sites */
-            for(int j=0;j<letAB.length;j++){
-                for(int k=0;k<letAB.length;k++){
+        for (int j = 0; j < letAB.length; j++) {
+            for (int k = 0; k < letAB.length; k++) {
                 String[] Spl1 = letAB[j].split(" ");
                 String[] Spl2 = letAB[k].split(" ");
 
-                if(!Spl1[0].equals(Spl2[0]) && !Spl1[1].equals(Spl2[1])){
-					/* Is co-varying site */
+                if (!Spl1[0].equals(Spl2[0]) && !Spl1[1].equals(Spl2[1])) {
+                                                  /* Is co-varying site */
                     currentRate[j * stateCount + k] = dependence;
                 }
 
-                }
             }
+        }
 
         /* Rescale diagonal elements in currentRate */
-            double diagonalSum=0;
-            for (int k = 0; k < stateCount; k++) {
-                for(int l=k*stateCount;l<((k+1)*stateCount);l++){
-                    if(l!=(k * stateCount + k)){
+        double diagonalSum = 0;
+        for (int k = 0; k < stateCount; k++) {
+            for (int l = k * stateCount; l < ((k + 1) * stateCount); l++) {
+                if (l != (k * stateCount + k)) {
                     diagonalSum += currentRate[l];
-                    }
                 }
-                /* assign value to diagonal element */
-                currentRate[k * stateCount + k] = -(diagonalSum);
-                diagonalSum=0;
             }
+                /* assign value to diagonal element */
+            currentRate[k * stateCount + k] = -(diagonalSum);
+            diagonalSum = 0;
+        }
 
         rateMatrix = currentRate;
 
@@ -163,9 +162,9 @@ public class DependentProductChainSubstitutionModel extends ProductChainSubstitu
 
         updateMatrix = false;
     }
-    
+
     private final Parameter dependenceParameter;
 
-    private final EigenSystem eigenSystemCES = new ColtEigenSystem();
+    private final EigenSystem eigenSystemCES; //= new ColtEigenSystem();
 
 }
