@@ -1,7 +1,7 @@
 /*
  * EigenDecomposition.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,13 +25,15 @@
 
 package dr.app.beagle.evomodel.substmodel;
 
+import java.io.Serializable;
+
 /**
  * @author Andrew Rambaut
  * @author Alexei Drummond
  * @Author Marc A. Suchard
  * @version $Id$
  */
-public class EigenDecomposition {
+public class EigenDecomposition implements Serializable {
 
     public EigenDecomposition(double[] evec, double[] ievc, double[] eval) {
         Evec = evec;
