@@ -1,7 +1,7 @@
 /*
  * HKY.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -128,7 +128,7 @@ public class HKY extends BaseSubstitutionModel {
         rates[5] = 1.0;
     }
 
-    public EigenDecomposition getEigenDecomposition() {
+    public synchronized EigenDecomposition getEigenDecomposition() {
 
         if (eigenDecomposition == null) {
             double[] evec = new double[stateCount * stateCount];
