@@ -1,7 +1,7 @@
 /*
  * MarkovModulatedSubstitutionModel.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,7 +25,11 @@
 
 package dr.app.beagle.evomodel.substmodel;
 
+import dr.app.beagle.evomodel.sitemodel.SiteRateModel;
 import dr.evolution.datatype.DataType;
+import dr.inference.loggers.LogColumn;
+import dr.inference.loggers.Loggable;
+import dr.inference.loggers.NumberColumn;
 import dr.inference.model.Model;
 import dr.inference.model.Parameter;
 import dr.inference.model.Variable;
@@ -41,7 +45,7 @@ import java.util.logging.Logger;
 /**
  * @author Marc A. Suchard
  */
-public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel implements Citable {
+public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel implements Citable, Loggable {
 
     private List<SubstitutionModel> baseModels;
     private final int numBaseModel;
@@ -58,12 +62,14 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
     private boolean birthDeathModel;
     private boolean geometricRates;
 
+    private final SiteRateModel gammaRateModel;
+
     public MarkovModulatedSubstitutionModel(String name,
                                             List<SubstitutionModel> baseModels,
                                             Parameter switchingRates,
                                             DataType dataType,
                                             EigenSystem eigenSystem) {
-        this(name, baseModels, switchingRates, dataType, eigenSystem, null, false);
+        this(name, baseModels, switchingRates, dataType, eigenSystem, null, false, null);
     }
 
     public MarkovModulatedSubstitutionModel(String name,
@@ -72,7 +78,8 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
                                             DataType dataType,
                                             EigenSystem eigenSystem,
                                             Parameter rateScalar,
-                                            boolean geometricRates) {
+                                            boolean geometricRates,
+                                            SiteRateModel gammaRateModel) {
 //        super(name, dataType, null, eigenSystem);
         super(name, dataType, null, null);
 
@@ -113,10 +120,13 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
         if (numBaseModel > 1) {
             if (switchingRates.getDimension() != 2 * (numBaseModel - 1)) {
                 birthDeathModel = false;
-                throw new RuntimeException("Wrong dimension of switching rates in MarkovModulatedSubstitutionModel");
+//                throw new RuntimeException("Wrong dimension of switching rates in MarkovModulatedSubstitutionModel " + switchingRates.getDimension() + " " + 2 * (numBaseModel - 1) + " " + numBaseModel);
             }
         }
 
+        if (gammaRateModel != null) addModel(gammaRateModel);
+        this.gammaRateModel = gammaRateModel;
+
         if (rateScalar != null) addVariable(rateScalar);
         this.rateScalar = rateScalar;
 
@@ -128,7 +138,15 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
                 + Citable.Utils.getCitationString(this));
     }
 
-    private double getModelRateScalar(int model) {
+    public int getNumBaseModel() {
+        return numBaseModel;
+    }
+
+    public double getModelRateScalar(int model) {
+        if (gammaRateModel != null) {
+//            System.err.println("M" + model + " = " + gammaRateModel.getRateForCategory(model));
+            return gammaRateModel.getRateForCategory(model);
+        }
         if (rateScalar == null) {
             return 1.0;
         } else {
@@ -164,6 +182,10 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
         // Add switching rates to matrix
         if (!IGNORE_RATES && numBaseModel > 1) {
             double[] swRates = switchingRates.getParameterValues();
+            double totalRate = 0.0;
+            for (double rate : swRates) {
+                totalRate += rate;
+            }
             int sw = 0;
             for (int g = 0; g < numBaseModel; ++g) {
                 for (int h = 0; h < numBaseModel; ++h) { // from g -> h
@@ -171,7 +193,8 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
                     if (valid) {
                         double rate = swRates[sw];
                         if (geometricRates) {
-                            rate *= getModelRateScalar(numBaseModel - h - 1); // TODO Why not: "/ numBaseModel" ??
+                            rate *= getModelRateScalar(numBaseModel - h - 1) /// numBaseModel; // TODO Why not: "/ numBaseModel" ??
+                                    / totalRate;
                         }
                         for (int i = 0; i < baseStateCount; ++i) {
                             matrix[g * baseStateCount + i][h * baseStateCount + i] = rate;
@@ -235,7 +258,42 @@ public class MarkovModulatedSubstitutionModel extends ComplexSubstitutionModel i
         if (variable == switchingRates || variable == rateScalar) {
             // Update rates
             updateMatrix = true;
+            if (variable == rateScalar)
+                fireModelChanged(); // TODO Determine if necessary for ExposeRateCategoriesWrapper
         }
         // else do nothing, action taken care of at individual base models
     }
+
+    public LogColumn[] getColumns() {
+
+        List<LogColumn> columns = new ArrayList<LogColumn>();
+        for (LogColumn parentColumn : super.getColumns()) {
+            columns.add(parentColumn);
+        }
+
+        for (int i = 0; i < numBaseModel; ++i) {
+            String label = "rateScalar." + i;
+            columns.add(new RateColumn(label, i));
+        }
+
+        return columns.toArray(new LogColumn[0]);
+    }
+
+    private class RateColumn extends NumberColumn {
+
+        private final int index;
+
+        public RateColumn(String label, int index) {
+            super(label);
+            this.index = index;
+        }
+
+        /**
+         * Returns the current value as a double.
+         */
+        @Override
+        public double getDoubleValue() {
+            return getModelRateScalar(index);
+        }
+    }
 }
