@@ -1,7 +1,7 @@
 /*
  * SubstitutionModel.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,7 +25,6 @@
 
 package dr.app.beagle.evomodel.substmodel;
 
-import dr.evolution.datatype.DataType;
 import dr.inference.model.Model;
 
 /**
@@ -36,43 +35,6 @@ import dr.inference.model.Model;
  * @author Alexei Drummond
  * @version $Id: SubstitutionModel.java,v 1.13 2005/05/24 20:25:58 rambaut Exp $
  */
-public interface SubstitutionModel extends Model {
-
-    /**
-     * Get the complete transition probability matrix for the given distance.
-     *
-     * @param distance the time (branch length)
-     * @param matrix   an array to store the matrix
-     */
-    void getTransitionProbabilities(double distance, double[] matrix);
-
-    /**
-     * This function returns the Eigen vectors.
-     *
-     * @return the array
-     */
-    EigenDecomposition getEigenDecomposition();
-
-    /**
-     * get the state frequencies
-     *
-     * @return the frequencies
-     */
-    FrequencyModel getFrequencyModel();
-
-    /**
-     * Get the infinitesimal rate matrix
-     * @return the rates
-     */
-    public void getInfinitesimalMatrix(double[] matrix);
-
-    /**
-     * @return the data type
-     */
-    DataType getDataType();
-
-    /**
-     * @return if substitution model can return complex diagonalizations
-     */
-    boolean canReturnComplexDiagonalization();
+public interface SubstitutionModel extends SubstitutionProcess, Model {
+    // Combines SubstitutionProcess and Model
 }
\ No newline at end of file
