@@ -1,7 +1,7 @@
 /*
  * TN93.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -76,7 +76,7 @@ public class TN93 extends BaseSubstitutionModel {
         rates[5] = 1.0;
     }
 
-    public EigenDecomposition getEigenDecomposition() {
+    public synchronized EigenDecomposition getEigenDecomposition() {
 
         if (eigenDecomposition == null) {
             double[] evec = new double[stateCount * stateCount];
