@@ -1,7 +1,7 @@
 /*
  * AbstractTreeLikelihood.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -44,18 +44,10 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
 
     protected static final boolean COUNT_TOTAL_OPERATIONS = false;
 
-    public AbstractTreeLikelihood(String name, PatternList patternList,
-                                  TreeModel treeModel) {
+    public AbstractTreeLikelihood(String name, TreeModel treeModel) {
 
         super(name);
 
-        this.patternList = patternList;
-        this.dataType = patternList.getDataType();
-        patternCount = patternList.getPatternCount();
-        stateCount = dataType.getStateCount();
-
-        patternWeights = patternList.getPatternWeights();
-
         this.treeModel = treeModel;
         addModel(treeModel);
 
@@ -117,32 +109,6 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
         likelihoodKnown = false;
     }
 
-    /**
-     * Set update flag for a pattern
-     */
-    protected void updatePattern(int i) {
-        if (updatePattern != null) {
-            updatePattern[i] = true;
-        }
-        likelihoodKnown = false;
-    }
-
-    /**
-     * Set update flag for all patterns
-     */
-    protected void updateAllPatterns() {
-        if (updatePattern != null) {
-            for (int i = 0; i < patternCount; i++) {
-                updatePattern[i] = true;
-            }
-        }
-        likelihoodKnown = false;
-    }
-
-    public final double[] getPatternWeights() {
-        return patternWeights;
-    }
-
     // **************************************************************
     // VariableListener IMPLEMENTATION
     // **************************************************************
@@ -198,7 +164,7 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
         }
         if (!likelihoodKnown) {
             if (COUNT_TOTAL_OPERATIONS)
-                totalcalculateLikelihoodCount++;
+                totalCalculateLikelihoodCount++;
             logLikelihood = calculateLogLikelihood();
             likelihoodKnown = true;
         }
@@ -213,7 +179,10 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
             totalMakeDirtyCount++;
         likelihoodKnown = false;
         updateAllNodes();
-        updateAllPatterns();
+    }
+    
+    public boolean isLikelihoodKnown() {
+    	return likelihoodKnown;
     }
 
     protected abstract double calculateLogLikelihood();
@@ -225,7 +194,7 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
              rtnValue += " total operations = " + totalOperationCount +
                          " matrix updates = " + totalMatrixUpdateCount + " model changes = " + totalModelChangedCount +
                          " make dirties = " + totalMakeDirtyCount +
-                         " calculate likelihoods = " + totalcalculateLikelihoodCount +
+                         " calculate likelihoods = " + totalCalculateLikelihoodCount +
                          " get likelihoods = " + totalGetLogLikelihoodCount +
                          " all rate updates = " + totalRateUpdateAllCount +
                          " partial rate updates = " + totalRateUpdateSingleCount;
@@ -244,38 +213,11 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
      */
     protected TreeModel treeModel = null;
 
-    /**
-     * the patternList
-     */
-    protected PatternList patternList = null;
-
-    protected DataType dataType = null;
-
-    /**
-     * the pattern weights
-     */
-    protected double[] patternWeights;
-
-    /**
-     * the number of patterns
-     */
-    protected int patternCount;
-
-    /**
-     * the number of states in the data
-     */
-    protected int stateCount;
-
     /**
      * the number of nodes in the tree
      */
     protected int nodeCount;
 
-    /**
-     * Flags to specify which patterns are to be updated
-     */
-    protected boolean[] updatePattern = null;
-
     /**
      * Flags to specify which nodes are to be updated
      */
@@ -293,7 +235,7 @@ public abstract class AbstractTreeLikelihood extends AbstractModelLikelihood imp
     protected int totalGetLogLikelihoodCount = 0;
     protected int totalModelChangedCount = 0;
     protected int totalMakeDirtyCount = 0;
-    protected int totalcalculateLikelihoodCount = 0;
+    protected int totalCalculateLikelihoodCount = 0;
     protected int totalRateUpdateAllCount = 0;
     protected int totalRateUpdateSingleCount = 0;
 
