@@ -1,7 +1,7 @@
 /*
  * BeagleTreeLikelihood.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -53,6 +53,7 @@ import dr.evomodel.tree.TreeModel;
 import dr.evomodel.treelikelihood.TipStatesModel;
 import dr.inference.model.Model;
 import dr.inference.model.Parameter;
+import dr.inference.model.ThreadAwareLikelihood;
 import dr.math.MathUtils;
 
 import java.util.*;
@@ -68,7 +69,7 @@ import java.util.logging.Logger;
  */
 
 @SuppressWarnings("serial")
-public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
+public class BeagleTreeLikelihood extends AbstractSinglePartitionTreeLikelihood implements ThreadAwareLikelihood {
 
     // This property is a comma-delimited list of resource numbers (0 == CPU) to
     // allocate each BEAGLE instance to. If less than the number of instances then
@@ -79,6 +80,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     private static final String SCALING_PROPERTY = "beagle.scaling";
     private static final String RESCALE_FREQUENCY_PROPERTY = "beagle.rescale";
     private static final String EXTRA_BUFFER_COUNT_PROPERTY = "beagle.extra.buffer.count";
+    private static final String FORCE_VECTORIZATION = "beagle.force.vectorization";
 
     // Which scheme to use if choice not specified (or 'default' is selected):
     private static final PartialsRescalingScheme DEFAULT_RESCALING_SCHEME = PartialsRescalingScheme.DYNAMIC;
@@ -238,7 +240,15 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                     preferenceFlags |= BeagleFlag.PROCESSOR_CPU.getMask();
             }
 
-            if (BeagleFlag.VECTOR_SSE.isSet(preferenceFlags) && stateCount != 4) {
+            boolean forceVectorization = false;
+            String vectorizationString = System.getProperty(FORCE_VECTORIZATION);
+            if (vectorizationString != null) {
+                forceVectorization = true;
+            }
+
+            if (BeagleFlag.VECTOR_SSE.isSet(preferenceFlags) && (stateCount != 4)
+                    && !forceVectorization
+                    ) {
                 // @todo SSE doesn't seem to work for larger state spaces so for now we override the
                 // SSE option.
                 preferenceFlags &= ~BeagleFlag.VECTOR_SSE.getMask();
@@ -428,21 +438,21 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         }
         return order;
     }
-    
+
     public TipStatesModel getTipStatesModel() {
-    	return tipStatesModel;
+        return tipStatesModel;
     }
-    
+
     public PatternList getPatternsList() {
-    	return patternList;
+        return patternList;
     }
 
     public TreeModel getTreeModel() {
         return treeModel;
     }
-    
+
     public BranchModel getBranchModel() {
-    	return branchModel;
+        return branchModel;
     }
 
     public SiteRateModel getSiteRateModel() {
@@ -452,17 +462,17 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     public BranchRateModel getBranchRateModel() {
         return branchRateModel;
     }
-    
+
     public PartialsRescalingScheme getRescalingScheme() {
-    	return rescalingScheme;
+        return rescalingScheme;
     }
-    
+
     public Map<Set<String>, Parameter> getPartialsRestrictions() {
-    	return partialsRestrictions;
+        return partialsRestrictions;
     }
-    
+
     public boolean useAmbiguities() {
-    	return useAmbiguities;
+        return useAmbiguities;
     }
 
     protected int getScaleBufferCount() {
@@ -562,7 +572,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     }
 
 
-//    public void setStates(int tipIndex, int[] states) {
+    //    public void setStates(int tipIndex, int[] states) {
 //        System.err.println("BTL:setStates");
 //        beagle.setTipStates(tipIndex, states);
 //        makeDirty();
@@ -573,6 +583,12 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 //        beagle.getTipStates(tipIndex, states);
 //    }
 
+    public final void setPatternWeights(double[] patternWeights) {
+        this.patternWeights = patternWeights;
+        beagle.setPatternWeights(patternWeights);
+    }
+
+
     // **************************************************************
     // ModelListener IMPLEMENTATION
     // **************************************************************
@@ -640,8 +656,11 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
                 for (int i = 0; i < treeModel.getNodeCount(); i++)
                     if (treeModel.getNodeTaxon(treeModel.getNode(i)) != null && treeModel.getNodeTaxon(treeModel.getNode(i)).getId().equalsIgnoreCase(((Taxon) object).getId()))
                         updateNode(treeModel.getNode(i));
-            } else
+            } else if (object instanceof Parameter) {
+                // ignore...
+            } else {
                 updateAllNodes();
+            }
         } else {
 
             throw new RuntimeException("Unknown componentChangedEvent");
@@ -930,6 +949,10 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         beagle.getPartials(partialBufferHelper.getOffsetIndex(number), cumulativeBufferIndex, partials);
     }
 
+    public boolean arePartialsRescaled() {
+        return useScaleFactors;
+    }
+
     protected void setPartials(int number, double[] partials) {
         beagle.setPartials(partialBufferHelper.getOffsetIndex(number), partials);
     }
@@ -1005,8 +1028,11 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
         // First update the transition probability matrix(ices) for this branch
         if (parent != null && updateNode[nodeNum]) {
 
-            final double branchRate = branchRateModel.getBranchRate(tree, node);
+            final double branchRate;
 
+            synchronized (branchRateModel) {
+                branchRate = branchRateModel.getBranchRate(tree, node);
+            }
             final double parentHeight = tree.getNodeHeight(parent);
             final double nodeHeight = tree.getNodeHeight(node);
 
@@ -1222,7 +1248,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
     private boolean ascertainedSitePatterns = false;
 
 
-    /***
+    /**
      * Flag to specify if ambiguity codes are in use
      */
     protected final boolean useAmbiguities;
@@ -1245,8 +1271,8 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             TreeModel treeModel = new TreeModel(tree);
 
             // create Frequency Model
-            Parameter freqs = new Parameter.Default(new double[] { 0.25, 0.25,
-                    0.25, 0.25 });
+            Parameter freqs = new Parameter.Default(new double[]{0.25, 0.25,
+                    0.25, 0.25});
             FrequencyModel freqModel = new FrequencyModel(Nucleotides.INSTANCE,
                     freqs);
 
@@ -1266,7 +1292,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             List<FrequencyModel> freqModels = new ArrayList<FrequencyModel>();
             freqModels.add(freqModel);
 
-            Parameter epochTimes = new Parameter.Default(1,20);
+            Parameter epochTimes = new Parameter.Default(1, 20);
 
             // create branch rate model
             Parameter rate = new Parameter.Default(1, 0.001);
@@ -1282,7 +1308,7 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
 
             // create partition
             Partition partition1 = new Partition(treeModel, //
-            		homogenousBranchSubstitutionModel,//
+                    homogenousBranchSubstitutionModel,//
                     siteRateModel, //
                     branchRateModel, //
                     freqModel, //
@@ -1296,14 +1322,14 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             // feed to sequence simulator and generate data
             BeagleSequenceSimulator simulator = new BeagleSequenceSimulator(partitionsList
 //            		, sequenceLength
-            		);
-            Alignment alignment = simulator.simulate(false);
+            );
+            Alignment alignment = simulator.simulate(false, false);
 
             BeagleTreeLikelihood nbtl = new BeagleTreeLikelihood(alignment, treeModel, homogeneousBranchModel, siteRateModel, branchRateModel, null, false, PartialsRescalingScheme.DEFAULT);
 
             System.out.println("nBTL(homogeneous) = " + nbtl.getLogLikelihood());
 
-             nbtl = new BeagleTreeLikelihood(alignment, treeModel, epochBranchModel, siteRateModel, branchRateModel, null, false, PartialsRescalingScheme.DEFAULT);
+            nbtl = new BeagleTreeLikelihood(alignment, treeModel, epochBranchModel, siteRateModel, branchRateModel, null, false, PartialsRescalingScheme.DEFAULT);
 
             System.out.println("nBTL(epoch) = " + nbtl.getLogLikelihood());
 
@@ -1312,20 +1338,30 @@ public class BeagleTreeLikelihood extends AbstractTreeLikelihood {
             System.exit(-1);
         } // END: try-catch block
     }
-    
-	public Double getUpdateTimer() {
-		return Double.valueOf(substitutionModelDelegate.updateTime);
-	}
-    
-	public Double getConvolveTimer() {
-		return Double.valueOf(substitutionModelDelegate.convolveTime);
-	}
 
-	public double[] getSiteLogLikelihoods() {
-		getLogLikelihood();
-		double[] siteLogLikelihoods = new double[patternCount];
-		beagle.getSiteLogLikelihoods(siteLogLikelihoods);
-		return siteLogLikelihoods;
-	}
-	
+    public Double getUpdateTimer() {
+        return Double.valueOf(substitutionModelDelegate.updateTime);
+    }
+
+    public Double getConvolveTimer() {
+        return Double.valueOf(substitutionModelDelegate.convolveTime);
+    }
+
+    public void getLogScalingFactors(int nodeIndex, double[] buffer) {
+        if (nodeIndex < tipCount) {
+            Arrays.fill(buffer, 0.0);
+        } else {
+//            final int scaleIndex = scaleBufferHelper.getOffsetIndex(nodeIndex - tipCount);
+            final int scaleIndex = scaleBufferIndices[nodeIndex - tipCount];
+            beagle.getLogScaleFactors(scaleIndex, buffer);
+        }
+    }
+
+    public double[] getSiteLogLikelihoods() {
+        getLogLikelihood();
+        double[] siteLogLikelihoods = new double[patternCount];
+        beagle.getSiteLogLikelihoods(siteLogLikelihoods);
+        return siteLogLikelihoods;
+    }
+
 }//END: class
\ No newline at end of file
