@@ -1,7 +1,7 @@
 /*
  * BufferIndexHelper.java
  *
- * Copyright (C) 2002-2012 Alexei Drummond, Andrew Rambaut & Marc A. Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,7 +25,9 @@
 
 package dr.app.beagle.evomodel.treelikelihood;
 
-public class BufferIndexHelper {
+import java.io.Serializable;
+
+public class BufferIndexHelper implements Serializable {
 	
     /**
      * @param maxIndexValue the number of possible input values for the index
