@@ -1,7 +1,7 @@
 /*
  * BeastMain.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,7 +25,6 @@
 
 package dr.app.beast;
 
-import beagle.BeagleFactory;
 import beagle.BeagleFlag;
 import beagle.BeagleInfo;
 import dr.app.plugin.Plugin;
@@ -60,8 +59,8 @@ public class BeastMain {
     static class BeastConsoleApp extends jam.console.ConsoleApplication {
         XMLParser parser = null;
 
-        public BeastConsoleApp(String nameString, String aboutString, javax.swing.Icon icon) throws IOException {
-            super(nameString, aboutString, icon, false);
+        public BeastConsoleApp(String nameString, String titleString, String aboutString, javax.swing.Icon icon) throws IOException {
+            super(nameString, titleString, aboutString, icon, false);
             getDefaultFrame().setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
         }
 
@@ -111,19 +110,19 @@ public class BeastMain {
             });
             logger.addHandler(messageHandler);
 
-//            // Add a handler to handle warnings and errors. This is a ConsoleHandler
-//            // so the messages will go to StdErr..
-//            handler = new ConsoleHandler();
-//            handler.setFilter(new Filter() {
-//                public boolean isLoggable(LogRecord record) {
-//                    if (verbose) {
-//                        return record.getLevel().intValue() >= Level.WARNING.intValue();
-//                    } else {
-//                        return record.getLevel().intValue() >= Level.SEVERE.intValue();
-//                    }
-//                }
-//            });
-//            logger.addHandler(handler);
+            // Add a handler to handle warnings and errors. This is a ConsoleHandler
+            // so the messages will go to StdErr..
+            Handler errorHandler = new ConsoleHandler();
+            errorHandler.setFilter(new Filter() {
+                public boolean isLoggable(LogRecord record) {
+                    if (verbose) {
+                        return record.getLevel().intValue() >= Level.WARNING.intValue();
+                    } else {
+                        return record.getLevel().intValue() >= Level.SEVERE.intValue();
+                    }
+                }
+            });
+            infoLogger.addHandler(errorHandler);
 
             logger.setUseParentHandlers(false);
 
@@ -133,10 +132,10 @@ public class BeastMain {
             // This is a special logger that is for logging numerical and statistical errors
             // during the MCMC run. It will tolerate up to maxErrorCount before throwing a
             // RuntimeException to shut down the run.
-            //Logger errorLogger = Logger.getLogger("error");
+            Logger errorLogger = Logger.getLogger("error");
             messageHandler = new ErrorLogHandler(maxErrorCount);
             messageHandler.setLevel(Level.WARNING);
-            logger.addHandler(messageHandler);
+            errorLogger.addHandler(messageHandler);
 
             for (String pluginName : PluginLoader.getAvailablePlugins()) {
                 Plugin plugin = PluginLoader.loadPlugin(pluginName);
@@ -181,7 +180,7 @@ public class BeastMain {
                     // turn off all messages for subsequent reads of the file (they will be the same as the
                     // first time).
                     messageHandler.setLevel(Level.OFF);
-                    parser = new BeastParser(new String[]{fileName}, null, verbose, parserWarning, strictXML);
+                    parser = new BeastParser(new String[]{fileName}, additionalParsers, verbose, parserWarning, strictXML);
 
                     chains[i] = (MCMC) parser.parse(fileReader, MCMC.class);
                     if (chains[i] == null) {
@@ -216,6 +215,7 @@ public class BeastMain {
                     dome.getMessage());
             throw new RuntimeException("Terminate");
         } catch (dr.xml.XMLParseException pxe) {
+            pxe.printStackTrace(System.err);
             if (pxe.getMessage() != null && pxe.getMessage().equals("Unknown root document element, beauti")) {
                 infoLogger.severe("Error running file: " + fileName);
                 infoLogger.severe(
@@ -232,8 +232,9 @@ public class BeastMain {
             }
             throw new RuntimeException("Terminate");
         } catch (RuntimeException rex) {
+
             if (rex.getMessage() != null && rex.getMessage().startsWith("The initial posterior is zero")) {
-                infoLogger.warning("Error running file: " + fileName);
+                infoLogger.severe("Error running file: " + fileName);
                 infoLogger.severe(
                         "The initial model is invalid because state has a zero probability.\n\n" +
                                 "If the log likelihood of the tree is -Inf, his may be because the\n" +
@@ -253,6 +254,7 @@ public class BeastMain {
                 infoLogger.warning("Error running file: " + fileName);
                 System.err.println("Fatal exception: " + rex.getMessage());
                 rex.printStackTrace(System.err);
+                System.err.flush();
             }
             throw new RuntimeException("Terminate");
         } catch (Exception ex) {
@@ -260,6 +262,7 @@ public class BeastMain {
             infoLogger.severe("Fatal exception: " + ex.getMessage());
             System.err.println("Fatal exception: " + ex.getMessage());
             ex.printStackTrace(System.err);
+            System.err.flush();
             throw new RuntimeException("Terminate");
         }
     }
@@ -322,17 +325,20 @@ public class BeastMain {
                         new Arguments.Option("java", "Use Java only, no native implementations"),
                         new Arguments.RealOption("threshold", 0.0, Double.MAX_VALUE, "Full evaluation test threshold (default 1E-6)"),
 
-                        new Arguments.Option("beagle", "Use beagle library if available"),
+                        new Arguments.Option("beagle_off", "Don't use the BEAGLE library"),
+                        new Arguments.Option("beagle", "Use BEAGLE library if available (default on)"),
                         new Arguments.Option("beagle_info", "BEAGLE: show information on available resources"),
                         new Arguments.StringOption("beagle_order", "order", "BEAGLE: set order of resource use"),
                         new Arguments.IntegerOption("beagle_instances", "BEAGLE: divide site patterns amongst instances"),
                         new Arguments.Option("beagle_CPU", "BEAGLE: use CPU instance"),
                         new Arguments.Option("beagle_GPU", "BEAGLE: use GPU instance if available"),
                         new Arguments.Option("beagle_SSE", "BEAGLE: use SSE extensions if available"),
+                        new Arguments.Option("beagle_SSE_off", "BEAGLE: turn off use of SSE extensions"),
                         new Arguments.Option("beagle_cuda", "BEAGLE: use CUDA parallization if available"),
                         new Arguments.Option("beagle_opencl", "BEAGLE: use OpenCL parallization if available"),
                         new Arguments.Option("beagle_single", "BEAGLE: use single precision if available"),
                         new Arguments.Option("beagle_double", "BEAGLE: use double precision if available"),
+                        new Arguments.Option("beagle_async", "BEAGLE: use asynchronous kernels if available"),
                         new Arguments.StringOption("beagle_scaling", new String[]{"default", "dynamic", "delayed", "always", "none"},
                                 false, "BEAGLE: specify scaling scheme to use"),
                         new Arguments.IntegerOption("beagle_rescale", "BEAGLE: frequency of rescaling (dynamic scaling only)"),
@@ -447,19 +453,7 @@ public class BeastMain {
         boolean beagleShowInfo = arguments.hasOption("beagle_info");
 
         // if any beagle flag is specified then use beagle...
-        boolean useBeagle = arguments.hasOption("beagle") ||
-                arguments.hasOption("beagle_CPU") ||
-                arguments.hasOption("beagle_GPU") ||
-                arguments.hasOption("beagle_SSE") ||
-                arguments.hasOption("beagle_cuda") ||
-                arguments.hasOption("beagle_opencl") ||
-                arguments.hasOption("beagle_double") ||
-                arguments.hasOption("beagle_single") ||
-                arguments.hasOption("beagle_order") ||
-                arguments.hasOption("beagle_scaling") ||
-                arguments.hasOption("beagle_rescale") ||
-                arguments.hasOption("beagle_instances") ||
-                beagleShowInfo;
+        boolean useBeagle = !arguments.hasOption("beagle_off");
 
         if (arguments.hasOption("beagle_CPU")) {
             beagleFlags |= BeagleFlag.PROCESSOR_CPU.getMask();
@@ -473,8 +467,7 @@ public class BeastMain {
         if (arguments.hasOption("beagle_opencl")) {
             beagleFlags |= BeagleFlag.FRAMEWORK_OPENCL.getMask();
         }
-        if (arguments.hasOption("beagle_SSE")) {
-            beagleFlags |= BeagleFlag.PROCESSOR_CPU.getMask();
+        if (!arguments.hasOption("beagle_SSE_off")) {
             beagleFlags |= BeagleFlag.VECTOR_SSE.getMask();
         }
         if (arguments.hasOption("beagle_double")) {
@@ -483,6 +476,9 @@ public class BeastMain {
         if (arguments.hasOption("beagle_single")) {
             beagleFlags |= BeagleFlag.PRECISION_SINGLE.getMask();
         }
+        if (arguments.hasOption("beagle_async")) {
+            beagleFlags |= BeagleFlag.COMPUTATION_ASYNCH.getMask();
+        }
 
         if (arguments.hasOption("beagle_order")) {
             System.setProperty("beagle.resource.order", arguments.getStringOption("beagle_order"));
@@ -557,13 +553,22 @@ public class BeastMain {
 
             javax.swing.Icon icon = IconUtils.getIcon(BeastMain.class, "images/beast.png");
 
-            String aboutString = "<html><div style=\"font-family:sans-serif;\"><center>" +
-                    "<div style=\"font-size:12;\"><p>Bayesian Evolutionary Analysis Sampling Trees<br>" +
-                    "Version " + version.getVersionString() + ", " + version.getDateString() + "</p>" +
+            String titleString = "<html>" +
+                    "<div style=\"font: HelveticaNeue, Helvetica, Arial, sans-serif\">" +
+                    "<p style=\"font-weight: 100; font-size: 42px\">BEAST</p>" +
+                    "<p style=\"font-weight: 200; font-size: 12px\">Bayesian Evolutionary Analysis Sampling Trees</p>" +
+                    "<p style=\"font-weight: 300; font-size: 11px\">Version " + version.getVersionString() + ", " + version.getDateString() + "</p>" +
+                    "</div></html>";
+
+            String aboutString = "<html>" +
+                    "<div style=\"font-family:HelveticaNeue-Light, 'Helvetica Neue Light', Helvetica, Arial, 'Lucida Grande',sans-serif; font-weight: 100\">" +
+                    "<center>" +
                     version.getHTMLCredits() +
                     "</div></center></div></html>";
 
-            consoleApp = new BeastConsoleApp(nameString, aboutString, icon);
+            consoleApp = new BeastConsoleApp(nameString, titleString, aboutString, icon);
+            consoleApp.initialize();
+
         }
 
         printTitle();
@@ -572,8 +577,12 @@ public class BeastMain {
 
         if (options && !beagleShowInfo) {
 
-            String titleString = "<html><center><p>Bayesian Evolutionary Analysis Sampling Trees<br>" +
-                    "Version " + version.getVersionString() + ", " + version.getDateString() + "</p></center></html>";
+            String titleString = "<html>" +
+                    "<div style=\"font: HelveticaNeue, Helvetica, Arial, sans-serif\">" +
+                    "<p style=\"font-weight: 100; font-size: 42px\">BEAST</p>" +
+                    "<p style=\"font-weight: 200; font-size: 12px\">Bayesian Evolutionary Analysis Sampling Trees</p>" +
+                    "<p style=\"font-weight: 300; font-size: 11px\">Version " + version.getVersionString() + ", " + version.getDateString() + "</p>" +
+                    "</div></html>";
             javax.swing.Icon icon = IconUtils.getIcon(BeastMain.class, "images/beast.png");
 
             BeastDialog dialog = new BeastDialog(new JFrame(), titleString, icon);
@@ -638,7 +647,7 @@ public class BeastMain {
 
             if (BeagleInfo.getVersion().startsWith("1.")) {
                 System.err.println("WARNING: You are currenly using BEAGLE v1.x. For best performance and compatibility\n" +
-                        "with models in BEAST, please upgrade to BEAGLE v2.0 at http://beagle-lib.googlecode.com/\n");
+                        "with models in BEAST, please upgrade to BEAGLE v2.x at http://beagle-lib.googlecode.com/\n");
             }
         }
 
@@ -718,11 +727,14 @@ public class BeastMain {
         try {
             new BeastMain(inputFile, consoleApp, maxErrorCount, verbose, parserWarning, strictXML, additionalParsers, useMC3, chainTemperatures, swapChainsEvery);
         } catch (RuntimeException rte) {
+            rte.printStackTrace(System.err);
             if (window) {
                 System.out.println();
                 System.out.println("BEAST has terminated with an error. Please select QUIT from the menu.");
                 // logger.severe will throw a RTE but we want to keep the console visible
             } else {
+                System.out.flush();
+                System.err.flush();
                 System.exit(1);
             }
         }
