@@ -16,6 +16,7 @@ import dr.app.beauti.components.continuous.ContinuousComponentFactory;
 import dr.app.beauti.components.discrete.DiscreteTraitsComponentFactory;
 import dr.app.beauti.components.dollo.DolloComponentFactory;
 import dr.app.beauti.components.hpm.HierarchicalModelComponentFactory;
+import dr.app.beauti.components.linkedparameters.LinkedParameterComponentFactory;
 import dr.app.beauti.components.marginalLikelihoodEstimation.MarginalLikelihoodEstimationComponentFactory;
 import dr.app.beauti.components.sequenceerror.SequenceErrorModelComponentFactory;
 import dr.app.beauti.components.tipdatesampling.TipDateSamplingComponentFactory;
@@ -28,7 +29,7 @@ import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.PartitionTreePrior;
 import dr.app.beauti.options.STARBEASTOptions;
 import dr.app.beauti.options.TraitData;
-import dr.app.beauti.priorsPanel.DefaultPriorDialog;
+import dr.app.beauti.priorsPanel.DefaultPriorTableDialog;
 import dr.app.beauti.priorsPanel.PriorsPanel;
 import dr.app.beauti.siteModelsPanel.SiteModelsPanel;
 import dr.app.beauti.taxonsetspanel.SpeciesSetPanel;
@@ -131,6 +132,7 @@ public class BeautiFrame extends DocumentFrame {
                 DiscreteTraitsComponentFactory.INSTANCE,
 //                DnDsComponentFactory.INSTANCE,
                 DolloComponentFactory.INSTANCE,
+                LinkedParameterComponentFactory.INSTANCE,
                 HierarchicalModelComponentFactory.INSTANCE,
                 MarginalLikelihoodEstimationComponentFactory.INSTANCE,
                 SequenceErrorModelComponentFactory.INSTANCE,
@@ -312,8 +314,9 @@ public class BeautiFrame extends DocumentFrame {
             mcmcPanel.setOptions(options);
 
             setStatusMessage();
-        } catch (IllegalArgumentException illegEx) {
-            JOptionPane.showMessageDialog(this, illegEx.getMessage(),
+        } catch (IllegalArgumentException iae) {
+            iae.printStackTrace(System.err);
+            JOptionPane.showMessageDialog(this, iae.getMessage(),
                     "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
         }
 
@@ -346,8 +349,9 @@ public class BeautiFrame extends DocumentFrame {
             priorsPanel.getOptions(options);
             operatorsPanel.getOptions(options);
             mcmcPanel.getOptions(options);
-        } catch (IllegalArgumentException illegEx) {
-            JOptionPane.showMessageDialog(this, illegEx.getMessage(),
+        } catch (IllegalArgumentException iae) {
+            iae.printStackTrace(System.err);
+            JOptionPane.showMessageDialog(this, iae.getMessage(),
                     "Illegal Argument Exception", JOptionPane.ERROR_MESSAGE);
         }
     }
@@ -445,6 +449,7 @@ public class BeautiFrame extends DocumentFrame {
         }
         fileIn.close();
 
+        options.registerComponents(components);
         generator = new BeastGenerator(options, components);
 
         return true;
@@ -547,6 +552,7 @@ public class BeautiFrame extends DocumentFrame {
                             JOptionPane.ERROR_MESSAGE);
                     return false;
                 } catch (Exception ex) {
+                    ex.printStackTrace(System.err);
                     JOptionPane.showMessageDialog(this, "Fatal exception: " + ex,
                             "Error reading file",
                             JOptionPane.ERROR_MESSAGE);
@@ -702,6 +708,7 @@ public class BeautiFrame extends DocumentFrame {
         try {
             generator.checkOptions();
         } catch (Generator.GeneratorException ge) {
+            ge.printStackTrace(System.err);
             JOptionPane.showMessageDialog(this, ge.getMessage(), "Invalid BEAUti setting : ",
                     JOptionPane.ERROR_MESSAGE);
             if (ge.getSwitchToPanel() != null) {
@@ -710,7 +717,7 @@ public class BeautiFrame extends DocumentFrame {
             return false;
         }
 
-        DefaultPriorDialog defaultPriorDialog = new DefaultPriorDialog(this);
+        DefaultPriorTableDialog defaultPriorDialog = new DefaultPriorTableDialog(this);
         if (!defaultPriorDialog.showDialog(options)) {
             return false;
         }
@@ -723,14 +730,17 @@ public class BeautiFrame extends DocumentFrame {
                 generator.generateXML(file);
 
             } catch (IOException ioe) {
+                ioe.printStackTrace(System.err);
                 JOptionPane.showMessageDialog(this, "Unable to generate file due to I/O issue: " + ioe.getMessage(),
                         "Unable to generate file", JOptionPane.ERROR_MESSAGE);
                 return false;
             } catch (Generator.GeneratorException e) {
+                e.printStackTrace(System.err);
                 JOptionPane.showMessageDialog(this, "The BEAST XML is incomplete because :\n" + e.getMessage(),
                         "The BEAST XML is incomplete", JOptionPane.ERROR_MESSAGE);
                 return false;
             } catch (Exception e) {
+                e.printStackTrace(System.err);
                 JOptionPane.showMessageDialog(this, "Unable to generate file: " + e.getMessage(),
                         "Unable to generate file", JOptionPane.ERROR_MESSAGE);
                 return false;
