@@ -1,7 +1,7 @@
 /*
- * ClockModelPanel.java
+ * OldClockModelsPanel.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -314,6 +314,7 @@ public class OldClockModelsPanel extends BeautiPanel implements Exportable {
                 OldClockType.UNCORRELATED_EXPONENTIAL,
 //                OldClockType.AUTOCORRELATED
                 OldClockType.RANDOM_LOCAL_CLOCK,
+                OldClockType.FIXED_LOCAL_CLOCK
         })));
 
         col = clockModelTable.getColumnModel().getColumn(4);
