@@ -1,18 +1,38 @@
+/*
+ * AncestralStatesComponentGenerator.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beauti.components.ancestralstates;
 
 import dr.app.beauti.generator.BaseComponentGenerator;
-import dr.app.beauti.generator.Generator;
 import dr.app.beauti.options.AbstractPartitionData;
 import dr.app.beauti.options.BeautiOptions;
-import dr.app.beauti.options.PartitionData;
 import dr.app.beauti.options.PartitionSubstitutionModel;
+import dr.app.beauti.options.PartitionTreeModel;
 import dr.app.beauti.util.XMLWriter;
-import dr.evolution.datatype.ContinuousDataType;
 import dr.evolution.datatype.DataType;
-import dr.evolution.datatype.GeneralDataType;
-import dr.evolution.datatype.Nucleotides;
 import dr.evomodelxml.treelikelihood.AncestralStateTreeLikelihoodParser;
-import dr.evomodelxml.treelikelihood.TreeLikelihoodParser;
 import dr.util.Attribute;
 
 /**
@@ -86,13 +106,13 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
                 writeCodonPartitionedRobustCounting(writer, component);
                 break;
             case IN_TREE_LIKELIHOOD:
-                writeCountingParameter(writer, (AbstractPartitionData)item, prefix);
+                writeCountingParameter(writer, (AbstractPartitionData) item, prefix);
                 break;
             case IN_FILE_LOG_PARAMETERS:
                 writeLogs(writer, component);
                 break;
             case IN_TREES_LOG:
-                writeTreeLogs(writer, component);
+                writeTreeLogs(writer, component, (PartitionTreeModel)item);
                 break;
             case AFTER_TREES_LOG:
                 writeAncestralStateLoggers(writer, component);
@@ -117,7 +137,6 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         StringBuilder matrix = new StringBuilder();
 
-
         DataType dataType = partition.getDataType();
         int stateCount = dataType.getStateCount();
 
@@ -135,10 +154,11 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
                 }
             }
         }
+
         writer.writeTag("parameter",
-                new Attribute[] {
+                new Attribute[]{
                         new Attribute.Default<String>("id", prefix + "count"),
-                        new Attribute.Default<String>("value", matrix.toString()) },
+                        new Attribute.Default<String>("value", matrix.toString())},
                 true);
 
     }
@@ -174,13 +194,14 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         // S operator
         writer.writeOpenTag("codonPartitionedRobustCounting",
-                new Attribute[] {
+                new Attribute[]{
                         new Attribute.Default<String>("id", prefix + "robustCounting1"),
                         new Attribute.Default<String>("labeling", "S"),
+                        new Attribute.Default<String>("prefix", prefix),
                         new Attribute.Default<String>("useUniformization",
                                 "true"),
                         new Attribute.Default<String>("unconditionedPerBranch",
-                                "true") });
+                                "true")});
 
         writer.writeIDref("treeModel", "treeModel");
         writer.writeOpenTag("firstPosition");
@@ -201,13 +222,14 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         // N operator:
         writer.writeOpenTag("codonPartitionedRobustCounting",
-                new Attribute[] {
+                new Attribute[]{
                         new Attribute.Default<String>("id", prefix + "robustCounting2"),
                         new Attribute.Default<String>("labeling", "N"),
+                        new Attribute.Default<String>("prefix", prefix),
                         new Attribute.Default<String>("useUniformization",
                                 "true"),
                         new Attribute.Default<String>("unconditionedPerBranch",
-                                "true") });
+                                "true")});
 
         writer.writeIDref("treeModel", "treeModel");
         writer.writeOpenTag("firstPosition");
@@ -241,57 +263,58 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
         }
     }
 
-    private void writeTreeLogs(XMLWriter writer, AncestralStatesComponentOptions component) {
+    private void writeTreeLogs(XMLWriter writer, AncestralStatesComponentOptions component, PartitionTreeModel treeModel) {
 
         for (AbstractPartitionData partition : options.getDataPartitions()) {
 
-            if (component.dNdSRobustCounting(partition)) {
-                String prefix = partition.getName() + ".";
+            if (partition.getPartitionTreeModel() == treeModel) {
+                if (component.dNdSRobustCounting(partition)) {
+                    String prefix = partition.getName() + ".";
 
-                writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting1");
-                writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting2");
-            }
+                    writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting1");
+                    writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting2");
+                }
 
-            if (component.reconstructAtNodes(partition)) {
+                if (component.reconstructAtNodes(partition)) {
 
-                // is an alignment data partition
-                PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
-                int cpCount = partition.getPartitionSubstitutionModel().getCodonPartitionCount();
+                    // is an alignment data partition
+                    PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
+                    int cpCount = partition.getPartitionSubstitutionModel().getCodonPartitionCount();
 
-                if (cpCount > 1) {
-                    for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
-                        String prefix = partition.getPrefix() + substModel.getPrefix(i);
-                        String name = partition.getName() + "." + substModel.getPrefix(i);
-                        if (name.endsWith(".")) {
-                            name = name.substring(0, name.length() - 1);
+                    if (cpCount > 1) {
+                        for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
+                            String prefix = partition.getPrefix() + substModel.getPrefixCodon(i);
+                            String name = partition.getName() + "." + substModel.getPrefixCodon(i);
+                            if (name.endsWith(".")) {
+                                name = name.substring(0, name.length() - 1);
+                            }
+                            writeTrait(writer, partition, prefix, AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG, name);
                         }
-                        writeTrait(writer, partition, prefix, AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG, name);
+                    } else {
+                        writeTrait(writer, partition, partition.getPrefix(), AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG, partition.getName());
                     }
-                } else {
-                    writeTrait(writer, partition, partition.getPrefix(), AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG, partition.getName());
-
-                }
-            }
-
-            if (component.isCountingStates(partition)) {
-                if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
-                    throw new RuntimeException("Can't do counting on Continuous data partition");
                 }
 
-                PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
-                int cpCount = partition.getPartitionSubstitutionModel().getCodonPartitionCount();
+                if (component.isCountingStates(partition)) {
+                    if (partition.getDataType().getType() == DataType.CONTINUOUS) {
+                        throw new RuntimeException("Can't do counting on Continuous data partition");
+                    }
 
-                if (cpCount > 1) {
-                    for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
-                        String prefix = partition.getPrefix() + substModel.getPrefix(i);
-                        String name = partition.getName() + "." + substModel.getPrefix(i) + "count";
-                        writeTrait(writer, partition, prefix, "count", name);
+                    PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
+                    int cpCount = partition.getPartitionSubstitutionModel().getCodonPartitionCount();
+
+                    if (cpCount > 1) {
+                        for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
+                            String prefix = partition.getPrefix() + substModel.getPrefixCodon(i);
+                            String name = partition.getName() + "." + substModel.getPrefixCodon(i) + "count";
+                            writeTrait(writer, partition, prefix, "count", name);
+                        }
+                    } else {
+                        writeTrait(writer, partition, partition.getPrefix(), "count", partition.getName() + ".count");
+
                     }
-                } else {
-                    writeTrait(writer, partition, partition.getPrefix(), "count", partition.getName() + ".count");
 
                 }
-
             }
         }
     }
@@ -299,17 +322,17 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
     private void writeTrait(XMLWriter writer, AbstractPartitionData partition, String prefix, String tag, String name) {
         String traitName = prefix + tag;
 
-        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS) {
             traitName = partition.getName();
         }
 
         writer.writeOpenTag("trait",
-                new Attribute[] {
+                new Attribute[]{
                         new Attribute.Default<String>("name", traitName),
                         new Attribute.Default<String>("tag", name)
                 }
         );
-        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS) {
             writer.writeIDref("multivariateTraitLikelihood", prefix + "traitLikelihood");
         } else {
             writer.writeIDref("ancestralTreeLikelihood", prefix + "treeLikelihood");
@@ -335,18 +358,18 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         writer.writeComment("Ancestral state reconstruction for: " + partition.getName());
 
-        writer.writeOpenTag("log", new Attribute[] {
+        writer.writeOpenTag("log", new Attribute[]{
                 new Attribute.Default<String>("id", "fileLog_" + partition.getName()),
                 new Attribute.Default<String>("logEvery", Integer.toString(options.logEvery)),
-                new Attribute.Default<String>("fileName", partition.getName() + STATE_LOG_SUFFIX) });
+                new Attribute.Default<String>("fileName",  options.fileNameStem + "." + partition.getName() + STATE_LOG_SUFFIX)});
 
         PartitionSubstitutionModel substModel = partition.getPartitionSubstitutionModel();
         int cpCount = partition.getPartitionSubstitutionModel().getCodonPartitionCount();
 
         if (cpCount > 1) {
             for (int i = 1; i <= substModel.getCodonPartitionCount(); i++) {
-                String prefix = partition.getPrefix() + substModel.getPrefix(i);
-                String name = partition.getName() + "." + substModel.getPrefix(i);
+                String prefix = partition.getPrefix() + substModel.getPrefixCodon(i);
+                String name = partition.getName() + "." + substModel.getPrefixCodon(i);
                 if (name.endsWith(".")) {
                     name = name.substring(0, name.length() - 1);
                 }
@@ -362,7 +385,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
     private void writeAncestralTrait(XMLWriter writer, AbstractPartitionData partition, String mrcaId, String prefix, String nameString) {
         String traitName = prefix + AncestralStateTreeLikelihoodParser.RECONSTRUCTION_TAG;
-        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS) {
             traitName = partition.getName();
         }
 
@@ -374,7 +397,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
         );
         writer.writeIDref("treeModel", partition.getPartitionTreeModel().getPrefix() + "treeModel");
 
-        if (partition.getDataType().getType() == DataType.CONTINUOUS)  {
+        if (partition.getDataType().getType() == DataType.CONTINUOUS) {
             writer.writeIDref("multivariateTraitLikelihood", prefix + "traitLikelihood");
         } else {
             writer.writeIDref("ancestralTreeLikelihood", prefix + "treeLikelihood");
@@ -394,13 +417,13 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         writer.writeComment("Robust counting for: " + partition.getName());
 
-        writer.writeOpenTag("log", new Attribute[] {
-                new Attribute.Default<String>("id", "fileLog_dNdS"),
+        writer.writeOpenTag("log", new Attribute[]{
+                new Attribute.Default<String>("id", "fileLog_dNdS_" + partition.getName()),
                 new Attribute.Default<String>("logEvery", Integer.toString(options.logEvery)),
-                new Attribute.Default<String>("fileName", partition.getName() + DNDS_LOG_SUFFIX) });
+                new Attribute.Default<String>("fileName", options.fileNameStem + "." + partition.getName() + DNDS_LOG_SUFFIX)});
 
         writer.writeOpenTag("dNdSLogger", new Attribute[]{new Attribute.Default<String>("id",
-                "dNdS")});
+                partition.getName() + ".dNdS")});
         writer.writeIDref("treeModel", "treeModel");
         writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting1");
         writer.writeIDref("codonPartitionedRobustCounting", prefix + "robustCounting2");
@@ -428,7 +451,7 @@ public class AncestralStatesComponentGenerator extends BaseComponentGenerator {
 
         writer.writeOpenTag("report");
 
-        writer.write("<dNdSPerSiteAnalysis fileName=" + '\"' + partition.getName() + DNDS_LOG_SUFFIX + '\"' + "/> \n");
+        writer.write("<dNdSPerSiteAnalysis fileName=\"" + options.fileNameStem + "." + partition.getName() + DNDS_LOG_SUFFIX + "\"/> \n");
 
         writer.writeCloseTag("report");
 
