@@ -7,14 +7,19 @@ import dr.app.beauti.options.ComponentOptions;
 
 /**
  * @author Andrew Rambaut
- * @version $Id: ContinuousComponentFactory.java 5861 2013-10-03 13:38:22Z rambaut $
+ * @version $Id$
  */
 
 public class ContinuousComponentFactory implements ComponentFactory {
 
 	public static ComponentFactory INSTANCE = new ContinuousComponentFactory();
 
-	public ComponentGenerator createGenerator(BeautiOptions beautiOptions) {
+    @Override
+    public Class getOptionsClass() {
+        return ContinuousComponentOptions.class;
+    }
+
+    public ComponentGenerator createGenerator(BeautiOptions beautiOptions) {
 		return new ContinuousComponentGenerator(beautiOptions);
 	}
 
