@@ -6,6 +6,7 @@ import dr.app.beauti.types.OperatorType;
 import dr.app.beauti.util.XMLWriter;
 import dr.evolution.datatype.ContinuousDataType;
 import dr.evolution.util.Taxon;
+import dr.evomodel.continuous.ContinuousDiffusionStatistic;
 import dr.evomodelxml.tree.TreeLoggerParser;
 import dr.util.Attribute;
 import dr.xml.AttributeParser;
@@ -347,7 +348,9 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         ContinuousComponentOptions component = (ContinuousComponentOptions) options
                 .getComponentOptions(ContinuousComponentOptions.class);
 
-        if (component.useLambda(partitionData.getPartitionSubstitutionModel())) {
+        PartitionSubstitutionModel model = partitionData.getPartitionSubstitutionModel();
+
+        if (component.useLambda(model)) {
             writer.writeOpenTag("transformedTreeModel");
             writer.writeIDref("treeModel", treeModelId);
             writer.writeTag("parameter", new Attribute[] {
@@ -365,6 +368,19 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         writer.writeTag("parameter", new Attribute.Default<String>("id", "leaf." + partitionData.getName()), true);
         writer.writeCloseTag("traitParameter");
 
+       if (model.getJitterWindow() > 0.0) {
+            StringBuilder sb = new StringBuilder(Double.toString(model.getJitterWindow()));
+            for (int i = 1; i < model.getContinuousTraitCount(); i++) {
+                sb.append(" ").append(Double.toString(model.getJitterWindow()));
+            }
+            writer.writeOpenTag("jitter", new Attribute[] {
+                    new Attribute.Default<String>("window", sb.toString()),
+                    new Attribute.Default<String>("duplicatesOnly", "true")
+            });
+            writer.writeTag("parameter", new Attribute.Default<String>("idref", "leaf." + partitionData.getName()), true);
+            writer.writeCloseTag("jitter");
+        }
+
         writer.writeOpenTag("conjugateRootPrior");
 
         writer.writeOpenTag("meanParameter");
@@ -452,7 +468,7 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
         writer.writeIDref("matrixParameter", precisionMatrixId);
         writer.writeCloseTag("matrixInverse");
 
-        writer.writeOpenTag("diffusionRateStatistic", (partitionData.getPartitionSubstitutionModel().isLatitudeLongitude() ?
+        writer.writeOpenTag(ContinuousDiffusionStatistic.CONTINUOUS_DIFFUSION_STATISTIC, (partitionData.getPartitionSubstitutionModel().isLatitudeLongitude() ?
                 new Attribute[] {
                         new Attribute.Default<String>("id", prefix + "diffusionRate"),
                         new Attribute.Default<String>("greatCircleDistance", "true")
@@ -461,7 +477,7 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
                         new Attribute.Default<String>("id", prefix + "diffusionRate"),
                 }));
         writer.writeIDref("multivariateTraitLikelihood", traitLikelihoodId);
-        writer.writeCloseTag("diffusionRateStatistic");
+        writer.writeCloseTag(ContinuousDiffusionStatistic.CONTINUOUS_DIFFUSION_STATISTIC);
     }
 
     private void writePrecisionGibbsOperators(XMLWriter writer,
@@ -499,7 +515,7 @@ public class ContinuousComponentGenerator extends BaseComponentGenerator {
 //            writer.writeIDref("productStatistic", prefix + "treeLengthPrecision2");
             }
             writer.writeIDref("matrixInverse", prefix + "varCovar");
-            writer.writeIDref("diffusionRateStatistic", prefix + "diffusionRate");
+            writer.writeIDref(ContinuousDiffusionStatistic.CONTINUOUS_DIFFUSION_STATISTIC, prefix + "diffusionRate");
 
             if (component.useLambda(model)) {
                 writer.writeIDref("parameter", model.getName() + "." + ContinuousComponentOptions.LAMBDA);
