@@ -1,7 +1,7 @@
 /*
  * DiscreteTraitsComponentOptions.java
  *
- * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -63,8 +63,8 @@ public class DiscreteTraitsComponentOptions implements ComponentOptions {
                 modelOptions.createDiscreteStatistic(prefix + "nonZeroRates", "the number of non-zero rates for BSSVS");
 
                 modelOptions.createScaleOperator(prefix + "frequencies", 0.75, 1.0);
-                modelOptions.createOperator(prefix + "rates", OperatorType.SCALE_INDEPENDENTLY, 0.75, 1.0);
-                modelOptions.createOperator(prefix + "indicators", OperatorType.BITFLIP, -1.0, 1.0);
+                modelOptions.createOperator(prefix + "rates", OperatorType.SCALE_INDEPENDENTLY, 0.75, 15.0);
+                modelOptions.createOperator(prefix + "indicators", OperatorType.BITFLIP, -1.0, 7.0);
 //                modelOptions.createScaleOperator(prefix + "mu", demoTuning, 10);
 
                 modelOptions.createZeroOneParameterUniformPrior(prefix + "root.frequencies", "discrete state root frequencies", 0.25);
@@ -76,26 +76,26 @@ public class DiscreteTraitsComponentOptions implements ComponentOptions {
                 modelOptions.createOperatorUsing2Parameters(RateBitExchangeOperator.OPERATOR_NAME,
                         "(indicators, rates)",
                         "rateBitExchangeOperator (If both BSSVS and asymmetric subst selected)",
-                        prefix + "indicators", prefix + "rates", OperatorType.RATE_BIT_EXCHANGE, -1.0, 6.0);
+                        prefix + "indicators", prefix + "rates", OperatorType.RATE_BIT_EXCHANGE, -1.0, 7.0);
 
             }
         }
     }
 
     public void selectParameters(final ModelOptions modelOptions, final List<Parameter> params) {
-        for (AbstractPartitionData partitionData : options.getDataPartitions(GeneralDataType.INSTANCE)) {
-            String prefix = partitionData.getName() + ".";
+        for (PartitionSubstitutionModel substitutionModel : options.getPartitionSubstitutionModels(GeneralDataType.INSTANCE)) {
+            String prefix = substitutionModel.getName() + ".";
 
-            if (partitionData.getPartitionSubstitutionModel().isActivateBSSVS()) {
+            if (substitutionModel.isActivateBSSVS()) {
                 modelOptions.getParameter(prefix + "indicators");
                 Parameter nonZeroRates = modelOptions.getParameter(prefix + "nonZeroRates");
 
-                Set<String> states = partitionData.getPartitionSubstitutionModel().getDiscreteStateSet();
+                Set<String> states = substitutionModel.getDiscreteStateSet();
                 int K = states.size();
-                if (partitionData.getPartitionSubstitutionModel().getDiscreteSubstType() == DiscreteSubstModelType.SYM_SUBST) {
+                if (substitutionModel.getDiscreteSubstType() == DiscreteSubstModelType.SYM_SUBST) {
                     nonZeroRates.mean = Math.log(2); // mean = 0.693 and offset = K-1
                     nonZeroRates.offset = K - 1;
-                } else if (partitionData.getPartitionSubstitutionModel().getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
+                } else if (substitutionModel.getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
                     nonZeroRates.mean = K - 1; // mean = K-1 and offset = 0
                     nonZeroRates.offset = 0.0;
                 } else {
@@ -107,8 +107,12 @@ public class DiscreteTraitsComponentOptions implements ComponentOptions {
 
             params.add(modelOptions.getParameter(prefix + "frequencies"));
             params.add(modelOptions.getParameter(prefix + "rates"));
+        }
 
-            if (partitionData.getPartitionSubstitutionModel().getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
+        for (AbstractPartitionData partition : options.getDataPartitions(GeneralDataType.INSTANCE)) {
+            String prefix = partition.getPrefix();
+
+            if (partition.getPartitionSubstitutionModel().getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
                 params.add(modelOptions.getParameter(prefix + "root.frequencies"));
             }
 
@@ -121,18 +125,19 @@ public class DiscreteTraitsComponentOptions implements ComponentOptions {
     }
 
     public void selectOperators(final ModelOptions modelOptions, final List<Operator> ops) {
-        for (AbstractPartitionData partitionData : options.getDataPartitions(GeneralDataType.INSTANCE)) {
-            String prefix = partitionData.getName() + ".";
+        for (PartitionSubstitutionModel substitutionModel : options.getPartitionSubstitutionModels(GeneralDataType.INSTANCE)) {
+            String prefix = substitutionModel.getName() + ".";
 
 //            ops.add(modelOptions.getOperator(prefix + "frequencies")); // Usually fixed
             ops.add(modelOptions.getOperator(prefix + "rates"));
 
-            if (partitionData.getPartitionSubstitutionModel().isActivateBSSVS()) {
+            if (substitutionModel.isActivateBSSVS()) {
                 ops.add(modelOptions.getOperator(prefix + "indicators"));
             }
-
+        }
+        for (AbstractPartitionData partitionData : options.getDataPartitions(GeneralDataType.INSTANCE)) {
             if (partitionData.getPartitionSubstitutionModel().getDiscreteSubstType() == DiscreteSubstModelType.ASYM_SUBST) {
-                ops.add(modelOptions.getOperator(prefix + "root.frequencies"));
+                ops.add(modelOptions.getOperator(partitionData.getName() + ".root.frequencies"));
             }
         }
     }
