@@ -7,14 +7,19 @@ import dr.app.beauti.options.ComponentOptions;
 
 /**
  * @author Marc Suchard
- * @version $Id: DolloComponentFactory.java 5861 2013-10-03 13:38:22Z rambaut $
+ * @version $Id$
  */
 
 public class DolloComponentFactory implements ComponentFactory {
 
 	public static ComponentFactory INSTANCE = new DolloComponentFactory();
 
-	public ComponentGenerator createGenerator(BeautiOptions beautiOptions) {
+    @Override
+    public Class getOptionsClass() {
+        return DolloComponentOptions.class;
+    }
+
+    public ComponentGenerator createGenerator(BeautiOptions beautiOptions) {
         return new DolloComponentGenerator(beautiOptions);
 	}
 
