@@ -1,7 +1,7 @@
 /*
  * BeastGenerator.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -155,9 +155,6 @@ public class BeastGenerator extends Generator {
                     }
                 }
             }
-        } catch (Exception e) {
-            throw new GeneratorException(e.getMessage());
-        }
 
         //++++++++++++++++ Taxon List ++++++++++++++++++
         TaxonList taxonList = options.taxonList;
@@ -332,10 +329,11 @@ public class BeastGenerator extends Generator {
             }
         }
 
+        //++++++++++++++++ Improper priors ++++++++++++++++++
         MarginalLikelihoodEstimationOptions mleOptions = (MarginalLikelihoodEstimationOptions)options.getComponentOptions(MarginalLikelihoodEstimationOptions.class);
         if (mleOptions.performMLE) {
             for (Parameter param : options.selectParameters()) {
-                    if (param.isPriorImproper() || param.priorType == PriorType.ONE_OVER_X_PRIOR) {
+                    if (param.isPriorImproper() || (param.priorType == PriorType.ONE_OVER_X_PRIOR && !param.getBaseName().contains("popSize"))) {
                         throw new GeneratorException("Parameter \"" + param.getName() + "\":" +
                                 "\nhas an improper prior and will not sample correctly when estimating " +
                                 "the marginal likelihood. " +
@@ -348,6 +346,11 @@ public class BeastGenerator extends Generator {
         // Speciation model with dated tips
         // Sampling rates without dated tips or priors on rate or nodes
 
+        } catch (Exception e) {
+            // catch any other exceptions here and rethrow to generate messages
+            throw new GeneratorException(e.getMessage());
+        }
+
     }
 
     /**
@@ -408,7 +411,7 @@ public class BeastGenerator extends Generator {
                 }
             }
         } catch (Exception e) {
-            System.err.println(e);
+            e.printStackTrace(System.err);
             throw new GeneratorException("Taxon list generation has failed:\n" + e.getMessage());
         }
 
@@ -568,7 +571,7 @@ public class BeastGenerator extends Generator {
             }
         } catch (Exception e) {
             e.printStackTrace();
-            throw new GeneratorException("Branch rates model generation is failed:\n" + e.getMessage());
+            throw new GeneratorException("Branch rates model generation has failed:\n" + e.getMessage());
         }
 
         //++++++++++++++++ Substitution Model & Site Model ++++++++++++++++++
@@ -677,6 +680,8 @@ public class BeastGenerator extends Generator {
 
         //++++++++++++++++ Operators ++++++++++++++++++
         try {
+            generateInsertionPoint(ComponentGenerator.InsertionPoint.BEFORE_OPERATORS, writer);
+
             List<Operator> operators = options.selectOperators();
             operatorsGenerator.writeOperatorSchedule(operators, writer);
             writer.writeText("");
@@ -826,7 +831,7 @@ public class BeastGenerator extends Generator {
     public void writeDifferentTaxa(AbstractPartitionData dataPartition, XMLWriter writer) {
         TaxonList taxonList = dataPartition.getTaxonList();
 
-        String name = dataPartition.getName();
+        String name = dataPartition.getPartitionTreeModel().getName();
 
         writer.writeComment("gene name = " + name + ", ntax= " + taxonList.getTaxonCount());
         writer.writeOpenTag(TaxaParser.TAXA, new Attribute[]{new Attribute.Default<String>(XMLParser.ID, name + "." + TaxaParser.TAXA)});
