@@ -1,3 +1,28 @@
+/*
+ * Generator.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.beauti.generator;
 
 import dr.app.beauti.components.ComponentFactory;
@@ -185,6 +210,31 @@ public abstract class Generator {
         }
     }
 
+    /**
+     * write a parameter
+     *
+     * @param parameterId     the parameter name/id
+     * @param parameterColumn the parameter column from which the samples are taken
+     * @param fileName        the file from which the samples are taken
+     * @param burnin          the number of samples to be discarded
+     * @param writer          the writer
+     */
+    public void writeParameter(String parameterId, String parameterColumn, String fileName, int burnin, XMLWriter writer) {
+        ArrayList<Attribute.Default> attributes = new ArrayList<Attribute.Default>();
+        attributes.add(new Attribute.Default<String>(XMLParser.ID, parameterId));
+
+        attributes.add(new Attribute.Default<String>("parameterColumn", parameterColumn));
+        attributes.add(new Attribute.Default<String>("fileName", fileName));
+        attributes.add(new Attribute.Default<String>("burnin", "" + burnin));
+
+        Attribute[] attrArray = new Attribute[attributes.size()];
+        for (int i = 0; i < attrArray.length; i++) {
+            attrArray[i] = attributes.get(i);
+        }
+
+        writer.writeTag(ParameterParser.PARAMETER, attrArray, true);
+    }
+
     public void writeParameter(String id, Parameter parameter, XMLWriter writer) {
         if (parameter.isFixed) {
             writeParameter(id, 1, parameter.initial, Double.NaN, Double.NaN, writer);
