@@ -1,7 +1,7 @@
 /*
  * LogGenerator.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -36,6 +36,7 @@ import dr.evolution.util.Taxa;
 import dr.evomodel.branchratemodel.BranchRateModel;
 import dr.evomodel.tree.TreeModel;
 import dr.evomodelxml.branchratemodel.DiscretizedBranchRatesParser;
+import dr.evomodelxml.branchratemodel.LocalClockModelParser;
 import dr.evomodelxml.branchratemodel.RandomLocalClockModelParser;
 import dr.evomodelxml.branchratemodel.StrictClockBranchRatesParser;
 import dr.evomodelxml.clock.ACLikelihoodParser;
@@ -313,6 +314,25 @@ public class LogGenerator extends Generator {
             branchRatesModelGenerator.writeLogStatistic(model, writer);
         }
 
+        if (options.logCoalescentEventsStatistic) {
+            writer.writeOpenTag("coalescentEventsStatistic");
+            // coalescentLikelihood
+            for (PartitionTreeModel model : options.getPartitionTreeModels()) {
+                PartitionTreePrior prior = model.getPartitionTreePrior();
+                treePriorGenerator.writePriorLikelihoodReferenceLog(prior, model, writer);
+                writer.writeText("");
+            }
+
+            /*for (PartitionTreePrior prior : options.getPartitionTreePriors()) {
+                if (prior.getNodeHeightPrior() == TreePriorType.EXTENDED_SKYLINE) {
+                    writer.writeIDref(CoalescentLikelihoodParser.COALESCENT_LIKELIHOOD, prior.getPrefix() + COALESCENT); // only 1 coalescent
+                } else if (prior.getNodeHeightPrior() == TreePriorType.SKYGRID) {
+                    writer.writeIDref(GMRFSkyrideLikelihoodParser.SKYGRID_LIKELIHOOD, prior.getPrefix() + "skygrid");
+                }
+            }*/
+            writer.writeCloseTag("coalescentEventsStatistic");
+        }
+
         generateInsertionPoint(ComponentGenerator.InsertionPoint.IN_FILE_LOG_PARAMETERS, writer);
 
         treeLikelihoodGenerator.writeTreeLikelihoodReferences(writer);
@@ -582,6 +602,10 @@ public class LogGenerator extends Generator {
                         id = model.getPrefix() + BranchRateModel.BRANCH_RATES;
                         break;
 
+                    case FIXED_LOCAL_CLOCK:
+                        tag = LocalClockModelParser.LOCAL_CLOCK_MODEL;
+                        id = model.getPrefix() + BranchRateModel.BRANCH_RATES;
+                        break;
                     case AUTOCORRELATED:
                         tag = ACLikelihoodParser.AC_LIKELIHOOD;
                         id =  options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES;
@@ -622,6 +646,12 @@ public class LogGenerator extends Generator {
                             BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
                     break;
 
+                case FIXED_LOCAL_CLOCK:
+                    writeTreeTrait(writer, LocalClockModelParser.LOCAL_CLOCK_MODEL,
+                            options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES,
+                            BranchRateModel.RATE, model.getPrefix() + BranchRateModel.RATE);
+                    break;
+
                 case AUTOCORRELATED:
                     writer.writeIDref(ACLikelihoodParser.AC_LIKELIHOOD,
                             options.noDuplicatedPrefix(model.getPrefix(), tree.getPrefix()) + BranchRateModel.BRANCH_RATES);
