@@ -1,7 +1,7 @@
 /*
  * BeautiOptions.java
  *
- * Copyright (c) 2002-2011 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -63,12 +63,26 @@ public class BeautiOptions extends ModelOptions {
     public BeautiOptions(ComponentFactory[] components) {
 
         // Install all the component's options from the given list of factories:
-        for (ComponentFactory component : components) {
-            addComponent(component.createOptions(this));
-        }
+        registerComponents(components);
     }
 
     /**
+     * This will register the list of components if not already there...
+     * @param components
+     */
+    public void registerComponents(ComponentFactory[] components) {
+
+        // Install all the component's options from the given list of factories:
+        for (ComponentFactory component : components) {
+            if (!hasComponent(component)) {
+                addComponent(component.createOptions(this));
+            }
+        }
+    }
+
+
+    /**
+     *
      * resets the options to the initial conditions
      */
     public void reset() {
@@ -333,9 +347,32 @@ public class BeautiOptions extends ModelOptions {
 
         selectComponentOperators(this, ops);
 
+        // no remove operators for parameters that are part of a joint prior...
+        List<Operator> toRemove = new ArrayList<Operator>();
+        for (Operator operator : ops) {
+            if ((operator.parameter1 != null && operator.parameter1.isLinked) ||
+                    (operator.parameter2 != null && operator.parameter2.isLinked)) {
+                toRemove.add(operator);
+            }
+        }
+
+        ops.removeAll(toRemove);
+
         return ops;
     }
 
+    public Operator getOperator(Parameter parameter) {
+        for (Operator operator : selectOperators()) {
+            if (operator.parameter1 == parameter || operator.parameter2 == parameter) {
+                return operator;
+            }
+        }
+        return null;
+    }
+
+
+
+
     public boolean hasData() {
         return dataPartitions.size() > 0;
     }
@@ -971,14 +1008,18 @@ public class BeautiOptions extends ModelOptions {
                 taxa = partition.getTaxonList();
             } else {
                 final TaxonList taxa1 = partition.getTaxonList();
-                if (taxa1.getTaxonCount() != taxa.getTaxonCount()) {
-                    return false;
-                }
-                for (int k = 0; k < taxa1.getTaxonCount(); ++k) {
-                    if (taxa.getTaxonIndex(taxa1.getTaxonId(k)) == -1) {
+                if (taxa1 != null) {
+                    // Trait partitions are assumed to have the same taxon list as the
+                    // tree they are using and so will return null.
+                    if (taxa1.getTaxonCount() != taxa.getTaxonCount()) {
+                        return false;
+                    }
+                    for (int k = 0; k < taxa1.getTaxonCount(); ++k) {
+                        if (taxa.getTaxonIndex(taxa1.getTaxonId(k)) == -1) {
 //                for (Taxon taxon : taxa1) {
 //                    if (taxa.getTaxonIndex(taxon) == -1) { // this is wrong code
-                        return false;
+                            return false;
+                        }
                     }
                 }
             }
@@ -1010,18 +1051,20 @@ public class BeautiOptions extends ModelOptions {
      * @return  num of taxon
      */
     public int getTaxonCount(List<AbstractPartitionData> partitionDataList) {
-        if (partitionDataList == null) return 0;
+        if (partitionDataList == null || partitionDataList.size() == 0) return 0;
 
         List<String> taxonNameList = new ArrayList<String>();
         for (AbstractPartitionData partition : partitionDataList) {
-            for (Taxon t : partition.getTaxonList()) {
-                if (!taxonNameList.contains(t.getId())) {
-                    if (partition instanceof PartitionPattern) {
-                        Patterns patterns = ((PartitionPattern) partition).getPatterns();
-                        if (!patterns.isMasked(patterns.getTaxonIndex(t)))
+            if (partition.getTaxonList() != null) { // not a trait partition
+                for (Taxon t : partition.getTaxonList()) {
+                    if (!taxonNameList.contains(t.getId())) {
+                        if (partition instanceof PartitionPattern) {
+                            Patterns patterns = ((PartitionPattern) partition).getPatterns();
+                            if (!patterns.isMasked(patterns.getTaxonIndex(t)))
+                                taxonNameList.add(t.getId());
+                        } else {
                             taxonNameList.add(t.getId());
-                    } else {
-                        taxonNameList.add(t.getId());
+                        }
                     }
                 }
             }
@@ -1123,19 +1166,30 @@ public class BeautiOptions extends ModelOptions {
 //            getPartitionTreeModels().get(0).addPartitionData(newTrait);
         }
 
-        ContinuousComponentOptions comp = (ContinuousComponentOptions) getComponentOptions(ContinuousComponentOptions.class);
-        comp.createParameters(this);
-
-        DiscreteTraitsComponentOptions comp2 = (DiscreteTraitsComponentOptions) getComponentOptions(DiscreteTraitsComponentOptions.class);
-        comp2.createParameters(this);
-
-        AncestralStatesComponentOptions comp3 = (AncestralStatesComponentOptions) getComponentOptions(AncestralStatesComponentOptions.class);
-        comp3.setReconstructAtNodes(partition, true);
-        comp3.setReconstructAtMRCA(partition, false);
+        updateTraitParameters(partition);
 
         return selRow; // only for trait panel
     }
 
+    private void updateTraitParameters(AbstractPartitionData partition) {
+        if (partition.isCreatedFromTrait()) {
+            ContinuousComponentOptions comp = (ContinuousComponentOptions) getComponentOptions(ContinuousComponentOptions.class);
+            comp.createParameters(this);
+
+            DiscreteTraitsComponentOptions comp2 = (DiscreteTraitsComponentOptions) getComponentOptions(DiscreteTraitsComponentOptions.class);
+            comp2.createParameters(this);
+
+            AncestralStatesComponentOptions comp3 = (AncestralStatesComponentOptions) getComponentOptions(AncestralStatesComponentOptions.class);
+            comp3.setReconstructAtNodes(partition, true);
+            comp3.setReconstructAtMRCA(partition, false);
+        }
+    }
+
+    public void renamePartition(AbstractPartitionData partition, String newName) {
+        partition.setName(newName);
+        updateTraitParameters(partition);
+    }
+
     public void removeTrait(String traitName) {
         if (traitExists(traitName)) {
             clearTraitValues(traitName); // Clear trait values
@@ -1342,7 +1396,7 @@ public class BeautiOptions extends ModelOptions {
 
     public boolean unlinkPartitionRates = true;
 
-    public Units.Type units = Units.Type.SUBSTITUTIONS;
+    public Units.Type units = Units.Type.YEARS;
 
     // Operator schedule options
     public int coolingSchedule = OperatorSchedule.DEFAULT_SCHEDULE;
@@ -1389,4 +1443,6 @@ public class BeautiOptions extends ModelOptions {
 
     public boolean shareMicroSat = true;
 
+    public boolean logCoalescentEventsStatistic = false;
+
 }
\ No newline at end of file
