@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * ClockModelOptions.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -10,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -477,21 +479,25 @@ public class ClockModelOptions extends ModelOptions {
 
                 for (AbstractPartitionData partition : partitions) {
                     if (partition.getDistances() != null) {   // ignore partitions that don't have distances
-                    Tree tree = new UPGMATree(partition.getDistances());
+                        Tree tree = new UPGMATree(partition.getDistances());
 
-                    Set<String> leafNodes = Taxa.Utils.getTaxonListIdSet(taxa);
+                        Set<String> leafNodes = Taxa.Utils.getTaxonListIdSet(taxa);
 
-                    if (leafNodes.size() < 1) {
-                        return -1;
+                        if (leafNodes.size() < 1) {
+                            return -1;
+                        }
+
+                        NodeRef node = Tree.Utils.getCommonAncestorNode(tree, leafNodes);
+
+                        if (node == null) {
+                            throw new IllegalArgumentException("Can't find MRCA node for taxon set, " + taxa.getId() + ", in partition: " + partition.getName());
+                        }
+
+                        calibrationDistance += tree.getNodeHeight(node);
+                        rootDistance += tree.getNodeHeight(tree.getRoot());
+
+                        siteCount += partition.getSiteCount();
                     }
-
-                    NodeRef node = Tree.Utils.getCommonAncestorNode(tree, leafNodes);
-
-                    calibrationDistance += tree.getNodeHeight(node);
-                    rootDistance += tree.getNodeHeight(tree.getRoot());
-
-                    siteCount += partition.getSiteCount();
-                }
                 }
 
                 rootDistance /= partitions.size();
