@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Parameter.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -10,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -45,7 +47,7 @@ public class Parameter implements Serializable {
     private boolean meanInRealSpace = false;
 
     // Required para
-    private final String baseName;
+    private String baseName;
     private final String description;
 
     // final Builder para
@@ -85,7 +87,8 @@ public class Parameter implements Serializable {
     public double uniformUpper;
     public double uniformLower;
 
-    public String hpmModelName;
+    public boolean isLinked;
+    public String linkedName;
 
     public static class Builder {
         // Required para
@@ -136,6 +139,37 @@ public class Parameter implements Serializable {
             this.description = description;
         }
 
+        public Builder duplicate(Parameter source) {
+            scaleType = source.scaleType;
+            taxaId = source.taxaId;
+            isNodeHeight = source.isNodeHeight;
+            isDiscrete = source.isDiscrete;
+            isHierarchical = source.isHierarchical;
+            isCMTCRate = source.isCMTCRate;
+            isNonNegative = source.isNonNegative;
+            isZeroOne = source.isZeroOne;
+            isStatistic = source.isStatistic;
+            isCached = source.isCached;
+            options = source.options;
+            priorType = source.priorType;
+            isPriorFixed = source.isPriorFixed;
+            initial = source.initial;
+            isTruncated = source.isTruncated;
+            truncationUpper = source.truncationUpper;
+            truncationLower = source.truncationLower;
+            mean = source.mean;
+            stdev = source.stdev;
+            shape = source.shape;
+            shapeB = source.shapeB;
+            scale = source.scale;
+            offset = source.offset;
+            precision = source.precision;
+            uniformUpper = source.uniformUpper;
+            uniformLower = source.uniformLower;
+            isFixed = source.isFixed;
+            return this;
+        }
+
         public Builder scaleType(PriorScaleType scaleType) {
             this.scaleType = scaleType;
             return this;
@@ -348,6 +382,11 @@ public class Parameter implements Serializable {
         }
     }
 
+    public void setName(String name) {
+        this.baseName = name;
+    }
+
+
     public String getXMLName() { // only for BeautiTemplate
         if (taxaId != null) {
             return "tmrca_" + getFullName();
@@ -404,7 +443,7 @@ public class Parameter implements Serializable {
             // might be appropriate:
             /* priorType == PriorType.ONE_OVER_X_PRIOR || */
                 (priorType == PriorType.NONE_IMPROPER) ||
-                (priorType == PriorType.UNIFORM_PRIOR && (Double.isInfinite(getLowerBound()) || Double.isInfinite(getUpperBound())))) {
+                        (priorType == PriorType.UNIFORM_PRIOR && (Double.isInfinite(getLowerBound()) || Double.isInfinite(getUpperBound())))) {
             return true;
         }
         return false;
@@ -454,4 +493,8 @@ public class Parameter implements Serializable {
         this.meanInRealSpace = meanInRealSpace;
     }
 
+    @Override
+    public String toString() {
+        return getName();
+    }
 }
