@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * PartitionClockModel.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -10,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -24,6 +26,7 @@
 package dr.app.beauti.options;
 
 import dr.app.beauti.types.*;
+import dr.evolution.util.Taxa;
 
 import java.util.List;
 
@@ -86,9 +89,9 @@ public class PartitionClockModel extends PartitionOptions {
     protected void initModelParametersAndOpererators() {
         rate = 1.0;
 
-        if (DEFAULT_CMTC_RATE_REFERENCE_PRIOR) {
-            new Parameter.Builder("clock.rate", "substitution rate").
-                    prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
+        if (DEFAULT_CMTC_RATE_REFERENCE_PRIOR || dataLength <= 10) { // TODO Discuss threshold
+            new Parameter.Builder("clock.rate", "substitution rate")
+                    .prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                     .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
             new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
@@ -98,32 +101,18 @@ public class PartitionClockModel extends PartitionOptions {
             new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
                     prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
                     .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
-
         } else {
-            if (dataLength <= 10) { // TODO Discuss threshold
-                new Parameter.Builder("clock.rate", "substitution rate").prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
+            new Parameter.Builder("clock.rate", "substitution rate").
+                    prior(PriorType.UNDEFINED).initial(rate)
+                    .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
-                new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
-                        prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
+            new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
+                    prior(PriorType.UNDEFINED).initial(rate)
+                    .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
 
-                new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
-                        prior(PriorType.CTMC_RATE_REFERENCE_PRIOR).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
-            } else {
-                new Parameter.Builder("clock.rate", "substitution rate").
-                        prior(PriorType.UNDEFINED).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
-
-                new Parameter.Builder(ClockType.UCED_MEAN, "uncorrelated exponential relaxed clock mean").
-                        prior(PriorType.UNDEFINED).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
-
-                new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
-                        prior(PriorType.UNDEFINED).initial(rate)
-                        .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
-            }
+            new Parameter.Builder(ClockType.UCLD_MEAN, "uncorrelated lognormal relaxed clock mean").
+                    prior(PriorType.UNDEFINED).initial(rate)
+                    .isCMTCRate(true).isNonNegative(true).partitionOptions(this).build(parameters);
         }
 
         new Parameter.Builder(ClockType.UCLD_STDEV, "uncorrelated lognormal relaxed clock stdev").
@@ -166,6 +155,21 @@ public class PartitionClockModel extends PartitionOptions {
                     params.add(getParameter(ClockType.LOCAL_CLOCK + ".relativeRates"));
                     break;
 
+                case FIXED_LOCAL_CLOCK:
+                    for (Taxa taxonSet : options.taxonSets) {
+                        if (options.taxonSetsMono.get(taxonSet)) {
+                            String parameterName = taxonSet.getId() + ".rate";
+                            if (!hasParameter(parameterName)) {
+                                new Parameter.Builder(parameterName, "substitution rate")
+                                        .prior(PriorType.UNDEFINED).initial(rate)
+                                        .isCMTCRate(false).isNonNegative(true).partitionOptions(this).build(parameters);
+                                createScaleOperator(parameterName, demoTuning, rateWeights);
+                            }
+                            params.add(getParameter(taxonSet.getId() + ".rate"));
+                        }
+                    }
+                    break;
+
                 case UNCORRELATED:
                     switch (clockDistributionType) {
                         case LOGNORMAL:
@@ -231,6 +235,7 @@ public class PartitionClockModel extends PartitionOptions {
         switch (clockType) {
             case STRICT_CLOCK:
             case RANDOM_LOCAL_CLOCK:
+            case FIXED_LOCAL_CLOCK:
                 rateParam = getParameter("clock.rate");
                 break;
 
@@ -284,6 +289,15 @@ public class PartitionClockModel extends PartitionOptions {
                         addRandomLocalClockOperators(ops);
                         break;
 
+                    case FIXED_LOCAL_CLOCK:
+                        ops.add(getOperator("clock.rate"));
+                        for (Taxa taxonSet : options.taxonSets) {
+                            if (options.taxonSetsMono.get(taxonSet)) {
+                                ops.add(getOperator(taxonSet.getId() + ".rate"));
+                            }
+                        }
+                        break;
+
                     case UNCORRELATED:
                         switch (clockDistributionType) {
                             case LOGNORMAL:
@@ -342,6 +356,13 @@ public class PartitionClockModel extends PartitionOptions {
                         addRandomLocalClockOperators(ops);
                         break;
 
+                    case FIXED_LOCAL_CLOCK:
+                        for (Taxa taxonSet : options.taxonSets) {
+                            if (options.taxonSetsMono.get(taxonSet)) {
+                                ops.add(getOperator(taxonSet.getId() + ".rate"));
+                            }
+                        }
+                        break;
 
                     default:
                         throw new IllegalArgumentException("Unknown clock model");
