@@ -19,6 +19,7 @@ import java.util.Set;
 
 /**
  * @author Alexei Drummond
+ * @author Andrew Rambaut
  * @author Walter Xie
  */
 abstract class PriorOptionsPanel extends OptionsPanel {
@@ -319,7 +320,7 @@ abstract class PriorOptionsPanel extends OptionsPanel {
 
     void setArguments(Parameter parameter, PriorType priorType) {
         this.isCalibratedYule = parameter.isCalibratedYule;
-        this.isInitializable = !parameter.isStatistic;
+        this.isInitializable = priorType.isInitializable;
         if (!parameter.isStatistic) {
             setFieldRange(initialField, parameter.isNonNegative, parameter.isZeroOne);
             initialField.setValue(parameter.initial);
@@ -338,7 +339,7 @@ abstract class PriorOptionsPanel extends OptionsPanel {
     }
 
     void getArguments(Parameter parameter, PriorType priorType) {
-        if (!parameter.isStatistic) {
+        if (priorType.isInitializable) {
             parameter.initial = initialField.getValue();
         }
         parameter.isTruncated = isTruncatedCheck.isSelected();
@@ -362,6 +363,26 @@ abstract class PriorOptionsPanel extends OptionsPanel {
 
     private static final String OFFSET = "Offset";
 
+    static final PriorOptionsPanel INFINITE_UNIFORM = new PriorOptionsPanel(false) {
+        void setup() {
+        }
+
+        Distribution getDistribution() {
+            return null;
+        }
+
+        void setArguments(Parameter parameter) {
+        }
+
+        void getArguments(Parameter parameter) {
+        }
+
+        @Override
+        boolean isInputValid() {
+            return getValue(0) > getValue(1);
+        }
+    };
+
     static final PriorOptionsPanel UNIFORM = new PriorOptionsPanel(false) {
         void setup() {
             addField("Upper", 1.0, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY);
