@@ -29,8 +29,11 @@ import dr.app.beauti.BeautiFrame;
 import dr.app.beauti.BeautiPanel;
 import dr.app.beauti.components.hpm.HierarchicalModelComponentOptions;
 import dr.app.beauti.components.hpm.HierarchicalPhylogeneticModel;
+import dr.app.beauti.components.linkedparameters.LinkedParameter;
+import dr.app.beauti.components.linkedparameters.LinkedParameterComponentOptions;
 import dr.app.beauti.options.BeautiOptions;
 import dr.app.beauti.options.ClockModelGroup;
+import dr.app.beauti.options.Operator;
 import dr.app.beauti.options.Parameter;
 import dr.app.beauti.types.ClockType;
 import dr.app.beauti.types.PriorType;
@@ -44,6 +47,7 @@ import javax.swing.*;
 import javax.swing.event.ListSelectionEvent;
 import javax.swing.event.ListSelectionListener;
 import javax.swing.plaf.BorderUIResource;
+import javax.swing.table.DefaultTableCellRenderer;
 import javax.swing.table.TableCellRenderer;
 import java.awt.*;
 import java.awt.event.ActionEvent;
@@ -63,6 +67,8 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
     PriorTableModel priorTableModel = null;
     JLabel messageLabel = new JLabel();
     JButton linkButton = null;
+    JButton hpmButton = null;
+    JButton unlinkButton = null;
 
     public ArrayList<Parameter> parameters = new ArrayList<Parameter>();
 
@@ -75,8 +81,6 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
 
     private final boolean isDefaultOnly;
 
-    private final static boolean HIERARCHICAL_ENABLED = true; // Change to false to disable
-
     public PriorsPanel(BeautiFrame parent, boolean isDefaultOnly) {
         this.frame = parent;
         this.isDefaultOnly = isDefaultOnly;
@@ -110,13 +114,54 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
 
         TableEditorStopper.ensureEditingStopWhenTableLosesFocus(priorTable);
 
+        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer() {
+            // override renderer preparation
+            public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
+                                                           boolean hasFocus,
+                                                           int row, int column)
+            {
+                // allow default preparation
+                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
+
+                Parameter parameter = parameters.get(row);
+
+                if (isSelected) {
+                    setForeground(parameter.isLinked ? Color.LIGHT_GRAY : SystemColor.textHighlightText);
+                } else {
+                    setForeground(parameter.isLinked ? Color.GRAY : SystemColor.controlText);
+                }
+//                if (parameter.isLinked) {
+//                    setFont(getFont().deriveFont(Font.ITALIC));
+//                }
+                return this;
+            }
+        };
+        priorTable.getColumnModel().getColumn(0).setCellRenderer(renderer);
+        priorTable.getColumnModel().getColumn(2).setCellRenderer(renderer);
+        priorTable.getColumnModel().getColumn(3).setCellRenderer(renderer);
+
         scrollPane = new JScrollPane(priorTable,
                 JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                 JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
 
         scrollPane.setOpaque(false);
 
-        Action setHierarchicalAction = new AbstractAction("Link parameters into a phylogenetic hierarchical model") {
+        Action setLinkedParametersAction = new AbstractAction("Link parameters together") {
+            public void actionPerformed(ActionEvent actionEvent) {
+                // Make list of selected parameters;
+                int[] rows = priorTable.getSelectedRows();
+                linkButtonPressed(rows);
+            }
+        };
+
+        linkButton = new JButton(setLinkedParametersAction);
+        linkButton.setVisible(true);
+        linkButton.setEnabled(false);
+        linkButton.setToolTipText("<html>Join the selected parameters so their values are<br>" +
+                "kept the same. This will create a new controlling<br>" +
+                "parameter which can be given a prior.");
+
+        Action setHierarchicalAction = new AbstractAction("Link parameters into a hierarchical model") {
             public void actionPerformed(ActionEvent actionEvent) {
                 // Make list of selected parameters;
                 int[] rows = priorTable.getSelectedRows();
@@ -124,10 +169,25 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
             }
         };
 
-        linkButton = new JButton(setHierarchicalAction);
-        linkButton.setVisible(true);
-        linkButton.setEnabled(false);
-        linkButton.setToolTipText(HierarchicalPhylogeneticModel.TIP_TOOL);
+        hpmButton = new JButton(setHierarchicalAction);
+        hpmButton.setVisible(true);
+        hpmButton.setEnabled(false);
+        hpmButton.setToolTipText(HierarchicalPhylogeneticModel.TIP_TOOL);
+
+        Action setUnlinkAction = new AbstractAction("Unlink parameters") {
+            public void actionPerformed(ActionEvent actionEvent) {
+                // Make list of selected parameters;
+                int[] rows = priorTable.getSelectedRows();
+                unlinkButtonPressed(rows);
+            }
+        };
+
+        unlinkButton = new JButton(setUnlinkAction);
+        unlinkButton.setVisible(true);
+        unlinkButton.setEnabled(false);
+        unlinkButton.setToolTipText("<html>Unlink parameters joined as a linked parameter<br>" +
+                "or hierarchical model. Will delete the enclosing<br>" +
+                "model.");
 
         messageLabel.setText(getMessage());
 
@@ -141,7 +201,7 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
 
         add(scrollPane, BorderLayout.CENTER);
 
-        if (HIERARCHICAL_ENABLED && !isDefaultOnly) {
+        if (!isDefaultOnly) {
             JPanel southPanel = new JPanel();
             southPanel.setLayout(new BorderLayout(0, 0));
             JToolBar toolBar1 = new JToolBar();
@@ -151,6 +211,15 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
 
             PanelUtils.setupComponent(linkButton);
             toolBar1.add(linkButton);
+
+            PanelUtils.setupComponent(hpmButton);
+            toolBar1.add(hpmButton);
+
+            toolBar1.addSeparator();
+
+            PanelUtils.setupComponent(unlinkButton);
+            toolBar1.add(unlinkButton);
+
             southPanel.add(toolBar1, BorderLayout.NORTH);
             southPanel.add(messageLabel, BorderLayout.SOUTH);
             add(southPanel, BorderLayout.SOUTH);
@@ -167,9 +236,21 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
     }
 
     public void selectionChanged() {
+        LinkedParameterComponentOptions comp = (LinkedParameterComponentOptions)
+                options.getComponentOptions(LinkedParameterComponentOptions.class);
+
         int[] selRows = priorTable.getSelectedRows();
-        boolean hasSelection = (selRows != null && selRows.length != 0);
-        linkButton.setEnabled(hasSelection);
+        int jointPriorCount = 0;
+        for (int i = 0; i < selRows.length; ++i) {
+            Parameter parameter = parameters.get(selRows[i]);
+            if (parameter.isLinked || comp.getLinkedParameterForArgument(parameter) != null) {
+                jointPriorCount++;
+            }
+        }
+
+        linkButton.setEnabled(selRows.length > 1);
+        hpmButton.setEnabled(selRows.length > 1);
+        unlinkButton.setEnabled(jointPriorCount > 0);
     }
 
 
@@ -249,6 +330,7 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
 
     private PriorDialog priorDialog = null;
     private HierarchicalPriorDialog hierarchicalPriorDialog = null;
+    private JointPriorDialog linkParameterDialog = null;
 
     private JButton continueButton = null;
 
@@ -259,11 +341,80 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
         }
     }
 
+    private void linkButtonPressed(int[] rows) {
+        Parameter firstParameter = parameters.get(rows[0]);
+
+        // gather the selected parameters...
+        List<Parameter> selectedParameters = new ArrayList<Parameter>();
+        boolean isLinked = false;
+        for (int i = 0; i < rows.length; ++i) {
+
+            Parameter parameter = parameters.get(rows[i]);
+            if (parameter.isStatistic) {
+                JOptionPane.showMessageDialog(frame,
+                        "Statistics cannot be linked together or\n" +
+                                "linked with parameter.",
+                        "Parameter linking error",
+                        JOptionPane.WARNING_MESSAGE);
+                return; // Bail out
+            }
+            if (parameter.truncationLower != firstParameter.truncationLower ||
+                    parameter.truncationUpper != firstParameter.truncationUpper ||
+                    options.getOperator(parameter).operatorType != options.getOperator(firstParameter).operatorType) {
+                JOptionPane.showMessageDialog(frame,
+                        "Only parameters that share the same bounds\n" +
+                                "and have the same operator types can be linked.",
+                        "Parameter linking error",
+                        JOptionPane.WARNING_MESSAGE);
+                return; // Bail out
+            }
+
+            if (parameter.isLinked) {
+                isLinked = true;
+            }
+            selectedParameters.add(parameters.get(rows[i]));
+        }
+
+        if (isLinked) {
+            int option = JOptionPane.showConfirmDialog(this,
+                    "One or more selected parameters are already linked or\n" +
+                            "part of a hierarchical model.\n\n" +
+                            "Constructing a new prior will remove these parameter\n" +
+                            "from the original model. Continue?",
+                    "Link warning",
+                    JOptionPane.YES_NO_OPTION,
+                    JOptionPane.WARNING_MESSAGE);
+            if (option == JOptionPane.NO_OPTION) {
+                return;
+            }
+        }
+
+        LinkedParameterComponentOptions comp = (LinkedParameterComponentOptions)
+                options.getComponentOptions(LinkedParameterComponentOptions.class);
+
+        LinkedParameter linkedParameter = comp.createLinkedParameter(null, selectedParameters);
+
+        editLinkedParameter(linkedParameter, selectedParameters);
+    }
+
+    private void unlinkButtonPressed(int[] rows) {
+        for (int i = 0; i < rows.length; ++i) {
+
+            Parameter parameter = parameters.get(rows[i]);
+            if (parameter.isLinked) {
+                removeFromJointPrior(parameter);
+            }
+        }
+
+        priorTableModel.fireTableDataChanged();
+        frame.setAllOptions();
+    }
+
     private void hierarchicalButtonPressed(int[] rows) {
 
         if (rows.length < 2) {
             JOptionPane.showMessageDialog(frame,
-                    "Less than two parameters selected.",
+                    "Fewer than two parameters selected.",
                     "Parameter linking error",
                     JOptionPane.WARNING_MESSAGE);
             return;
@@ -372,69 +523,138 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
     }
 
     private void priorButtonPressed(int row) {
-        Parameter param = parameters.get(row);
+        Parameter parameter = parameters.get(row);
 
-        if (HIERARCHICAL_ENABLED && hierarchicalPriorDialog != null) {
-            // Check that parameter is not already in a HPM
-            HierarchicalModelComponentOptions comp = (HierarchicalModelComponentOptions)
-                    options.getComponentOptions(HierarchicalModelComponentOptions.class);
-            if (comp.isHierarchicalParameter(param)) {
+        LinkedParameterComponentOptions lpco = (LinkedParameterComponentOptions)
+                options.getComponentOptions(LinkedParameterComponentOptions.class);
+
+        HierarchicalModelComponentOptions hmco = (HierarchicalModelComponentOptions)
+                options.getComponentOptions(HierarchicalModelComponentOptions.class);
+
+        if (lpco.isArgumentParameter(parameter)) {
+            LinkedParameter linkedParameter = lpco.getLinkedParameterForArgument(parameter);
+            editLinkedParameter(linkedParameter, linkedParameter.getDependentParameterList());
+        } else {
+            if (hmco.isHierarchicalParameter(parameter)) {
                 int option = JOptionPane.showConfirmDialog(this,
-                        "Parameter already exists in a HPM. Selecting a\n" +
-                                "new prior will remove the parameter. Continue?",
-                        "HPM warning",
+                        "This parameter forms part of an Hierarchical Model.\n" +
+                                "Selecting a new prior will remove the parameter from\n" +
+                                "this HPM.\nContinue?",
+                        "Hierarchical parameter warning",
                         JOptionPane.YES_NO_OPTION,
                         JOptionPane.WARNING_MESSAGE);
                 if (option == JOptionPane.NO_OPTION) {
                     return;
                 }
             }
+            if (parameter.isLinked) {
+                int option = JOptionPane.showConfirmDialog(this,
+                        "Parameter linked to other parameter. Selecting a\n" +
+                                "new prior will remove the link.\nContinue?",
+                        "Linked parameter warning",
+                        JOptionPane.YES_NO_OPTION,
+                        JOptionPane.WARNING_MESSAGE);
+                if (option == JOptionPane.NO_OPTION) {
+                    return;
+                }
+            }
+
+            editPrior(parameter);
+        }
+    }
+
+    private void editLinkedParameter(LinkedParameter linkedParameter, List<Parameter> selectedParameters) {
+        Parameter sourceParameter = linkedParameter.getArgumentParameter();
+        Operator sourceOperator = linkedParameter.getArgumentOperator();
+
+        List<Parameter> compatibleParameters = new ArrayList<Parameter>();
+        for (Parameter parameter : parameters) {
+            boolean isCompatible = true;
+            if (parameter.isStatistic) {
+                isCompatible = false;
+            }
+            if (parameter.truncationLower != sourceParameter.truncationLower) {
+                isCompatible = false;
+            }
+            Operator operator = options.getOperator(parameter);
+            if (operator == null || operator.operatorType != sourceOperator.operatorType) {
+                isCompatible = false;
+            }
+            if (isCompatible) {
+                compatibleParameters.add(parameter);
+            }
         }
 
-        int result;
+        if (linkParameterDialog == null) {
+            linkParameterDialog = new JointPriorDialog(frame, options);
+        }
+
+        linkParameterDialog.setLinkedParameter(linkedParameter);
+        linkParameterDialog.setCompatibleParameterList(compatibleParameters);
+        linkParameterDialog.setDependentParameterList(selectedParameters);
+
+        boolean done = false;
+
+        while (!done) {
+            int result = linkParameterDialog.showDialog();
+            if (result == JOptionPane.OK_OPTION && linkParameterDialog.validateModelName()) {
+                done = true;
+            }
+            if (result == JOptionPane.CANCEL_OPTION) {
+                return;
+            }
+        }
+
+        linkedParameter.setName(linkParameterDialog.getName());
+
+        linkedParameter.linkDependentParameters(linkParameterDialog.getDependentParameterList());
+
+        linkParameterDialog.getArguments(linkedParameter.getArgumentParameter());
+
+        for (Parameter parameter : linkParameterDialog.getDependentParameterList()) {
+            parameter.setPriorEdited(true);
+        }
+
+        priorTableModel.fireTableDataChanged();
+        frame.setAllOptions();
+    }
+
+    private void editPrior(Parameter parameter) {
         if (priorDialog == null) {
             priorDialog = new PriorDialog(frame);
         }
 
-        priorDialog.setParameter(param);
+        priorDialog.setParameter(parameter);
+
+        int result;
 
         do {
             result = priorDialog.showDialog();
         } while (result == JOptionPane.OK_OPTION && priorDialog.hasInvalidInput(true));
 
         if (result == JOptionPane.OK_OPTION) {
-            // move to individual Dialog, otherwise it will change if Cancel
-            priorDialog.getArguments(param);
             // Only do this if OK button is pressed (not cancel):
 
-            if (HIERARCHICAL_ENABLED && hierarchicalPriorDialog != null) {
-                // Possibly remove parameter from its HPM
-                HierarchicalModelComponentOptions comp = (HierarchicalModelComponentOptions)
-                        options.getComponentOptions(HierarchicalModelComponentOptions.class);
-                if (comp.isHierarchicalParameter(param)) {
-                    if (comp.removeParameter(this, param, true) == JOptionPane.NO_OPTION) {
-                        // Bail out
-                        System.err.println("Bailing out of modification");
-                        return;
-                    }
-                    System.err.println("Parameter removed from an HPM");
-                }
-            }
+            // If this was part of a linked parameter, remove it from that...
+            removeFromJointPrior(parameter);
 
-            param.setPriorEdited(true);
+            // move to individual Dialog, otherwise it will change if Cancel
+            priorDialog.getArguments(parameter);
+
+            parameter.setPriorEdited(true);
 
             if (isDefaultOnly) {
                 setParametersList(options);
             }
 
-            if (param.getBaseName().endsWith("treeModel.rootHeight") || param.taxaId != null) { // param.taxa != null is TMRCA
+            if (parameter.getBaseName().endsWith("treeModel.rootHeight") || parameter.taxaId != null) { // param.taxa != null is TMRCA
 
-                if (options.treeModelOptions.isNodeCalibrated(param)) {
+                if (options.treeModelOptions.isNodeCalibrated(parameter)) {
                     List<ClockModelGroup> groupList;
                     if (options.useStarBEAST) {
                         groupList = options.clockModelOptions.getClockModelGroups();
                     } else {
-                        groupList = options.clockModelOptions.getClockModelGroups(options.getDataPartitions(param.getOptions()));
+                        groupList = options.clockModelOptions.getClockModelGroups(options.getDataPartitions(parameter.getOptions()));
                     }
 
                     for (ClockModelGroup clockModelGroup : groupList) {
@@ -447,6 +667,29 @@ public class PriorsPanel extends BeautiPanel implements Exportable {
             }
 
             priorTableModel.fireTableDataChanged();
+            frame.setAllOptions();
+        }
+    }
+
+    private void removeFromJointPrior(Parameter parameter) {
+        LinkedParameterComponentOptions comp = (LinkedParameterComponentOptions)
+                options.getComponentOptions(LinkedParameterComponentOptions.class);
+
+        LinkedParameter linkedParameter = comp.getLinkedParameter(parameter);
+        if (linkedParameter != null) {
+            comp.removeDependentParameter(parameter);
+        }
+
+        // Possibly remove parameter from its HPM
+        HierarchicalModelComponentOptions hmco = (HierarchicalModelComponentOptions)
+                options.getComponentOptions(HierarchicalModelComponentOptions.class);
+        if (hmco.isHierarchicalParameter(parameter)) {
+            if (hmco.removeParameter(this, parameter, true) == JOptionPane.NO_OPTION) {
+                // Bail out
+                System.err.println("Bailing out of modification");
+                return;
+            }
+            System.err.println("Parameter removed from an HPM");
         }
 
     }
