@@ -1,7 +1,7 @@
 /*
  * TraitsPanel.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -207,7 +207,7 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         ActionPanel actionPanel1 = new ActionPanel(false);
         actionPanel1.setAddAction(addTraitAction);
         actionPanel1.setRemoveAction(removeTraitAction);
-//        actionPanel1.getAddActionButton().setTooltipText(ADD_TRAITS_TOOLTIP);
+        actionPanel1.setAddToolTipText(ADD_TRAITS_TOOLTIP);
 
         removeTraitAction.setEnabled(false);
 
@@ -473,10 +473,10 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
             boolean done = frame.doImportTraits();
             if (done) {
                 if (isSpeciesTrait) {
-                // check that we did indeed import a 'species' trait
+                    // check that we did indeed import a 'species' trait
                     if (!options.traitExists(TraitData.TRAIT_SPECIES)) {
                         JOptionPane.showMessageDialog(this,
-                                        "The imported trait file didn't contain a trait\n" +
+                                "The imported trait file didn't contain a trait\n" +
                                         "called '" + TraitData.TRAIT_SPECIES + "', required for *BEAST.\n" +
                                         "Please edit it or select a different file.",
                                 "Reserved trait name",
@@ -495,6 +495,47 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         return true;
     }
 
+    public void createTraitPartition() {
+        int[] selRows = traitsTable.getSelectedRows();
+        java.util.List<TraitData> traits = new ArrayList<TraitData>();
+        int discreteCount = 0;
+        int continuousCount = 0;
+        for (int row : selRows) {
+            TraitData trait = options.traits.get(row);
+            traits.add(trait);
+
+            if (trait.getTraitType() == TraitData.TraitType.DISCRETE) {
+                discreteCount ++;
+            }
+            if (trait.getTraitType() == TraitData.TraitType.CONTINUOUS) {
+                continuousCount ++;
+            }
+        }
+
+        boolean success = false;
+        if (discreteCount > 0) {
+            if (continuousCount > 0)  {
+                JOptionPane.showMessageDialog(TraitsPanel.this, "Don't mix discrete and continuous traits when creating partition(s).", "Mixed Trait Types", JOptionPane.ERROR_MESSAGE);
+                return;
+            }
+
+            // with discrete traits, create a separate partition for each
+            for (TraitData trait : traits) {
+                java.util.List<TraitData> singleTrait = new ArrayList<TraitData>();
+                singleTrait.add(trait);
+                if (dataPanel.createFromTraits(singleTrait)) {
+                    success = true;
+                }
+            }
+        } else {
+            // with
+            success = dataPanel.createFromTraits(traits);
+        }
+        if (success) {
+            frame.switchToPanel(BeautiFrame.DATA_PARTITIONS);
+        }
+    }
+
     public void addTrait(TraitData newTrait) {
         int selRow = options.addTrait(newTrait);
         traitsTable.getSelectionModel().setSelectionInterval(selRow, selRow);
@@ -527,7 +568,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         public ClearTraitAction() {
             super("Clear trait values");
-            setToolTipText("Use this tool to remove trait values from each taxon");
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -540,7 +580,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         public GuessTraitsAction() {
             super("Guess trait values");
-            setToolTipText("Use this tool to guess the trait values from the taxon labels");
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -552,7 +591,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
 
         public AddTraitAction() {
             super("Add trait");
-            setToolTipText("Use this button to add a new trait");
         }
 
         public void actionPerformed(ActionEvent ae) {
@@ -578,6 +616,15 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         }
     }
 
+    public class CreateTraitPartitionAction extends AbstractAction {
+        public CreateTraitPartitionAction() {
+            super("Create partition from trait ...");
+        }
+
+        public void actionPerformed(ActionEvent ae) {
+            createTraitPartition();
+        }
+    }
 
     class TraitsTableModel extends AbstractTableModel {
 
@@ -752,52 +799,6 @@ public class TraitsPanel extends BeautiPanel implements Exportable {
         }
     }
 
-    public class CreateTraitPartitionAction extends AbstractAction {
-        public CreateTraitPartitionAction() {
-            super("Create partition from trait ...");
-            setToolTipText("Create a data partition from a trait. Traits can be defined in the Traits panel.");
-        }
 
-        public void actionPerformed(ActionEvent ae) {
-            int[] selRows = traitsTable.getSelectedRows();
-            java.util.List<TraitData> traits = new ArrayList<TraitData>();
-            int discreteCount = 0;
-            int continuousCount = 0;
-            for (int row : selRows) {
-                TraitData trait = options.traits.get(row);
-                traits.add(trait);
+}
 
-                if (trait.getTraitType() == TraitData.TraitType.DISCRETE) {
-                    discreteCount ++;
-                }
-                if (trait.getTraitType() == TraitData.TraitType.CONTINUOUS) {
-                    continuousCount ++;
-                }
-            }
-
-            boolean success = false;
-            if (discreteCount > 0) {
-                if (continuousCount > 0)  {
-                    JOptionPane.showMessageDialog(TraitsPanel.this, "Don't mix discrete and continuous traits when creating partition(s).", "Mixed Trait Types", JOptionPane.ERROR_MESSAGE);
-                    return;
-                }
-
-                // with discrete traits, create a separate partition for each
-                for (TraitData trait : traits) {
-                    java.util.List<TraitData> singleTrait = new ArrayList<TraitData>();
-                    singleTrait.add(trait);
-                    if (dataPanel.createFromTraits(singleTrait)) {
-                        success = true;
-                }
-                }
-            } else {
-                // with
-                success = dataPanel.createFromTraits(traits);
-            }
-            if (success) {
-                frame.switchToPanel(BeautiFrame.DATA_PARTITIONS);
-            }
-        }
-    }
-
-}
\ No newline at end of file
