@@ -48,7 +48,6 @@ public class BeagleSequenceSimulatorConsoleApp {
     private boolean VERBOSE = true;
 
     private Arguments arguments;
-    private PartitionData data;
     private PartitionDataList dataList;
 
     private static final String SPLIT_PARTITION = ":";
@@ -102,11 +101,11 @@ public class BeagleSequenceSimulatorConsoleApp {
     private static final String TO = "to";
     private static final String EVERY = "every";
 
-    private static final String ANCESTRAL_SEQUENCE = "ancestralSequence";
+    private static final String ROOT_SEQUENCE = "rootSequence";
 
     public BeagleSequenceSimulatorConsoleApp() {
 
-        data = new PartitionData();
+//        data = new PartitionData();
         dataList = new PartitionDataList();
 
         // //////////////////
@@ -183,7 +182,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                         new Arguments.RealArrayOption(NUCLEOTIDE_FREQUENCY_PARAMETER_VALUES, 4, "specify nucleotide frequency parameter values"),
                         new Arguments.RealArrayOption(CODON_FREQUENCY_PARAMETER_VALUES, 61, "specify codon frequency parameter values"),
 
-                        new Arguments.StringOption(ANCESTRAL_SEQUENCE, "ancestral sequence",
+                        new Arguments.StringOption(ROOT_SEQUENCE, "ancestral sequence",
                                 "specify ancestral sequence"),
 
                 });
@@ -239,6 +238,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                 // ---INTERROGATE---//
                 // ///////////////////
 
+                PartitionData data = new PartitionData();
                 String option = null;
                 double[] values = null;
 
@@ -287,7 +287,7 @@ public class BeagleSequenceSimulatorConsoleApp {
 
                             values = arguments
                                     .getRealArrayOption(CONSTANT_POPULATION_PARAMETER_VALUES);
-                            parseDemographicValues(index, values);
+                            parseDemographicValues(index, values, data);
 
                         }
 
@@ -301,7 +301,7 @@ public class BeagleSequenceSimulatorConsoleApp {
 
                             values = arguments
                                     .getRealArrayOption(EXPONENTIAL_GROWTH_RATE_PARAMETER_VALUES);
-                            parseDemographicValues(index, values);
+                            parseDemographicValues(index, values, data);
 
                         }
 
@@ -316,7 +316,7 @@ public class BeagleSequenceSimulatorConsoleApp {
 
                             values = arguments
                                     .getRealArrayOption(EXPONENTIAL_GROWTH_DOUBLING_TIME_PARAMETER_VALUES);
-                            parseDemographicValues(index, values);
+                            parseDemographicValues(index, values, data);
 
                         }
 
@@ -341,7 +341,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(HKY_SUBSTITUTION_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(HKY_SUBSTITUTION_PARAMETER_VALUES);
-                            parseSubstitutionValues(index, values);
+                            parseSubstitutionValues(index, values, data);
                         }
 
                     } else if (option.equalsIgnoreCase(GTR)) {
@@ -353,7 +353,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(GTR_SUBSTITUTION_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(GTR_SUBSTITUTION_PARAMETER_VALUES);
-                            parseSubstitutionValues(index, values);
+                            parseSubstitutionValues(index, values, data);
                         }
 
                     } else if (option.equalsIgnoreCase(TN93)) {
@@ -365,7 +365,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(TN93_SUBSTITUTION_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(TN93_SUBSTITUTION_PARAMETER_VALUES);
-                            parseSubstitutionValues(index, values);
+                            parseSubstitutionValues(index, values, data);
                         }
 
                     } else if (option.equalsIgnoreCase(GY94_CODON_MODEL)) {
@@ -377,7 +377,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(GY94_SUBSTITUTION_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(GY94_SUBSTITUTION_PARAMETER_VALUES);
-                            parseSubstitutionValues(index, values);
+                            parseSubstitutionValues(index, values, data);
                         }
 
                     } else {
@@ -405,7 +405,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(GAMMA_SITE_RATE_MODEL_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(GAMMA_SITE_RATE_MODEL_PARAMETER_VALUES);
-                            parseSiteRateValues(index, values);
+                            parseSiteRateValues(index, values, data);
                         }
 
                     } else {
@@ -426,7 +426,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                         if (arguments.hasOption(STRICT_CLOCK_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(STRICT_CLOCK_PARAMETER_VALUES);
-                            parseClockValues(index, values);
+                            parseClockValues(index, values, data);
                         }
 
                     } else if (option.equalsIgnoreCase(LOGNORMAL_RELAXED_CLOCK)) {
@@ -437,7 +437,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(LOGNORMAL_RELAXED_CLOCK_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(LOGNORMAL_RELAXED_CLOCK_PARAMETER_VALUES);
-                            parseClockValues(index, values);
+                            parseClockValues(index, values, data);
                         }
 
                     } else if (option
@@ -449,7 +449,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(EXPONENTIAL_RELAXED_CLOCK_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(EXPONENTIAL_RELAXED_CLOCK_PARAMETER_VALUES);
-                            parseClockValues(index, values);
+                            parseClockValues(index, values, data);
                         }
 
                     } else if (option
@@ -461,7 +461,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(INVERSE_GAUSSIAN_RELAXED_CLOCK_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(INVERSE_GAUSSIAN_RELAXED_CLOCK_PARAMETER_VALUES);
-                            parseClockValues(index, values);
+                            parseClockValues(index, values, data);
                         }
 
                     } else {
@@ -483,7 +483,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(NUCLEOTIDE_FREQUENCY_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(NUCLEOTIDE_FREQUENCY_PARAMETER_VALUES);
-                            parseFrequencyValues(index, values);
+                            parseFrequencyValues(index, values, data);
                         }
 
                     } else if (option.equalsIgnoreCase(CODON_FREQUENCIES)) {
@@ -494,7 +494,7 @@ public class BeagleSequenceSimulatorConsoleApp {
                                 .hasOption(CODON_FREQUENCY_PARAMETER_VALUES)) {
                             values = arguments
                                     .getRealArrayOption(CODON_FREQUENCY_PARAMETER_VALUES);
-                            parseFrequencyValues(index, values);
+                            parseFrequencyValues(index, values, data);
                         }
 
                     } else {
@@ -533,14 +533,17 @@ public class BeagleSequenceSimulatorConsoleApp {
                         data.every // every
                 );
 
-                if (arguments.hasOption(ANCESTRAL_SEQUENCE)) {
+                if (arguments.hasOption(ROOT_SEQUENCE)) {
 
-                    data.ancestralSequenceString = arguments.getStringOption(ANCESTRAL_SEQUENCE);
-                    partition.setAncestralSequence(data.createAncestralSequence());
+                    data.ancestralSequenceString = arguments.getStringOption(ROOT_SEQUENCE);
+                    partition.setRootSequence(data.createAncestralSequence());
 
                 }// END: ANCESTRAL_SEQUENCE option check
 
                 partitionsList.add(partition);
+                
+//                System.err.println(data.from);
+                
                 dataList.add(data);
 
             }// END: partitionArgs loop
@@ -550,6 +553,9 @@ public class BeagleSequenceSimulatorConsoleApp {
             // ////////////////
 
             if (this.VERBOSE) {
+            	
+//        		System.out.println(dataList.get(0).from + " " + dataList.get(1).from);
+            	
                 Utils.printPartitionDataList(dataList);
                 System.out.println();
             }
@@ -601,12 +607,16 @@ public class BeagleSequenceSimulatorConsoleApp {
             }
 
             if (leftoverArguments.length > 2) {
-                dataList.useParallel = Boolean.parseBoolean(leftoverArguments[2]);
+                dataList.outputAncestralSequences = Boolean.parseBoolean(leftoverArguments[2]);
+            }
+            
+            if (leftoverArguments.length > 3) {
+                dataList.useParallel = Boolean.parseBoolean(leftoverArguments[3]);
             }
 
             BeagleSequenceSimulator beagleSequenceSimulator = new BeagleSequenceSimulator(
                     partitionsList);
-            alignment = beagleSequenceSimulator.simulate(dataList.useParallel);
+            alignment = beagleSequenceSimulator.simulate(dataList.useParallel, dataList.outputAncestralSequences);
             alignment.setOutputType(dataList.outputFormat);
 
             PrintWriter writer = new PrintWriter(new FileWriter(outputFile));
@@ -619,6 +629,7 @@ public class BeagleSequenceSimulatorConsoleApp {
             printUsage(arguments);
             System.out.println();
             System.out.println(e.getMessage());
+            e.printStackTrace();
             System.exit(1);
 
         }// END: try-catch block
@@ -626,7 +637,8 @@ public class BeagleSequenceSimulatorConsoleApp {
     }// END: simulate
 
     private void parseDemographicValues(int demographicModelIndex,
-                                        double[] values) {
+                                        double[] values, 
+                                        PartitionData data) {
         for (int i = 0; i < PartitionData.demographicParameterIndices[demographicModelIndex].length; i++) {
 
             int k = PartitionData.demographicParameterIndices[demographicModelIndex][i];
@@ -636,7 +648,8 @@ public class BeagleSequenceSimulatorConsoleApp {
     }// END: parseDemographicValues
 
     private void parseSubstitutionValues(int substitutionModelIndex,
-                                         double[] values) {
+                                         double[] values,
+                                         PartitionData data) {
         for (int i = 0; i < PartitionData.substitutionParameterIndices[substitutionModelIndex].length; i++) {
 
             int k = PartitionData.substitutionParameterIndices[substitutionModelIndex][i];
@@ -646,7 +659,8 @@ public class BeagleSequenceSimulatorConsoleApp {
     }// END: parseSubstitutionValues
 
     private void parseSiteRateValues(int siteRateModelIndex,
-                                     double[] values) {
+                                     double[] values,
+                                     PartitionData data) {
         for (int i = 0; i < PartitionData.siteRateModelParameterIndices[siteRateModelIndex].length; i++) {
 
             int k = PartitionData.siteRateModelParameterIndices[siteRateModelIndex][i];
@@ -656,7 +670,8 @@ public class BeagleSequenceSimulatorConsoleApp {
     }// END: parseSiteRateModelParameterValues
 
     private void parseClockValues(int clockModelIndex,
-                                  double[] values) {
+                                  double[] values,
+                                  PartitionData data) {
         for (int i = 0; i < PartitionData.clockParameterIndices[clockModelIndex].length; i++) {
 
             int k = PartitionData.clockParameterIndices[clockModelIndex][i];
@@ -666,7 +681,8 @@ public class BeagleSequenceSimulatorConsoleApp {
     }// END: parseClockValues
 
     private void parseFrequencyValues(int frequencyModelIndex,
-                                      double[] values) {
+                                      double[] values,
+                                      PartitionData data) {
         for (int i = 0; i < data.frequencyParameterIndices[frequencyModelIndex].length; i++) {
 
             int k = data.frequencyParameterIndices[frequencyModelIndex][i];
@@ -688,11 +704,11 @@ public class BeagleSequenceSimulatorConsoleApp {
 
         arguments.printUsage(
                 "java -Djava.library.path=/usr/local/lib -jar buss.jar", " "
-                + SPLIT_PARTITION + " " + "[<output-file-name>] [<seed>] [<true|false>]");
+                + SPLIT_PARTITION + " " + "[<output-file-name>] [<seed>] [<true|false>] [<true|false>]");
         System.out.println();
 
         System.out
-                .println("  Example: java -Djava.library.path=/usr/local/lib -jar buss.jar "
+                .println("  Example: java -Djava.library.path=/usr/local/lib -jar pibuss.jar "
                         + "-treeFile SimTree.figtree -from 1 -to 500 -every 1 -branchSubstitutionModel HKY -HKYsubstitutionParameterValues 1.0"
                         + " "
                         + SPLIT_PARTITION
