@@ -10,6 +10,7 @@ import dr.app.beagle.evomodel.substmodel.FrequencyModel;
 import dr.app.beagle.evomodel.substmodel.GTR;
 import dr.app.beagle.evomodel.substmodel.GY94CodonModel;
 import dr.app.beagle.evomodel.substmodel.HKY;
+import dr.app.beagle.evomodel.substmodel.MG94HKYCodonModel;
 import dr.app.beagle.evomodel.substmodel.TN93;
 import dr.evolution.coalescent.CoalescentSimulator;
 import dr.evolution.coalescent.ConstantPopulation;
@@ -56,13 +57,13 @@ public class PartitionData implements Serializable {
 	}// END: Constructor
 
 	public int from = 1;
-	public int to = 1000;
+	public int to = 10;
 	public int every = 1;
 
 	public int createPartitionSiteCount() {
 		return ((to - from) / every) + 1;
 	}
-	
+
 	public void resetIdrefs() {
 		resetClockModelIdref();
 		resetFrequencyModelIdref();
@@ -73,10 +74,27 @@ public class PartitionData implements Serializable {
 		resetTaxaIdref();
 	}
 
+	// ///////////////////////
+	// ---TREE ANNOTATING---//
+	// ///////////////////////
+	
+//	private LinkedHashMap<NodeRef, int[]> sequenceMap;
+//	
+//	public void setSequenceMap(LinkedHashMap<NodeRef, int[]> sequenceMap) {
+//		this.sequenceMap = sequenceMap;
+//	}
+//	
+//	public LinkedHashMap<NodeRef, int[]> getSequenceMap() {
+//		return sequenceMap;
+//	}
+	
 	// /////////////////////////
 	// ---DEMOGRAPHIC MODEL---//
 	// /////////////////////////
 
+	//TODO: LogisticGrowth.getInverseIntensity
+	public static final int lastImplementedIndex = 3;
+	
 	public int demographicModelIndex = 0;
 	
 	public String demographicModelIdref = Utils.DEMOGRAPHIC_MODEL;
@@ -89,8 +107,8 @@ public class PartitionData implements Serializable {
     	"No Model (user-specified tree)",
     	"Constant Population",
         "Exponential Growth (Growth Rate)",
-        "Exponential Growth (Doubling Time)"
-//        "Logistic Growth (Growth Rate)",
+        "Exponential Growth (Doubling Time)",
+        "Logistic Growth (Growth Rate)",
 //        "Logistic Growth (Doubling Time)",
 //        "Expansion (Growth Rate)",
 //        "Expansion (Doubling Time)",
@@ -98,19 +116,25 @@ public class PartitionData implements Serializable {
 	
 	public static String[] demographicParameterNames = new String[] {
 	        "Population Size", // Constant Population
-			"Population Size", // Exponential Growth (Growth Rate)
+			
+	        "Population Size", // Exponential Growth (Growth Rate)
 			"Growth Rate", // Exponential Growth (Growth Rate)
+			
 			"Population Size", // Exponential Growth (Doubling Time)
-			"Doubling Time" // Exponential Growth (Doubling Time)
+			"Doubling Time", // Exponential Growth (Doubling Time)
+			
 //			"Population Size", // Logistic Growth (Growth Rate)
 //			"Growth Rate", // Logistic Growth (Growth Rate)
 //			"Logistic Shape (Half-life)", // Logistic Growth (Growth Rate)
+			
 //			"Population Size", // Logistic Growth (Doubling Time)
 //			"Doubling Time", // Logistic Growth (Doubling Time)
 //			"Logistic Shape (Half-life)", // Logistic Growth (Doubling Time)
+			
 //			"Population Size", // Expansion (Growth Rate)
 //			"Ancestral Proportion", // Expansion (Growth Rate)
 //			"Growth Rate", // Expansion (Growth Rate)
+			
 //			"Population Size", // Expansion (Doubling Time)
 //			"Ancestral Proportion", // Expansion (Doubling Time)
 //			"Doubling Time", // Expansion (Doubling Time)
@@ -120,7 +144,7 @@ public class PartitionData implements Serializable {
 		    {  }, // No model
 			{ 0 }, // Constant Population
 			{ 1, 2 }, // Exponential Growth (Growth Rate)
-			{ 3, 4 }// Exponential Growth (Doubling Time)
+			{ 3, 4 }, // Exponential Growth (Doubling Time)
 //			{ 5, 6, 7 }, // Logistic Growth (Growth Rate)
 //			{ 8, 9, 10 }, // Logistic Growth (Doubling Time)
 //			{ 11, 12, 13 }, // Expansion (Growth Rate)
@@ -139,11 +163,13 @@ public class PartitionData implements Serializable {
 			
 			/*Exponential Growth (Doubling Time)*/
 			1000.0, // Population Size
-			10.0 // Doubling Time
+			10.0, // Doubling Time
 			
+//			/*Logistic Growth (Growth Rate)*/
 //			1000.0, // Population Size
 //			0.5, // Growth Rate
 //			50.0, // Logistic Shape (Half-life)
+			
 //			1000.0, // Population Size
 //			10.0, // Doubling Time
 //			50.0, // Logistic Shape (Half-life)
@@ -247,13 +273,13 @@ public class PartitionData implements Serializable {
 			
 			treeModel = new TreeModel(this.record.getTree());
 			
-		} else if( (this.demographicModelIndex > 0 && this.demographicModelIndex <= 3) && this.record.isTreeSet()) {
+		} else if( (this.demographicModelIndex > 0 && this.demographicModelIndex <= lastImplementedIndex) && this.record.isTreeSet()) {
 			
 			Taxa taxa = new Taxa(this.record.getTree().asList()); 
 			CoalescentSimulator topologySimulator = new CoalescentSimulator();
 			treeModel = new TreeModel(topologySimulator.simulateTree(taxa, createDemographicFunction()));			
 			
-		} else if((this.demographicModelIndex > 0 && this.demographicModelIndex <= 3) && this.record.isTaxaSet()) {
+		} else if((this.demographicModelIndex > 0 && this.demographicModelIndex <= lastImplementedIndex) && this.record.isTaxaSet()) {
 			
 			Taxa taxa = this.record.getTaxa();
 			CoalescentSimulator topologySimulator = new CoalescentSimulator();
@@ -323,6 +349,7 @@ public class PartitionData implements Serializable {
 			"GTR", //
 			"TN93", //
 			"GY94CodonModel", //
+			"MG94CodonModel",
             "Blosum62", //	
 			"CPREV", //
 			"Dayhoff", //
@@ -337,6 +364,7 @@ public class PartitionData implements Serializable {
 			0, // GTR
 			0, // TN93
 			1, // GY94CodonModel
+			1, // MG94CodonModel
 			2, // Blosum62
 			2, // CPREV
 			2, // Dayhoff
@@ -357,14 +385,19 @@ public class PartitionData implements Serializable {
 			"GT", // GTR
 			"Kappa 1 (A-G)", // TN93
 			"Kappa 2 (C-T)", // TN93
-			"Omega value", // Yang Codon Model
-			"Kappa value" // Yang Codon Model
+			"Omega value", // GY94CodonModel
+			"Kappa value", // GY94CodonModel
+			"Alpha value", // MG94CodonModel
+			"Beta value", // MG94CodonModel
+			"Kappa value" // MG94CodonModel
+			
 	};
 
 	public static int[][] substitutionParameterIndices = { { 0 }, // HKY
 			{ 1, 2, 3, 4, 5, 6 }, // GTR
 			{ 7, 8 }, // TN93
-			{ 9, 10 }, // Yang Codon Model
+			{ 9, 10 }, // GY94CodonModel
+			{11, 12, 13}, // MG94CodonModel
 			{}, // Blosum62
 			{}, // CPREV
 			{}, // Dayhoff
@@ -385,7 +418,10 @@ public class PartitionData implements Serializable {
 			1.0, // Kappa 1
 			1.0, // Kappa 2
 			0.1, // Omega value
-			1.0 // Kappa value
+			1.0, // Kappa value
+			10.0, // Alpha value
+			1.0, // Beta value
+			1.0 // kappa value
 	};
 
 	public BranchModel createBranchModel() {
@@ -450,7 +486,21 @@ public class PartitionData implements Serializable {
 
 			branchModel = new HomogeneousBranchModel(yangCodonModel);
 
-		} else if (this.substitutionModelIndex == 4) { // Blosum62
+			
+		} else if(this.substitutionModelIndex == 4) { // MG94CodonModel
+			
+			
+			FrequencyModel frequencyModel = this.createFrequencyModel();
+
+			Parameter alpha = new Parameter.Default(1, substitutionParameterValues[11]);
+			Parameter beta = new Parameter.Default(1, substitutionParameterValues[12]);
+			Parameter kappa = new Parameter.Default(1, substitutionParameterValues[13]);
+			
+			MG94HKYCodonModel mg94 = new MG94HKYCodonModel(Codons.UNIVERSAL, alpha, beta, kappa, frequencyModel);
+
+			branchModel = new HomogeneousBranchModel(mg94);
+			
+		} else if (this.substitutionModelIndex == 5) { // Blosum62
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -462,7 +512,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);
 			
-        } else if (this.substitutionModelIndex == 5) { // CPREV
+        } else if (this.substitutionModelIndex == 6) { // CPREV
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -474,7 +524,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);
 			
-        } else if (this.substitutionModelIndex == 6) { // Dayhoff
+        } else if (this.substitutionModelIndex == 7) { // Dayhoff
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -486,7 +536,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);
 		
-        } else if (this.substitutionModelIndex == 7) { // JTT
+        } else if (this.substitutionModelIndex == 8) { // JTT
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -498,7 +548,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);
 		
-        } else if (this.substitutionModelIndex == 8) { // LG
+        } else if (this.substitutionModelIndex == 9) { // LG
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -510,7 +560,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);
 			
-        } else if (this.substitutionModelIndex == 9) { // MTREV
+        } else if (this.substitutionModelIndex == 10) { // MTREV
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -522,7 +572,7 @@ public class PartitionData implements Serializable {
 			branchModel = new HomogeneousBranchModel(
 					empiricalAminoAcidModel);	
 			
-        } else if (this.substitutionModelIndex == 10) { // WAG
+        } else if (this.substitutionModelIndex == 11) { // WAG
 			
 			FrequencyModel frequencyModel = this.createFrequencyModel();
 			
@@ -868,8 +918,8 @@ public class PartitionData implements Serializable {
 	};
 
 	public double[] clockParameterValues = new double[] { 1.0, // clockrate
-			1.0, // ucld.mean
-			2.0, // ucld.stdev
+			0.001, // ucld.mean
+			1.0, // ucld.stdev
 			0.0, // ucld.offset
 			1.0, // uced.mean
             0.0, // uced.offset
@@ -895,10 +945,18 @@ public class PartitionData implements Serializable {
 			
 			Parameter mean = new Parameter.Default(LogNormalDistributionModelParser.MEAN, 1, clockParameterValues[1]);
 			Parameter stdev = new Parameter.Default(LogNormalDistributionModelParser.STDEV, 1, clockParameterValues[2]);
-	        ParametricDistributionModel distributionModel = new LogNormalDistributionModel(mean, stdev, clockParameterValues[3], true, true);
+			//TODO: choose between log scale / real scale
+	        ParametricDistributionModel distributionModel = new LogNormalDistributionModel(mean, stdev, clockParameterValues[3], false, false);
 	        
-	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), rateCategoryParameter, 
-	                distributionModel, 1, false, Double.NaN);
+	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), //
+	        		rateCategoryParameter, //
+	                distributionModel, //
+	                1, // 
+	                false, // 
+	                Double.NaN, //
+	                true, //randomizeRates
+	                false // keepRates
+	                );
 
 		} else if(this.clockModelIndex == 2) { // Exponential relaxed clock
 		
@@ -908,9 +966,19 @@ public class PartitionData implements Serializable {
 			Parameter mean = new Parameter.Default(DistributionModelParser.MEAN, 1, clockParameterValues[4]);
 	        ParametricDistributionModel distributionModel = new ExponentialDistributionModel(mean, clockParameterValues[5]);
 			
-	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), rateCategoryParameter, 
-	                distributionModel, 1, false, Double.NaN);
+//	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), rateCategoryParameter, 
+//	                distributionModel, 1, false, Double.NaN);
 			
+	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), //
+	        		rateCategoryParameter, //
+	                distributionModel, //
+	                1, // 
+	                false, // 
+	                Double.NaN, //
+	                true, //randomizeRates
+	                false // keepRates
+	                );
+	        
 		} else if(this.clockModelIndex == 3) { // Inverse Gaussian
 
 			double numberOfBranches = 2 * (createTreeModel().getTaxonCount() - 1);
@@ -921,8 +989,18 @@ public class PartitionData implements Serializable {
 	        ParametricDistributionModel distributionModel = new InverseGaussianDistributionModel(
 					mean, stdev, clockParameterValues[8], false);
      
-	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), rateCategoryParameter, 
-	                distributionModel, 1, false, Double.NaN);
+//	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), rateCategoryParameter, 
+//	                distributionModel, 1, false, Double.NaN);
+	        
+	        branchRateModel = new DiscretizedBranchRates(createTreeModel(), //
+	        		rateCategoryParameter, //
+	                distributionModel, //
+	                1, // 
+	                false, // 
+	                Double.NaN, //
+	                true, //randomizeRates
+	                false // keepRates
+	                );
 	        
 		} else {
 
