@@ -1,3 +1,28 @@
+/*
+ * Utils.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.app.bss;
 
 import java.awt.Desktop;
@@ -12,9 +37,12 @@ import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.net.URL;
 import java.nio.charset.Charset;
+import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.LinkedHashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Map.Entry;
 import java.util.concurrent.ConcurrentHashMap;
@@ -24,15 +52,24 @@ import javax.swing.JOptionPane;
 import javax.swing.JTabbedPane;
 import javax.swing.SwingUtilities;
 
+import org.apache.commons.math.random.MersenneTwister;
+
+import dr.app.bss.test.AncestralSequenceTrait;
+import dr.evolution.datatype.Codons;
+import dr.evolution.datatype.DataType;
 import dr.evolution.io.Importer.ImportException;
 import dr.evolution.io.NewickImporter;
 import dr.evolution.io.NexusImporter;
+import dr.evolution.sequence.Sequence;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
+import dr.evolution.tree.TreeTraitProvider;
 import dr.evolution.util.MutableTaxonList;
 import dr.evolution.util.Taxa;
 import dr.evolution.util.Taxon;
 import dr.evomodel.tree.TreeModel;
+import dr.inference.model.Parameter;
+import dr.math.MathUtils;
 
 /**
  * @author Filip Bielejec
@@ -40,6 +77,245 @@ import dr.evomodel.tree.TreeModel;
  */
 public class Utils {
 
+	// ////////////////////
+	// ---THREAD UTILS---//
+	// ////////////////////
+
+	public static void sleep(int seconds) {
+		try {
+
+			Thread.sleep(seconds * 1000);
+		} catch (InterruptedException e) {
+			e.printStackTrace();
+		}
+	}// END: sleep
+	
+	// ////////////////////////////////
+	// ---RANDOM NUMB3R GENERATION---//
+	// ////////////////////////////////
+	
+	private static MersenneTwister random = new MersenneTwister(
+			MathUtils.nextLong());
+
+    public static double rLogNormal(double stdev, double mean) {
+    	
+    	double rNorm = random.nextGaussian() * stdev + mean;
+    	double rLognormal = Math.exp(rNorm);
+    	
+    	return rLognormal;
+	}// END: drawRandom
+	
+	public static int rMultinom(double[] probabilities) {
+
+		int range = probabilities.length + 1;
+		double[] distribution = new double[range];
+		double sumProb = 0;
+
+		for (double value : probabilities) {
+			sumProb += value;
+		}// END: probabilities loop
+
+		distribution[0] = 0;
+		for (int i = 1; i < range; ++i) {
+
+			distribution[i] = distribution[i - 1]
+					+ (probabilities[i - 1] / sumProb);
+
+		}// END: i loop
+
+		distribution[range - 1] = 1.0;
+
+		double key = random.nextDouble();
+
+		int mindex = 1;
+		int maxdex = range - 1;
+		int midpoint = mindex + (maxdex - mindex) / 2;
+		while (mindex <= maxdex) {
+
+			if (key < distribution[midpoint - 1]) {
+				
+				maxdex = midpoint - 1;
+				
+			} else if (key > distribution[midpoint]) {
+				
+				mindex = midpoint + 1;
+				
+			} else {
+				
+				return midpoint - 1;
+
+			}
+			
+			midpoint = mindex + (int) Math.ceil((maxdex - mindex) / 2);
+
+		}//END: mindex loop
+		
+		System.out.println("Error in rMultinom!");
+		
+		return range - 1;
+	}//END: rMultinom
+    
+	// ////////////
+	// ---MATH---//
+	// ////////////
+    
+	public static int sample(double[] probabilities) {
+
+		int samplePos = -Integer.MAX_VALUE;
+		double cumProb = 0.0;
+		double u = random.nextDouble();
+
+		for (int i = 0; i < probabilities.length; i++) {
+
+			cumProb += probabilities[i];
+
+			if (u <= cumProb) {
+				samplePos = i;
+				break;
+			}
+		}
+
+		return samplePos;
+	}// END: randomChoicePDF
+	
+    public static void rescale(double[] logX) {
+        double max = max(logX);
+        for (int i = 0; i < logX.length; i++) {
+            logX[i] -= max;
+		}
+	}// END: rescale
+
+	public double getParameterVariance(Parameter param) {
+		
+		int n = param.getSize();
+		double mean = getParameterMean(param);
+		double var = 0;
+		
+		for (int i = 0; i < n; i++) {
+
+			var+= Math.pow( (param.getValue(i) - mean), 2);
+
+		}
+		
+		var/= (n-1);
+		
+		return var;
+	}// END: getParameterVariance
+
+	public double getParameterMean(Parameter param) {
+		double mean = 0;
+		int n = param.getSize();
+		for (int i = 0; i < n; i++) {
+			mean += param.getValue(i);
+		}
+		mean /= n;
+
+		return mean;
+	}// END: getParameterMean
+	
+	public static double getNorm(double[] vector) {
+		double norm = 0;
+		for (int i = 0; i < vector.length; i++) {
+			norm += Math.pow(vector[i], 2);
+		}
+
+		return Math.sqrt(norm);
+	}// END: getNorm
+
+	public static void normalize(double[] vector) {
+
+		double norm = getNorm(vector);
+		for (int i = 0; i < vector.length; i++) {
+			vector[i] = vector[i] / norm;
+		}
+
+	}// END: normalize
+	
+	// ////////////////////
+	// ---ARRAYS UTILS---//
+	// ////////////////////
+
+	public static void exponentiate(double[] array) {
+		for (int i = 0; i < array.length; i++) {
+			array[i] = Math.exp(array[i]);
+		}
+	}// END: exponentiate
+    
+	public static int max(int[] array) {
+
+		int max = -Integer.MAX_VALUE;
+
+		for (int i = 0; i < array.length; i++) {
+		
+			if (array[i] > max) {
+
+				max = (int)array[i];
+
+			}// END: if check
+
+		}// END: i loop
+
+		return max;
+	}// END: findMaximum
+	
+	public static int max(double[] array) {
+
+		int max = -Integer.MAX_VALUE;
+
+		for (int i=0; i< array.length;i++) {
+		
+			if (array[i] > max) {
+
+				max = (int)array[i];
+
+			}// END: if check
+
+		}// END: i loop
+
+		return max;
+	}// END: findMaximum
+	
+	public static int max(ArrayList<Integer> array) {
+
+		int max = -Integer.MAX_VALUE;
+
+		for (Integer element : array) {
+
+			if (element > max) {
+
+				max = element;
+
+			}// END: if check
+
+		}// END: i loop
+
+		return max;
+	}// END: findMaximum
+
+	public static double sumArray(int[] array) {
+
+		double sum = 0.0;
+		for (int i = 0; i < array.length; i++) {
+
+			sum += array[i];
+
+		}
+
+		return sum;
+	}// END: sumArray
+    
+	public static double sumArray(double[] array) {
+
+		double sum = 0.0;
+		for (int i = 0; i < array.length; i++) {
+
+			sum += array[i];
+
+		}
+
+		return sum;
+	}// END: sumArray
+	
 	// /////////////////
 	// ---CONSTANTS---//
 	// /////////////////
@@ -50,7 +326,7 @@ public class Utils {
 	public static final int BRANCH_RATE_MODEL_ELEMENT = 3;
 	public static final int FREQUENCY_MODEL_ELEMENT = 4;
 	public static final int DEMOGRAPHIC_MODEL_ELEMENT = 5;
-	
+
 	public static final String TOPOLOGY = "topology";
 	public static final String ABSOLUTE_HEIGHT = "absoluteHeight";
 	public static final String TREE_FILENAME = "treeFilename";
@@ -61,7 +337,7 @@ public class Utils {
 	public static final String CHOOSE_FILE = "Choose file...";
 	public static final String EDIT_TAXA_SET = "Edit taxa set...";
 	public static final String ANCESTRAL_SEQUENCE = "ancestralSequence";
-	
+
 	public static final String BSS_ICON = "icons/bss.png";
 	public static final String CHECK_ICON = "icons/check.png";
 	public static final String ERROR_ICON = "icons/error.png";
@@ -71,11 +347,48 @@ public class Utils {
 	public static final String BUBBLE_BLUE_ICON = "icons/bubble-blue.png";
 	public static final String SAVE_ICON = "icons/save.png";
 	public static final String TEXT_FILE_ICON = "icons/file.png";
-	
+
+	public static double[] UNIFORM_CODON_FREQUENCIES = new double[] {
+			0.0163936, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344, 0.01639344, 0.01639344, 0.01639344, 0.01639344,
+			0.01639344 };
+
 	// ///////////////////////////////
 	// ---GENERAL UTILITY METHODS---//
 	// ///////////////////////////////
 
+	public static double logfactor(int n) {
+
+		double logfactor = 0.0;
+		for (int i = 1; i <= n; i++) {
+			logfactor += Math.log(i);
+		}
+
+		return logfactor;
+	}
+
+	public static double map(double value, double low1, double high1,
+			double low2, double high2) {
+		/**
+		 * maps a single value from its range into another interval
+		 * 
+		 * @param low1
+		 *            , high1 - range of value; low2, high2 - interval
+		 * @return the mapped value
+		 */
+		return (value - low1) / (high1 - low1) * (high2 - low2) + low2;
+	}// END: map
+
 	public static String[] loadStrings(String filename) throws IOException {
 
 		int linesCount = countLines(filename);
@@ -124,7 +437,7 @@ public class Utils {
 		return (count == 0 && !empty) ? 1 : count;
 
 	}// END: countLines
-	
+
 	public static Taxa importTaxaFromFile(File file) throws IOException {
 
 		Taxa taxa = new Taxa();
@@ -145,7 +458,7 @@ public class Utils {
 
 		return taxa;
 	}// END: importTaxaFromFile
-	
+
 	public static Tree importTreeFromFile(File file) throws IOException,
 			ImportException {
 
@@ -170,23 +483,24 @@ public class Utils {
 
 		return tree;
 	}// END: importTreeFromFile
-	
+
 	public static void removeTaxaWithAttributeValue(PartitionDataList dataList,
 			String attribute, String value) {
-		
-			for (int i = 0; i < dataList.allTaxa.getTaxonCount(); i++) {
 
-				Taxon taxon = dataList.allTaxa.getTaxon(i);
-				if (taxon.getAttribute(attribute).toString().equalsIgnoreCase(value)) {
-					
-					dataList.allTaxa.removeTaxon(taxon);
-					i--;
-					
-				}
+		for (int i = 0; i < dataList.allTaxa.getTaxonCount(); i++) {
+
+			Taxon taxon = dataList.allTaxa.getTaxon(i);
+			if (taxon.getAttribute(attribute).toString()
+					.equalsIgnoreCase(value)) {
+
+				dataList.allTaxa.removeTaxon(taxon);
+				i--;
+
 			}
+		}
 
 	}// END: removeTaxaWithAttributeValue
-	
+
 	public static void centreLine(String line, int pageWidth) {
 		int n = pageWidth - line.length();
 		int n1 = n / 2;
@@ -237,9 +551,10 @@ public class Utils {
 		boolean exists = false;
 		for (Taxon taxon2 : taxonList) {
 
-			if (taxon.equals(taxon2) 
-//					&& taxon.getAttribute(Utils.TREE_FILENAME).toString().equalsIgnoreCase(taxon2.getAttribute(Utils.TREE_FILENAME).toString())
-					) {
+			if (taxon.equals(taxon2)
+			// &&
+			// taxon.getAttribute(Utils.TREE_FILENAME).toString().equalsIgnoreCase(taxon2.getAttribute(Utils.TREE_FILENAME).toString())
+			) {
 				exists = true;
 				break;
 			}
@@ -249,21 +564,21 @@ public class Utils {
 		return exists;
 	}// END: taxonExists
 
-//	 private boolean isFileInList(File file) {
-//	 boolean exists = false;
-//	
-//	 for (File file2 : dataList.treesList) {
-//	
-//	 if (file.getName().equalsIgnoreCase(file2.getName())) {
-//	 exists = true;
-//	 break;
-//	 }
-//	
-//	 }
-//	
-//	 return exists;
-//	 }// END: isFileInList
-	
+	// private boolean isFileInList(File file) {
+	// boolean exists = false;
+	//
+	// for (File file2 : dataList.treesList) {
+	//
+	// if (file.getName().equalsIgnoreCase(file2.getName())) {
+	// exists = true;
+	// break;
+	// }
+	//
+	// }
+	//
+	// return exists;
+	// }// END: isFileInList
+
 	public static double getAbsoluteTaxonHeight(Taxon taxon, Tree tree) {
 
 		double height = 0.0;
@@ -296,15 +611,15 @@ public class Utils {
 
 		return exists;
 	}// END: isRecordInList
-	
-	public static boolean isTaxaInList(Taxa taxa,
-			ArrayList<Taxa> taxaList) {
+
+	public static boolean isTaxaInList(Taxa taxa, ArrayList<Taxa> taxaList) {
 
 		boolean exists = false;
 
 		for (Taxa taxa2 : taxaList) {
 
-			if (taxaToString(taxa,true).equalsIgnoreCase(taxaToString(taxa2,true))) {
+			if (taxaToString(taxa, true).equalsIgnoreCase(
+					taxaToString(taxa2, true))) {
 				exists = true;
 				break;
 			}
@@ -313,15 +628,15 @@ public class Utils {
 
 		return exists;
 	}// END: isTaxaInList
-	
-	public static int taxaIsIdenticalWith(Taxa taxa,
-			ArrayList<Taxa> taxaList) {
+
+	public static int taxaIsIdenticalWith(Taxa taxa, ArrayList<Taxa> taxaList) {
 
 		int index = -Integer.MAX_VALUE;
 
 		for (Taxa taxa2 : taxaList) {
 
-			if (taxaToString(taxa, true).equalsIgnoreCase(taxaToString(taxa2, true))) {
+			if (taxaToString(taxa, true).equalsIgnoreCase(
+					taxaToString(taxa2, true))) {
 				index = taxaList.indexOf(taxa2);
 				break;
 			}
@@ -330,7 +645,7 @@ public class Utils {
 
 		return index;
 	}// END: treeModelIsIdenticalWith
-	
+
 	public static boolean isTreeModelInList(TreeModel treeModel,
 			ArrayList<TreeModel> treeModelList) {
 
@@ -383,7 +698,7 @@ public class Utils {
 			}
 
 			break;
-		
+
 		case BRANCH_RATE_MODEL_ELEMENT:
 
 			for (PartitionData data2 : partitionList) {
@@ -459,7 +774,7 @@ public class Utils {
 			}
 
 			break;
-		
+
 		case BRANCH_RATE_MODEL_ELEMENT:
 
 			for (PartitionData data2 : partitionList) {
@@ -517,7 +832,6 @@ public class Utils {
 		return index;
 	}// END: isIdenticalWith
 
-
 	// /////////////////
 	// ---GUI UTILS---//
 	// /////////////////
@@ -581,8 +895,7 @@ public class Utils {
 	}
 
 	public static String getMultipleWritePath(File outFile,
-			String defaultExtension, 
-			int i) {
+			String defaultExtension, int i) {
 
 		String path = outFile.getParent();
 		String[] nameArray = outFile.getName().split("\\.", 2);
@@ -703,23 +1016,40 @@ public class Utils {
 	// ///////////////////
 
 	public static void printMap(Map<?, ?> mp) {
+
 		Iterator<?> it = mp.entrySet().iterator();
 		while (it.hasNext()) {
+
 			Entry<?, ?> pairs = (Entry<?, ?>) it.next();
 			Object obj = pairs.getValue();
+
 			if (obj instanceof int[]) {
+
 				int[] seq = (int[]) obj;
 				System.out.print(pairs.getKey() + " =");
+
 				for (int i = 0; i < seq.length; ++i) {
 					System.out.print(" " + seq[i]);
 				}
 				System.out.println();
+
+			} else if (obj instanceof double[]) {
+
+				double[] seq = (double[]) obj;
+				System.out.print(pairs.getKey() + " =");
+
+				for (int i = 0; i < seq.length; ++i) {
+					System.out.print(" " + seq[i]);
+				}
+				System.out.println();
+
 			} else {
 				System.out.println(pairs.getKey() + " = " + pairs.getValue());
-			}
+			}// END: obj class check
+
 		}
 	}// END: printMap
-	
+
 	public static void printHashMap(ConcurrentHashMap<?, ?> hashMap) {
 
 		Iterator<?> iterator = hashMap.entrySet().iterator();
@@ -739,8 +1069,9 @@ public class Utils {
 
 	public static void printArray(Object[] x) {
 		for (int i = 0; i < x.length; i++) {
-			System.out.println(x[i]);
+			System.out.print(x[i] + " ");
 		}
+		System.out.println();
 	}// END: printArray
 
 	public static void printArray(int[] x) {
@@ -752,10 +1083,11 @@ public class Utils {
 
 	public static void printArray(double[] x) {
 		for (int i = 0; i < x.length; i++) {
-			System.out.println(x[i]);
+			System.out.print(x[i] + " ");
 		}
+		System.out.println();
 	}// END: printArray
-	
+
 	public static void printArray(String[] x) {
 		for (int i = 0; i < x.length; i++) {
 			System.out.println(x[i]);
@@ -771,6 +1103,40 @@ public class Utils {
 		}
 	}// END: print2DArray
 
+	public static void print2DArray(int[][] array) {
+		for (int row = 0; row < array.length; row++) {
+			for (int col = 0; col < array[row].length; col++) {
+				System.out.print(array[row][col] + " ");
+			}
+			System.out.print("\n");
+		}
+	}// END: print2DArray
+	
+	public static void print2Arrays(int[] array1, double[] array2, int nrow) {
+		for (int row = 0; row < nrow; row++) {
+			System.out.print(array1[row] + " " + array2[row] + " ");
+			System.out.print("\n");
+		}
+	}// END: print2DArray
+
+	public static void print2DArray(double[][] array, int formatEvery) {
+
+		int i = 0;
+		for (int row = 0; row < array.length; row++) {
+			for (int col = 0; col < array[row].length; col++) {
+
+				if (i == formatEvery) {
+					System.out.print("\n");
+					i = 0;
+				}
+
+				System.out.print(array[row][col] + " ");
+				i++;
+			}
+			System.out.print("\n");
+		}
+	}// END: print2DArray
+
 	public static void printBranchSubstitutionModel(PartitionData data) {
 		System.out.print("\tBranch Substitution model: ");
 		System.out.print(branchSubstitutionModelToString(data));
@@ -806,7 +1172,7 @@ public class Utils {
 		System.out.print(dataTypeToString(data));
 		System.out.print("\n");
 	}// END: printDataType
-	
+
 	public static void printTaxaSet(Taxa taxa) {
 		for (int i = 0; i < taxa.getTaxonCount(); i++) {
 			Taxon taxon = taxa.getTaxon(i);
@@ -818,21 +1184,21 @@ public class Utils {
 		System.out.print(record.getTree().toString());
 		System.out.print("\n");
 	}// END: printTree
-	
+
 	public static void printRecord(TreesTableRecord record) {
-		
+
 		if (record == null) {
 
 			System.out.println("\tRecord: NOT SET");
 
 		} else if (record.isTreeSet()) {
 
-			System.out.print("\t"+record.getName() + ": ");
+			System.out.print("\t" + record.getName() + ": ");
 			printTree(record);
 
 		} else if (record.isTaxaSet()) {
 
-			System.out.println("\t"+record.getName() + ":");
+			System.out.println("\t" + record.getName() + ":");
 			printTaxaSet(record.getTaxa());
 
 		} else {
@@ -845,9 +1211,9 @@ public class Utils {
 			printRecord(record);
 		}// END: record loop
 	}// END: printRecords
-	
+
 	public static void printPartitionData(PartitionData data) {
-        printRecord(data.record);
+		printRecord(data.record);
 		printDataType(data);
 		printDemographicModel(data);
 		System.out.println("\tFrom: " + data.from);
@@ -861,14 +1227,17 @@ public class Utils {
 
 	public static void printPartitionDataList(PartitionDataList dataList) {
 
+		// System.out.println(dataList.get(0).from + " " +
+		// dataList.get(1).from);
+
 		if (BeagleSequenceSimulatorApp.DEBUG) {
 			System.out.println("Possible records: ");
 			printRecords(dataList);
 		}
-		
+
 		System.out.println("\tSite count: " + getSiteCount(dataList));
 		System.out.println("\tOutput type: " + dataList.outputFormat);
-		
+
 		if (dataList.setSeed) {
 			System.out.println("\tStarting seed: " + dataList.startingSeed);
 		}
@@ -888,78 +1257,130 @@ public class Utils {
 		System.out.println(taxaToString(dataList.allTaxa, true));
 	}// END: printTaxonList
 
+	public static Sequence intArray2Sequence(Taxon taxon, int[] seq,
+			int gapFlag, DataType dataType) {
+
+		StringBuilder sSeq = new StringBuilder();
+		int partitionSiteCount = seq.length;
+
+		if (dataType instanceof Codons) {
+
+			for (int i = 0; i < partitionSiteCount; i++) {
+
+				int state = seq[i];
+
+				if (state == gapFlag) {
+					sSeq.append(dataType.getTriplet(dataType.getGapState()));
+				} else {
+					sSeq.append(dataType.getTriplet(seq[i]));
+				}// END: gap check
+
+			}// END: replications loop
+
+		} else {
+
+			for (int i = 0; i < partitionSiteCount; i++) {
+
+				int state = seq[i];
+
+				if (state == gapFlag) {
+					sSeq.append(dataType.getCode(dataType.getGapState()));
+				} else {
+					sSeq.append(dataType.getCode(seq[i]));
+				}// END: gap check
+
+			}// END: replications loop
+
+		}// END: dataType check
+
+		return new Sequence(taxon, sSeq.toString());
+	}// END: intArray2Sequence
+
 	// //////////////////////
 	// ---TOSTRING UTILS---//
 	// //////////////////////
 
 	public static String taxonToString(Taxon taxon, boolean printNames) {
-		
+
 		String string = null;
-		
-		if(printNames) {
-		 string = taxon.getId() + " ("
-				+ taxon.getAttribute(Utils.ABSOLUTE_HEIGHT) + ","
-				+ taxon.getAttribute(Utils.TREE_FILENAME) + ")";
-		
+
+		if (printNames) {
+			string = taxon.getId() + " ("
+					+ taxon.getAttribute(Utils.ABSOLUTE_HEIGHT) + ","
+					+ taxon.getAttribute(Utils.TREE_FILENAME) + ")";
+
 		} else {
-			 string = taxon.getId() + " ("
-					+ taxon.getAttribute(Utils.ABSOLUTE_HEIGHT)+")";
+			string = taxon.getId() + " ("
+					+ taxon.getAttribute(Utils.ABSOLUTE_HEIGHT) + ")";
 		}
-		
+
 		return string;
 	}// END: taxonToString
-	
+
 	public static String taxaToString(Taxa taxa, boolean printNames) {
 
 		String string = "";
 
 		for (int i = 0; i < taxa.getTaxonCount(); i++) {
-			
+
 			Taxon taxon = taxa.getTaxon(i);
 			string += taxonToString(taxon, printNames) + ("\n");
-			
+
 		}
-		
+
 		return string;
 	}// END: taxaToString
-	
-	public static String partitionDataToString(PartitionData data, TreeModel simulatedTreeModel) {
+
+	public static String partitionDataToString(PartitionData data,
+			TreeModel simulatedTreeModel
+	// , LinkedHashMap<NodeRef, int[]> sequencesMap
+	) {
 
 		String string = "";
-		
-//		if (data.record.isTreeSet()) {
-//
-//			string += ("Tree: " + data.record.getTree().toString())+ ("\n");
-//			
-//		} else if (data.record.isTaxaSet()) {
-//
-//			string += ("Taxa Set: \n" + taxaToString(data.record.getTaxa(), false));//+ ("\n");
-//
-//		} else {
-//			//
-//		}
-		
+
+		// if (data.record.isTreeSet()) {
+		//
+		// string += ("Tree: " + data.record.getTree().toString())+ ("\n");
+		//
+		// } else if (data.record.isTaxaSet()) {
+		//
+		// string += ("Taxa Set: \n" + taxaToString(data.record.getTaxa(),
+		// false));//+ ("\n");
+		//
+		// } else {
+		// //
+		// }
+
 		string += ("Tree model: " + simulatedTreeModel.toString()) + ("\n");
+		// string += ("Tree model: "
+		// +annotatedTreeModelToString(simulatedTreeModel, sequencesMap,
+		// data.createDataType()) ) + ("\n");
+		string += ("From: " + data.from) + ("\n");
+		string += ("To: " + data.to) + ("\n");
+		string += ("Every: " + data.every) + ("\n");
 		string += ("Data type: ") + dataTypeToString(data) + ("\n");
-		string += ("Demographic model: ") + demographicModelToString(data) + ("\n");
-		string += ("From: " + data.from)+ ("\n");
-		string += ("To: " + data.to)+ ("\n");
-		string += ("Every: " + data.every)+ ("\n");
-		string += ("Branch Substitution model: ") + branchSubstitutionModelToString(data) + ("\n");
-		string += ("Site Rate model: ") + siteRateModelToString(data) + ("\n");
-		string += ("Clock Rate model: ") + clockRateModelToString(data) + ("\n");
+		string += ("Demographic model: ") + demographicModelToString(data)
+				+ ("\n");
+		string += ("Branch Substitution model: ")
+				+ branchSubstitutionModelToString(data) + ("\n");
 		string += ("Frequency model: ") + frequencyModelToString(data) + ("\n");
+		string += ("Site Rate model: ") + siteRateModelToString(data) + ("\n");
+		string += ("Clock Rate model: ") + clockRateModelToString(data)
+				+ ("\n");
 
 		return string;
 	}// END: partitionDataToString
-	
-	public static String partitionDataListToString(PartitionDataList dataList,
+
+	public static String partitionDataListToString(PartitionDataList dataList, //
 			ArrayList<TreeModel> simulatedTreeModelList
-			) {
+	// ,LinkedHashMap<Integer,LinkedHashMap<NodeRef, int[]>>
+	// partitionSequencesMap
+	) {
 
 		String string = "";
 		TreeModel simulatedTreeModel;
-		
+		// LinkedHashMap<NodeRef, int[]> sequencesMap;
+
 		string += ("Site count: " + getSiteCount(dataList)) + ("\n");
 		if (dataList.setSeed) {
 			string += ("Starting seed: " + dataList.startingSeed) + ("\n");
@@ -969,9 +1390,12 @@ public class Utils {
 		for (PartitionData data : dataList) {
 
 			simulatedTreeModel = simulatedTreeModelList.get(row);
+			// sequencesMap = partitionSequencesMap.get(row);
 
 			string += ("Partition: " + (row + 1)) + ("\n");
-			string += partitionDataToString(data, simulatedTreeModel);
+			string += partitionDataToString(data, simulatedTreeModel
+			// , sequencesMap
+			);
 			string += ("\n");
 			row++;
 
@@ -979,12 +1403,36 @@ public class Utils {
 
 		return string;
 	}// END: partitionDataListToString
-	
+
+	// TODO: doesn't work
+	public static String annotatedTreeModelToString(TreeModel treeModel,
+			LinkedHashMap<NodeRef, int[]> sequencesMap, DataType dataType) {
+
+		StringBuffer buffer = new StringBuffer();
+		NumberFormat format = NumberFormat.getNumberInstance(Locale.ENGLISH);
+		boolean useTipLabels = true;
+
+		AncestralSequenceTrait ancestralSequence = new AncestralSequenceTrait(
+				sequencesMap, dataType);
+		TreeTraitProvider[] treeTraitProviders = new TreeTraitProvider[] { ancestralSequence };
+
+		Tree.Utils.newick(treeModel, //
+				treeModel.getRoot(), //
+				useTipLabels, //
+				Tree.BranchLengthType.LENGTHS_AS_TIME, //
+				format, //
+				null, //
+				treeTraitProviders, //
+				null, buffer);
+
+		return buffer.toString();
+	}
+
 	private static String dataTypeToString(PartitionData data) {
 		String string = PartitionData.dataTypes[data.dataTypeIndex];
 		return string;
 	}
-	
+
 	public static String demographicModelToString(PartitionData data) {
 
 		String string = PartitionData.demographicModels[data.demographicModelIndex];
@@ -998,7 +1446,7 @@ public class Utils {
 
 		return string;
 	}
-	
+
 	public static String clockRateModelToString(PartitionData data) {
 
 		String string = PartitionData.clockModels[data.clockModelIndex];
@@ -1010,6 +1458,13 @@ public class Utils {
 		}// END: indices loop
 		string += ")";
 
+		if(data.clockModelIndex == data.LRC_INDEX) {
+			
+			String space = (data.lrcParametersInRealSpace == true ? "real" : "log");
+			string += " ( " +  "Parameters in " + space + " space )";
+			
+		}
+		
 		return string;
 	}
 
@@ -1054,7 +1509,7 @@ public class Utils {
 
 		return string;
 	}
-	
+
 	public static String demographyModelToString(PartitionData data) {
 
 		String string = PartitionData.demographicModels[data.demographicModelIndex];
@@ -1068,5 +1523,6 @@ public class Utils {
 
 		return string;
 	}
-	
+
+
 }// END: class
