@@ -1,7 +1,7 @@
 /*
- * RootToTip.java
+ * TemporalRooting.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -41,8 +41,9 @@ import java.util.*;
 public class TemporalRooting {
 
     public enum RootingFunction {
+        HEURISTIC_RESIDUAL_MEAN_SQUARED("heuristic residual mean squared"),
         RESIDUAL_MEAN_SQUARED("residual mean squared"),
-//        SUM_RESIDUAL_SQUARED("sum squared residuals"),
+        //        SUM_RESIDUAL_SQUARED("sum squared residuals"),
         CORRELATION("correlation"),
         R_SQUARED("R squared");
 
@@ -61,6 +62,7 @@ public class TemporalRooting {
     private boolean contemporaneous = false;
     private final TaxonList taxa;
     private final Map<String, Double> dates;
+    private final Map<String, Double> precisions;
     private boolean useTargetRate = false;
     private double targetRate = 0.0;
     private double dateMin;
@@ -69,10 +71,13 @@ public class TemporalRooting {
     private int currentRootBranch = 0;
     private int totalRootBranches = 0;
 
+    private boolean forcePositiveRate = false;
+
     public TemporalRooting(TaxonList taxa) {
         this.taxa = taxa;
 
         dates = new HashMap<String, Double>();
+        precisions = new HashMap<String, Double>();
 
         dateMin = Double.MAX_VALUE;
         dateMax = -Double.MAX_VALUE;
@@ -83,6 +88,10 @@ public class TemporalRooting {
             double d = 0.0;
             if (date != null) {
                 d = date.getAbsoluteTimeValue();
+                if (date.getPrecision() > 0.0) {
+                    d += date.getPrecision() / 2;
+                    precisions.put(taxon.getId(), date.getPrecision());
+                }
             }
             if (d > dateMax) {
                 dateMax = d;
@@ -99,6 +108,10 @@ public class TemporalRooting {
         }
     }
 
+    public void setForcePositiveRate(boolean forcePositiveRate) {
+        this.forcePositiveRate = forcePositiveRate;
+    }
+
     public void setTargetRate(double targetRate) {
         this.targetRate = targetRate;
     }
@@ -114,7 +127,17 @@ public class TemporalRooting {
     public Tree findRoot(Tree tree, RootingFunction rootingFunction) {
 
         double[] dates = getTipDates(tree);
-        return findGlobalRoot(tree, dates, rootingFunction);
+        return findGlobalRoot(tree, dates, rootingFunction, forcePositiveRate);
+    }
+
+    public Tree findLocalRoot(Tree tree, RootingFunction rootingFunction) {
+
+        double[] dates = getTipDates(tree);
+        FlexibleTree bestTree = new FlexibleTree(tree);
+
+        findLocalRoot(bestTree, dates, rootingFunction, forcePositiveRate);
+
+        return bestTree;
     }
 
     public Regression getRootToTipRegression(Tree tree) {
@@ -218,6 +241,19 @@ public class TemporalRooting {
         return d;
     }
 
+    public double[] getTipDatePrecisions(Tree tree) {
+        double[] p = new double[tree.getExternalNodeCount()];
+        for (int i = 0; i < tree.getExternalNodeCount(); i++) {
+            NodeRef tip = tree.getExternalNode(i);
+            Double precision = precisions.get(tree.getNodeTaxon(tip).getId());
+            if (precision == null) {
+                precision = 0.0;
+            }
+            p[i] = precision;
+        }
+        return p;
+    }
+
     public String[] getTipLabels(Tree tree) {
         String[] labels = new String[tree.getExternalNodeCount()];
         for (int i = 0; i < tree.getExternalNodeCount(); i++) {
@@ -227,10 +263,10 @@ public class TemporalRooting {
         return labels;
     }
 
-    private Tree findGlobalRoot(final Tree source, final double[] dates, RootingFunction rootingFunction) {
+    private Tree findGlobalRoot(final Tree source, final double[] dates, RootingFunction rootingFunction, boolean forcePositiveRate) {
 
         FlexibleTree bestTree = new FlexibleTree(source);
-        double minF = findLocalRoot(bestTree, dates, rootingFunction);
+        double minF = findLocalRoot(bestTree, dates, rootingFunction, forcePositiveRate);
         double minDiff = Double.MAX_VALUE;
 
         totalRootBranches = source.getNodeCount();
@@ -241,7 +277,7 @@ public class TemporalRooting {
                 double length = tmpTree.getBranchLength(node);
                 tmpTree.changeRoot(node, length * 0.5, length * 0.5);
 
-                double f = findLocalRoot(tmpTree, dates, rootingFunction);
+                double f = findLocalRoot(tmpTree, dates, rootingFunction, forcePositiveRate);
                 if (useTargetRate) {
                     Regression r = getRootToTipRegression(tmpTree);
                     if (Math.abs(r.getGradient() - targetRate) < minDiff) {
@@ -260,7 +296,14 @@ public class TemporalRooting {
         return bestTree;
     }
 
-    private double findLocalRoot(final FlexibleTree tree, final double[] dates, final RootingFunction rootingFunction) {
+    private double findLocalRoot(final FlexibleTree tree,
+                                 final double[] dates,
+                                 final RootingFunction rootingFunction,
+                                 final boolean forcePositiveRate) {
+
+        if (rootingFunction == RootingFunction.RESIDUAL_MEAN_SQUARED) {
+            return findAnalyticalLocalRoot(tree, dates, rootingFunction);
+        }
 
         NodeRef node1 = tree.getChild(tree.getRoot(), 0);
         NodeRef node2 = tree.getChild(tree.getRoot(), 1);
@@ -294,6 +337,7 @@ public class TemporalRooting {
 
                 if (!contemporaneous) {
                     Regression r = new Regression(dates, y);
+
                     switch (rootingFunction) {
 
                         case CORRELATION:
@@ -302,12 +346,18 @@ public class TemporalRooting {
                         case R_SQUARED:
                             score = -r.getRSquared();
                             break;
+                        case HEURISTIC_RESIDUAL_MEAN_SQUARED:
                         case RESIDUAL_MEAN_SQUARED:
                             score = r.getResidualMeanSquared();
                             break;
                         default:
                             throw new RuntimeException("Unknown enum value");
                     }
+
+                    if (forcePositiveRate) {
+                        score = (r.getGradient() < 0.0 ? -score : score);
+                    }
+
                 } else {
                     score = DiscreteStatistics.variance(y);
                 }
@@ -339,7 +389,6 @@ public class TemporalRooting {
         double x = minimum.findMinimum(f);
 
         double fminx = minimum.fminx;
-
         double l1 = x * sumLength;
         double l2 = (1.0 - x) * sumLength;
 
@@ -349,6 +398,86 @@ public class TemporalRooting {
         return fminx;
     }
 
+    private double findAnalyticalLocalRoot(final FlexibleTree tree,
+                                           final double[] t,
+                                           final RootingFunction rootingFunction) {
+
+        if (rootingFunction != RootingFunction.RESIDUAL_MEAN_SQUARED) {
+            throw new UnsupportedOperationException("Analytical local root solution only for residual mean squared");
+        }
+
+        NodeRef node1 = tree.getChild(tree.getRoot(), 0);
+        NodeRef node2 = tree.getChild(tree.getRoot(), 1);
+
+        final double length1 = tree.getBranchLength(node1);
+        final double length2 = tree.getBranchLength(node2);
+
+        final double sumLength = length1 + length2;
+
+        final Set<NodeRef> tipSet1 = Tree.Utils.getExternalNodes(tree, node1);
+        final Set<NodeRef> tipSet2 = Tree.Utils.getExternalNodes(tree, node2);
+        
+        int N = tipSet1.size() + tipSet2.size();
+        int n = tipSet2.size();
+   
+        final double[] c = new double[N];
+        for (NodeRef tip : tipSet2) {
+            int i = tip.getNumber();
+            c[i] = 1;
+        }
+        
+        final double[] y = getRootToTipDistances(tree);
+        for (int j = 0; j < y.length; j++) { // little fiddling with the root-to-tip divergences to get the right input vector
+        	y[j] = y[j] + (1-c[j])*(sumLength-length1) - c[j]*(sumLength-length1);
+        }
+        
+        double sum_tt = 0.0;
+        double sum_t = 0.0;
+        double sum_y = 0.0;
+        double sum_ty = 0.0;
+        double sum_tc = 0.0;
+
+        for (int i = 0; i < N; i++) {
+            sum_tt += t[i] * t[i];
+            sum_t += t[i];
+            sum_y += y[i];
+            sum_ty += t[i] * y[i];
+            sum_tc += t[i] * c[i];
+        }
+        double y_bar = sum_y / N;
+        double t_bar = sum_t / N;
+
+        double C = sum_tt - (sum_t * sum_t / N);
+        double sumAB = 0.0;
+        double sumAA = 0.0;
+        double Nd = N;
+        double nd = n;  // need to set these naughty ones to doubles
+        for (int i = 0; i < N; i++) {
+            double Ai = 2*c[i] - 
+            		    ((2*nd-Nd)/Nd) +
+            		(2*(t_bar-t[i])/(C*Nd))*(Nd*sum_tc - nd*sum_t) - 1;
+            double Bi = (y[i] - y_bar)
+                    + ((t_bar - t[i]) / (C * Nd)) * ((Nd * sum_ty) - (sum_t * sum_y));
+
+            sumAB += Ai * Bi;
+            sumAA += Ai * Ai;
+        }
+
+        double x = -sumAB / (sumLength * sumAA);
+        x = Math.min(Math.max(x, 0.0), 1.0);
+
+        double l1 = (1.0 - x) * sumLength;
+        double l2 = x * sumLength;
+        
+        
+        tree.setBranchLength(node1, l1);
+        tree.setBranchLength(node2, l2);
+        
+        Regression r = new Regression(t, getRootToTipDistances(tree));
+        
+        return r.getResidualMeanSquared();
+    }
+
     public double getRootToTipDistance(Tree tree, NodeRef node) {
         double distance = 0;
         while (node != null) {
@@ -463,5 +592,4 @@ public class TemporalRooting {
         }
     }
 
-
-}
\ No newline at end of file
+}
