@@ -1,7 +1,7 @@
 /*
  * TreeAnnotator.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -68,7 +68,7 @@ public class TreeAnnotator {
 
     enum Target {
         MAX_CLADE_CREDIBILITY("Maximum clade credibility tree"),
-        MAX_SUM_CLADE_CREDIBILITY("Maximum sum of clade credibilities"),
+        //MAX_SUM_CLADE_CREDIBILITY("Maximum sum of clade credibilities"),
         USER_TARGET_TREE("User target tree");
 
         String desc;
@@ -107,7 +107,22 @@ public class TreeAnnotator {
     private final String location2Attribute = "longLat2";
     private final String locationOutputAttribute = "location";
 
-    public TreeAnnotator(final int burnin,
+    /**
+     * Burnin can be specified as the number of trees or the number of states
+     * (one or other should be zero).
+     * @param burninTrees
+     * @param burninStates
+     * @param heightsOption
+     * @param posteriorLimit
+     * @param hpd2D
+     * @param targetOption
+     * @param targetTreeFileName
+     * @param inputFileName
+     * @param outputFileName
+     * @throws IOException
+     */
+    public TreeAnnotator(final int burninTrees,
+                         final int burninStates,
                          HeightsSummary heightsOption,
                          double posteriorLimit,
                          double hpd2D,
@@ -125,6 +140,8 @@ public class TreeAnnotator {
 
         CladeSystem cladeSystem = new CladeSystem();
 
+        int burnin = -1;
+
         totalTrees = 10000;
         totalTreesUsed = 0;
 
@@ -144,7 +161,26 @@ public class TreeAnnotator {
                 while (importer.hasTree()) {
                     Tree tree = importer.importNextTree();
 
-                    if (totalTrees >= burnin) {
+                    int state = Integer.MAX_VALUE;
+
+                    if (burninStates > 0) {
+                        // if burnin has been specified in states, try to parse it out...
+                        String name = tree.getId().trim();
+
+                        if (name != null && name.length() > 0 && name.startsWith("STATE_")) {
+                            state = Integer.parseInt(name.split("_")[1]);
+                        }
+                    }
+
+                    if (totalTrees >= burninTrees && state >= burninStates) {
+                        // if either of the two burnin thresholds have been reached...
+
+                        if (burnin < 0) {
+                            // if this is the first time this point has been reached,
+                            // record the number of trees this represents for future use...
+                            burnin = totalTrees;
+                        }
+
                         cladeSystem.add(tree, false);
 
                         totalTreesUsed += 1;
@@ -178,8 +214,11 @@ public class TreeAnnotator {
             cladeSystem.calculateCladeCredibilities(totalTreesUsed);
 
             progressStream.println("Total trees read: " + totalTrees);
-            if (burnin > 0) {
-                progressStream.println("Ignoring first " + burnin + " trees.");
+            if (burninTrees > 0) {
+                progressStream.println("Ignoring first " + burninTrees + " trees" +
+                        (burninStates > 0 ? " (" + burninStates + " states)." : "." ));
+            } else if (burninStates > 0) {
+                progressStream.println("Ignoring first " + burninStates + " states (" + burnin + " trees).");
             }
 
             progressStream.println("Total unique clades: " + cladeSystem.getCladeMap().keySet().size());
@@ -217,14 +256,15 @@ public class TreeAnnotator {
             }
             case MAX_CLADE_CREDIBILITY: {
                 progressStream.println("Finding maximum credibility tree...");
-                targetTree = new FlexibleTree(summarizeTrees(burnin, cladeSystem, inputFileName, false));
-                break;
-            }
-            case MAX_SUM_CLADE_CREDIBILITY: {
-                progressStream.println("Finding maximum sum clade credibility tree...");
-                targetTree = new FlexibleTree(summarizeTrees(burnin, cladeSystem, inputFileName, true));
+                targetTree = new FlexibleTree(summarizeTrees(burnin, cladeSystem, inputFileName /*, false*/));
                 break;
             }
+//            case MAX_SUM_CLADE_CREDIBILITY: {
+//                progressStream.println("Finding maximum sum clade credibility tree...");
+//                targetTree = new FlexibleTree(summarizeTrees(burnin, cladeSystem, inputFileName, true));
+//                break;
+//            }
+            default: throw new IllegalArgumentException("Unknown targetOption");
         }
 
         progressStream.println("Collecting node information...");
@@ -320,8 +360,8 @@ public class TreeAnnotator {
         }
     }
 
-    private Tree summarizeTrees(int burnin, CladeSystem cladeSystem, String inputFileName,
-                                boolean useSumCladeCredibility) throws IOException {
+    private Tree summarizeTrees(int burnin, CladeSystem cladeSystem, String inputFileName /*, boolean useSumCladeCredibility */)
+            throws IOException {
 
         Tree bestTree = null;
         double bestScore = Double.NEGATIVE_INFINITY;
@@ -341,7 +381,7 @@ public class TreeAnnotator {
                 Tree tree = importer.importNextTree();
 
                 if (counter >= burnin) {
-                    double score = scoreTree(tree, cladeSystem, useSumCladeCredibility);
+                    double score = scoreTree(tree, cladeSystem /*, useSumCladeCredibility*/);
 //                    progressStream.println(score);
                     if (score > bestScore) {
                         bestTree = tree;
@@ -362,21 +402,21 @@ public class TreeAnnotator {
         progressStream.println();
         progressStream.println();
         progressStream.println("Best tree: " + bestTree.getId() + " (tree number " + bestTreeNumber + ")");
-        if (useSumCladeCredibility) {
-            progressStream.println("Highest Sum Clade Credibility: " + bestScore);
-        } else {
+//        if (useSumCladeCredibility) {
+//            progressStream.println("Highest Sum Clade Credibility: " + bestScore);
+//        } else {
             progressStream.println("Highest Log Clade Credibility: " + bestScore);
-        }
+//        }
 
         return bestTree;
     }
 
-    private double scoreTree(Tree tree, CladeSystem cladeSystem, boolean useSumCladeCredibility) {
-        if (useSumCladeCredibility) {
-            return cladeSystem.getSumCladeCredibility(tree, tree.getRoot(), null);
-        } else {
+    private double scoreTree(Tree tree, CladeSystem cladeSystem /*, boolean useSumCladeCredibility*/) {
+//        if (useSumCladeCredibility) {
+//            return cladeSystem.getSumCladeCredibility(tree, tree.getRoot(), null);
+//        } else {
             return cladeSystem.getLogCladeCredibility(tree, tree.getRoot(), null);
-        }
+//        }
     }
 
     private class CladeSystem {
@@ -551,33 +591,33 @@ public class TreeAnnotator {
             }
         }
 
-        public double getSumCladeCredibility(Tree tree, NodeRef node, BitSet bits) {
-
-            double sum = 0.0;
-
-            if (tree.isExternal(node)) {
-
-                int index = taxonList.getTaxonIndex(tree.getNodeTaxon(node).getId());
-                bits.set(index);
-            } else {
-
-                BitSet bits2 = new BitSet();
-                for (int i = 0; i < tree.getChildCount(node); i++) {
-
-                    NodeRef node1 = tree.getChild(node, i);
-
-                    sum += getSumCladeCredibility(tree, node1, bits2);
-                }
-
-                sum += getCladeCredibility(bits2);
-
-                if (bits != null) {
-                    bits.or(bits2);
-                }
-            }
-
-            return sum;
-        }
+//        public double getSumCladeCredibility(Tree tree, NodeRef node, BitSet bits) {
+//
+//            double sum = 0.0;
+//
+//            if (tree.isExternal(node)) {
+//
+//                int index = taxonList.getTaxonIndex(tree.getNodeTaxon(node).getId());
+//                bits.set(index);
+//            } else {
+//
+//                BitSet bits2 = new BitSet();
+//                for (int i = 0; i < tree.getChildCount(node); i++) {
+//
+//                    NodeRef node1 = tree.getChild(node, i);
+//
+//                    sum += getSumCladeCredibility(tree, node1, bits2);
+//                }
+//
+//                sum += getCladeCredibility(bits2);
+//
+//                if (bits != null) {
+//                    bits.or(bits2);
+//                }
+//            }
+//
+//            return sum;
+//        }
 
         public double getLogCladeCredibility(Tree tree, NodeRef node, BitSet bits) {
 
@@ -1279,7 +1319,8 @@ public class TreeAnnotator {
                 return;
             }
 
-            int burnin = dialog.getBurnin();
+            int burninStates = dialog.getBurninStates();
+            int burninTrees = dialog.getBurninTrees();
             double posteriorLimit = dialog.getPosteriorLimit();
             double hpd2D = 0.80;
             Target targetOption = dialog.getTargetOption();
@@ -1304,7 +1345,9 @@ public class TreeAnnotator {
             }
 
             try {
-                new TreeAnnotator(burnin,
+                new TreeAnnotator(
+                        burninTrees,
+                        burninStates,
                         heightsOption,
                         posteriorLimit,
                         hpd2D,
@@ -1335,6 +1378,7 @@ public class TreeAnnotator {
                         new Arguments.StringOption("heights", new String[]{"keep", "median", "mean", "ca"}, false,
                                 "an option of 'keep' (default), 'median', 'mean' or 'ca'"),
                         new Arguments.IntegerOption("burnin", "the number of states to be considered as 'burn-in'"),
+                        new Arguments.IntegerOption("burninTrees", "the number of trees to be considered as 'burn-in'"),
                         new Arguments.RealOption("limit", "the minimum posterior probability for a node to be annotated"),
                         new Arguments.StringOption("target", "target_file_name", "specifies a user target tree to be annotated"),
                         new Arguments.Option("help", "option to print this message"),
@@ -1374,9 +1418,13 @@ public class TreeAnnotator {
             }
         }
 
-        int burnin = -1;
+        int burninStates = -1;
+        int burninTrees = -1;
         if (arguments.hasOption("burnin")) {
-            burnin = arguments.getIntegerOption("burnin");
+            burninStates = arguments.getIntegerOption("burnin");
+        }
+        if (arguments.hasOption("burninTrees")) {
+            burninTrees = arguments.getIntegerOption("burninTrees");
         }
 
         double posteriorLimit = 0.0;
@@ -1412,7 +1460,7 @@ public class TreeAnnotator {
             }
         }
 
-        new TreeAnnotator(burnin, heights, posteriorLimit, hpd2D, target, targetTreeFileName, inputFileName, outputFileName);
+        new TreeAnnotator(burninTrees, burninStates, heights, posteriorLimit, hpd2D, target, targetTreeFileName, inputFileName, outputFileName);
 
         System.exit(0);
     }
