@@ -1,7 +1,7 @@
 /*
  * CombinedTraces.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -138,7 +138,7 @@ public class CombinedTraces extends FilteredTraceList { //implements TraceList {
     /**
      * @return the last state in the chain
      */
-    public int getMaxState() {
+    public long getMaxState() {
         return getStateCount() * getStepSize();
     }
 
