@@ -1,7 +1,7 @@
 /*
  * Patterns.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -114,7 +114,25 @@ public class Patterns implements PatternList {
      */
     public Patterns(SiteList siteList, int from, int to, int every, int subSet, int subSetCount) {
         addPatterns(siteList, from, to, every);
+        subSetPatterns(subSet, subSetCount);
+    }
 
+    /**
+     * Constructor
+     */
+    public Patterns(PatternList patternList) {
+        addPatterns(patternList);
+    }
+
+    /**
+     * Constructor
+     */
+    public Patterns(PatternList patternList, int subSet, int subSetCount) {
+        addPatterns(patternList);
+        subSetPatterns(subSet, subSetCount);
+    }
+
+    private void subSetPatterns(int subSet, int subSetCount) {
         if (subSetCount > 0) {
             // if we are using subSetCount then cut it down to only the subset we want...
             int div = patternCount / subSetCount;
@@ -143,13 +161,6 @@ public class Patterns implements PatternList {
         }
     }
 
-    /**
-     * Constructor
-     */
-    public Patterns(PatternList patternList) {
-        addPatterns(patternList);
-    }
-
     /**
      * adds patterns to the list from a SiteList
      */
