@@ -1,3 +1,28 @@
+/*
+ * MultidimensionalScalingLikelihood.java
+ *
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.antigenic;
 
 import dr.inference.model.*;
@@ -168,11 +193,39 @@ public class MultidimensionalScalingLikelihood extends AbstractModelLikelihood {
     }
 
     protected void setupLocationsParameter(MatrixParameter locationsParameter) {
-        locationsParameter.setColumnDimension(mdsDimension);
-        locationsParameter.setRowDimension(locationCount);
+        if (locationsParameter.getColumnDimension() > 0){
+            boolean proceed = true;
+            //TODO: allow for a generic tipTrait/location specification with mismatches: tipTraits which are not represented by the pairwise distances (tipTraits > locations) and more locations based on pairwise distances than represented by the tipTraits
+            if (locationsParameter.getColumnDimension() != locationCount){
+                System.err.println("locationsParameter column dimension ("+locationsParameter.getColumnDimension()+") is not equal to the locationCount ("+locationCount+")");
+                proceed = false;
+            }
+            if (locationsParameter.getRowDimension() != mdsDimension){
+                System.err.println("locationsParameter row dimension ("+locationsParameter.getRowDimension()+") is not equal to the mdsDimension ("+mdsDimension+")");
+                proceed = false;
+            }
+            if (!proceed) {
+                System.exit(-1);
+            }
+        } else{
+            locationsParameter.setColumnDimension(mdsDimension);
+            locationsParameter.setRowDimension(locationCount);
+        }
         for (int i = 0; i < locationLabels.length; i++) {
             locationsParameter.getParameter(i).setId(locationLabels[i]);
         }
+
+        for (int i = 0; i < locationsParameter.getParameterCount(); ++i) {
+            Parameter param = locationsParameter.getParameter(i);
+            try {
+                if (param.getBounds() != null) {
+                    // Do nothing
+                }
+            } catch (NullPointerException exception) {
+                param.addBounds(new Parameter.DefaultBounds(
+                        Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, param.getDimension()));
+            }
+        }
     }
 
     @Override
