@@ -1,7 +1,7 @@
 /*
  * IntegratedMultivariateTraitLikelihood.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -36,6 +36,7 @@ import dr.inference.model.Parameter;
 import dr.inference.model.Variable;
 import dr.math.distributions.MultivariateNormalDistribution;
 import dr.math.distributions.WishartSufficientStatistics;
+import dr.math.matrixAlgebra.IllegalDimension;
 import dr.math.matrixAlgebra.Matrix;
 import dr.math.matrixAlgebra.SymmetricMatrix;
 import dr.math.matrixAlgebra.Vector;
@@ -135,6 +136,58 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     }
 
+    public IntegratedMultivariateTraitLikelihood(String traitName,
+                                                 MultivariateTraitTree treeModel,
+                                                 MultivariateDiffusionModel diffusionModel,
+                                                 CompoundParameter traitParameter,
+                                                 Parameter deltaParameter,
+                                                 List<Integer> missingIndices,
+                                                 boolean cacheBranches, boolean scaleByTime, boolean useTreeLength,
+                                                 BranchRateModel rateModel,
+                                                 List<BranchRateModel> optimalValues,
+                                                 BranchRateModel strengthOfSelection,
+                                                 Model samplingDensity,
+                                                 boolean reportAsMultivariate,
+                                                 boolean reciprocalRates) {
+
+        super(traitName, treeModel, diffusionModel, traitParameter, deltaParameter, missingIndices, cacheBranches, scaleByTime,
+                useTreeLength, rateModel, optimalValues, strengthOfSelection, samplingDensity, reportAsMultivariate, reciprocalRates);
+
+        // Delegate caches to helper
+        meanCache = new double[dim * treeModel.getNodeCount()];
+
+        if (optimalValues != null) {
+            cacheHelper = new OUCacheHelper(dim * treeModel.getNodeCount(), cacheBranches);
+        } else {
+            cacheHelper = new CacheHelper(dim * treeModel.getNodeCount(), cacheBranches);
+        }
+
+
+        drawnStates = new double[dim * treeModel.getNodeCount()];
+        upperPrecisionCache = new double[treeModel.getNodeCount()];
+        lowerPrecisionCache = new double[treeModel.getNodeCount()];
+        logRemainderDensityCache = new double[treeModel.getNodeCount()];
+
+        if (cacheBranches) {
+            storedMeanCache = new double[dim * treeModel.getNodeCount()];
+            storedUpperPrecisionCache = new double[treeModel.getNodeCount()];
+            storedLowerPrecisionCache = new double[treeModel.getNodeCount()];
+            storedLogRemainderDensityCache = new double[treeModel.getNodeCount()];
+        }
+
+        // Set up reusable temporary storage
+        Ay = new double[dimTrait];
+        tmpM = new double[dimTrait][dimTrait];
+        tmp2 = new double[dimTrait];
+
+        zeroDimVector = new double[dim];
+
+        missingTraits = new MissingTraits.CompletelyMissing(treeModel, missingIndices, dim);
+        setTipDataValuesForAllNodes();
+
+    }
+
+
     private void setTipDataValuesForAllNodes() {
         for (int i = 0; i < treeModel.getExternalNodeCount(); i++) {
             NodeRef node = treeModel.getExternalNode(i);
@@ -187,12 +240,25 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         citations.add(
                 new Citation(
                         new Author[]{
-                                new Author("O", "Pybus"),
+                                new Author("OG", "Pybus"),
+                                new Author("MA", "Suchard"),
                                 new Author("P", "Lemey"),
+                                new Author("F", "Bernadin"),
                                 new Author("A", "Rambaut"),
-                                new Author("MA", "Suchard")
+                                new Author("FW", "Crawford"),
+                                new Author("RR", "Gray"),
+                                new Author("N", "Arinaminpathy"),
+                                new Author("S", "Stramer"),
+                                new Author("MP", "Busch"),
+                                new Author("E", "Delwart")
+
                         },
-                        Citation.Status.IN_PREPARATION
+                        "Unifying the spatial epidemiology and evolution of emerging epidemics",
+                        2012,
+                        "Proceedings of the National Academy of Sciences",
+                        109,
+                        15066, 15071,
+                        Citation.Status.PUBLISHED
                 )
         );
         return citations;
@@ -214,11 +280,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         final boolean computeWishartStatistics = getComputeWishartSufficientStatistics();
 
         if (computeWishartStatistics) {
-//            if (wishartStatistics == null) {
             wishartStatistics = new WishartSufficientStatistics(dimTrait);
-//            } else {
-//                wishartStatistics.clear();
-//            }
         }
 
         // Use dynamic programming to compute conditional likelihoods at each internal node
@@ -308,6 +370,16 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         if (DEBUG_PNAS) {
             checkLogLikelihood(logLikelihood, sumLogRemainders(), conditionalRootMean,
                     conditionalRootPrecision, traitPrecision);
+
+            // Check log remainder densities
+
+            for (int i = 0; i < logRemainderDensityCache.length; ++i) {
+                if (logRemainderDensityCache[i] < -1E10) {
+                    System.err.println(logRemainderDensityCache[i] + " @ " + i);
+                }
+
+            }
+
         }
 
         areStatesRedrawn = false;  // Should redraw internal node states when needed
@@ -325,7 +397,14 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             if (index > dimTrait * treeModel.getExternalNodeCount()) {
                 throw new RuntimeException("Attempting to update an invalid index");
             }
-            cacheHelper.setMeanCache(index, traitParameter.getValue(index));
+
+            if (index != -1) {
+                cacheHelper.setMeanCache(index, traitParameter.getValue(index));
+            } else {
+                for (int idx = 0; idx < traitParameter.getDimension(); ++idx) {
+                    cacheHelper.setMeanCache(idx, traitParameter.getValue(idx));
+                }
+            }
             // meanCache[index] = traitParameter.getValue(index);
             likelihoodKnown = false;
 //            if (!cacheBranches) {
@@ -382,7 +461,9 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                 upperPrecisionCache[thisNumber] = 0;
                 lowerPrecisionCache[thisNumber] = 0; // Needed in the pre-order traversal
             } else { // not missing tip trait
-                upperPrecisionCache[thisNumber] = 1.0 / getRescaledBranchLengthForPrecision(node);
+                // changeou
+                //    upperPrecisionCache[thisNumber] = (1.0 / getRescaledBranchLengthForPrecision(node)) * Math.pow(cacheHelper.getOUFactor(node), 2);
+                upperPrecisionCache[thisNumber] = cacheHelper.getUpperPrecFactor(node) * Math.pow(cacheHelper.getOUFactor(node), 2);
                 lowerPrecisionCache[thisNumber] = Double.POSITIVE_INFINITY;
             }
             return;
@@ -408,7 +489,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         // Multiply child0 and child1 densities
 
-        // Delegate this!
+        // changeou
         cacheHelper.computeMeanCaches(meanThisOffset, meanOffset0, meanOffset1,
                 totalPrecision, precision0, precision1, missingTraits, node, childNode0, childNode1);
 //        if (totalPrecision == 0) {
@@ -426,11 +507,14 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         if (!treeModel.isRoot(node)) {
             // Integrate out trait value at this node
-            double thisPrecision = 1.0 / getRescaledBranchLengthForPrecision(node);
+            //changeou
+            //  double thisPrecision = 1.0 / getRescaledBranchLengthForPrecision(node);
+            double thisPrecision = cacheHelper.getUpperPrecFactor(node);
             if (Double.isInfinite(thisPrecision)) {
+                // must handle this case for ouprocess
                 upperPrecisionCache[thisNumber] = totalPrecision;
             } else {
-                upperPrecisionCache[thisNumber] = totalPrecision * thisPrecision / (totalPrecision + thisPrecision);
+                upperPrecisionCache[thisNumber] = (totalPrecision * thisPrecision / (totalPrecision + thisPrecision)) * Math.pow(cacheHelper.getOUFactor(node), 2);
             }
         }
 
@@ -439,7 +523,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         logRemainderDensityCache[thisNumber] = 0;
 
         if (precision0 != 0 && precision1 != 0) {
-
+            // changeou
             incrementRemainderDensities(
                     precisionMatrix,
                     logDetPrecisionMatrix, thisNumber, meanThisOffset,
@@ -447,6 +531,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                     meanOffset1,
                     precision0,
                     precision1,
+                    cacheHelper.getOUFactor(childNode0),
+                    cacheHelper.getOUFactor(childNode1),
                     cacheOuterProducts);
         }
     }
@@ -459,6 +545,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                                              int childOffset1,
                                              double precision0,
                                              double precision1,
+                                             double OUFactor0,
+                                             double OUFactor1,
                                              boolean cacheOuterProducts) {
 
         final double remainderPrecision = precision0 * precision1 / (precision0 + precision1);
@@ -502,8 +590,43 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             logRemainderDensityCache[thisIndex] +=
                     -dimTrait * LOG_SQRT_2_PI
                             + 0.5 * (dimTrait * Math.log(remainderPrecision) + logDetPrecisionMatrix)
-                            - 0.5 * (childSS0 + childSS1 - crossSS);
+                            - 0.5 * (childSS0 + childSS1 - crossSS)
+                            // changeou
+                            - dimTrait * (Math.log(OUFactor0) + Math.log(OUFactor1));
+
+
+            if (logRemainderDensityCache[thisIndex] > 1E2) {
+                System.err.println(thisIndex);
+                System.err.println(logRemainderDensityCache[thisIndex]);
+                System.err.println("rP = " + remainderPrecision);
+                System.err.println("p0 = " + precision0);
+                System.err.println("p1 = " + precision1 + "\n");
+                System.err.println(new Matrix(precisionMatrix));
+                System.err.println(childSS0);
+                System.err.println(childSS1);
+                System.err.println(crossSS);
+
+
+                for (int i = 0; i < dimTrait; ++i) {
+                    System.err.println("\t"
+                                    + cacheHelper.getCorrectedMeanCache()[childOffset0 + 0 * dimTrait + i] + " "
+                                    + cacheHelper.getCorrectedMeanCache()[childOffset1 + 0 * dimTrait + i]
+                    );
+                }
+                System.exit(-1);
+            }
+            //               double tempnum = childSS0 + childSS1 - crossSS;
+            //   System.err.println("childSS0 + childSS1 - crossSS:  " + tempnum);
         }
+        //     System.err.println("logRemainderDensity: " + logRemainderDensityCache[thisIndex]);
+        //    System.err.println("thisIndex: " + thisIndex);
+        //   System.err.println("remainder precision: " + remainderPrecision);
+        // System.err.println("precision0: " + precision0);
+        // System.err.println("precision1: " + precision1);
+        // System.err.println("precision0*precision1: " + precision0*precision1);
+
+        //    System.err.println("logDetPrecisionMatrix: " + logDetPrecisionMatrix);
+
     }
 
     private void incrementOuterProducts(int thisOffset,
@@ -514,33 +637,176 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         final double[][] outerProduct = wishartStatistics.getScaleMatrix();
 
+//        for (int k = 0; k < numData; k++) {
+//
+//            for (int i = 0; i < dimTrait; i++) {
+//
+//                // final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
+//                // final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
+//                final double wChild0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i] * precision0;
+//                final double wChild1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i] * precision1;
+//
+//                for (int j = 0; j < dimTrait; j++) {
+//
+//                    //final double child0j = meanCache[childOffset0 + k * dimTrait + j];
+//                    //final double child1j = meanCache[childOffset1 + k * dimTrait + j];
+//                    final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
+//                    final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
+//
+////                    outerProduct[i][j] += wChild0i * child0j;
+////                    outerProduct[i][j] += wChild1i * child1j;
+////
+////                    //outerProduct[i][j] -= (wChild0i + wChild1i) * meanCache[thisOffset + k * dimTrait + j];
+////                    outerProduct[i][j] -= (wChild0i + wChild1i) * cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
+//                    outerProduct[i][j] += wChild0i * child0j;
+//                    outerProduct[i][j] += wChild1i * child1j;
+//
+//                    //outerProduct[i][j] -= (wChild0i + wChild1i) * meanCache[thisOffset + k * dimTrait + j];
+//                    outerProduct[i][j] -= (wChild0i + wChild1i) * cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
+//                }
+//            }
+//        }
+
+//        final double[][] increment = new double[dimTrait][dimTrait];
+//
+//         for (int k = 0; k < numData; k++) {
+//
+//            for (int i = 0; i < dimTrait; i++) {
+//
+//                final double child0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i];
+//                final double child1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i];
+//                final double nodei = cacheHelper.getMeanCache()[thisOffset + k * dimTrait + i];
+//
+//                for (int j = 0; j < dimTrait; j++) {
+//                    final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
+//                    final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
+//                    final double nodej = cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
+//
+//                    final double inc =
+//                            (child0i * child0j - nodei * nodej) * precision0 +
+//                                    (child1i * child1j - nodei * nodej) * precision1;
+//
+//                    increment[i][j] += inc;
+////                    outerProduct[i][j] += inc;
+//                }
+//            }
+//        }
+
+//        final double[][] increment2 = new double[dimTrait][dimTrait];
+//
+//
+//        for (int k = 0; k < numData; k++) {
+//
+//            for (int i = 0; i < dimTrait; i++) {
+//
+//                final double child0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i];
+//                final double child1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i];
+////               final double nodei = cacheHelper.getMeanCache()[thisOffset + k * dimTrait + i];
+//
+//                for (int j = 0; j < dimTrait; j++) {
+//                    final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
+//                    final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
+////                   final double nodej = cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
+//
+//                    final double inc = (child0i * child0j + child1i * child1j
+//                            - child0i * child1j -  child1i * child0j) * weight;
+//
+//                    increment2[i][j] += inc;
+//                }
+//            }
+//        }
+//
+//        final double[][] increment3 = new double[dimTrait][dimTrait];
+
+        if (precision0 == 0.0 || precision1 == 0.0) {
+            System.err.println("ZERO PRECISION");
+//            System.exit(-1);
+        }
+
+        if (precision0 < 1E-16 || precision1 < 1E-16) {
+            System.err.println("LOW PRECISION");
+        }
+
+        final double weight = precision0 * precision1 / (precision0 + precision1);
+
         for (int k = 0; k < numData; k++) {
 
             for (int i = 0; i < dimTrait; i++) {
 
-                // final double wChild0i = meanCache[childOffset0 + k * dimTrait + i] * precision0;
-                // final double wChild1i = meanCache[childOffset1 + k * dimTrait + i] * precision1;
-                final double wChild0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i] * precision0;
-                final double wChild1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i] * precision1;
+                final double child0i = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + i];
+                final double child1i = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + i];
 
                 for (int j = 0; j < dimTrait; j++) {
-
-                    //final double child0j = meanCache[childOffset0 + k * dimTrait + j];
-                    //final double child1j = meanCache[childOffset1 + k * dimTrait + j];
                     final double child0j = cacheHelper.getCorrectedMeanCache()[childOffset0 + k * dimTrait + j];
                     final double child1j = cacheHelper.getCorrectedMeanCache()[childOffset1 + k * dimTrait + j];
 
-                    outerProduct[i][j] += wChild0i * child0j;
-                    outerProduct[i][j] += wChild1i * child1j;
+                    final double inc = (child0i - child1i) * (child0j - child1j) * weight;
 
-                    //outerProduct[i][j] -= (wChild0i + wChild1i) * meanCache[thisOffset + k * dimTrait + j];
-                    outerProduct[i][j] -= (wChild0i + wChild1i) * cacheHelper.getMeanCache()[thisOffset + k * dimTrait + j];
+                    outerProduct[i][j] += inc;
+//                    increment3[i][j] += inc;
                 }
             }
         }
+
+
+//        double det = increment3[0][0] * increment3[1][1] - increment3[0][1] * increment3[1][0];
+//
+//        System.err.println("INC Check\n"
+//                + new Matrix(increment) + "\n"
+//                + new Matrix(increment2) + "\n"
+//                + new Matrix(increment3) +"\n"
+//                + det + "\n\n");
+
+
+
+        // For numerical stability: Ensure increment is positive-definite
+//        if (checkIsPositiveDefinite(increment)) {
+//            for (int i = 0; i < dimTrait; ++i) {
+//                for (int j = 0; j < dimTrait; ++j) {
+//                    outerProduct[i][j] += increment3[i][j];
+//                }
+//            }
+//        }
+//        else {
+//
+//            System.err.println("ERROR INC\n" + new Matrix(increment) + "\n" + new Matrix(increment3) + "\n\n");
+////            System.exit(-1);
+//
+//        }
+
         wishartStatistics.incrementDf(1); // Peeled one node
     }
 
+//    private boolean checkIsPositiveDefinite(double[][] S) {
+//
+//        final int dim = S.length;
+//
+//        for (int i = 0; i < dim; ++i) {
+//            if (S[i][i] < 0.0) return false;
+//        }
+//
+//        if (dim == 2) {
+//            return (S[0][0] * S[1][1] > S[0][1] * S[1][0]);
+//        } else {
+//            try {
+//                return (new Matrix(S).isPD());
+//            } catch (IllegalDimension illegalDimension) {
+//                illegalDimension.printStackTrace();
+//            }
+//        }
+//        return false;
+//    }
+
+//    private boolean checkDiagonals(double[][] S) {
+//        for (int i = 0; i < S.length; ++i) {
+//            if (S[i][i] < 0.0) {
+//                System.err.println("IMTL ERROR diag(S)\n" + new Matrix(S));
+//                return false;
+//            }
+//        }
+//        return true;
+//    }
+
 //    private void computeWeightedMeanCache(int thisOffset,
 //                                          int childOffset0,
 //                                          int childOffset1,
@@ -648,9 +914,10 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                                               double[] out2, int offset2,
                                               int length) {
 
-        final double totalInverseWeight = 1.0 / (weight0 + weight1);
+//        final double totalInverseWeight = 1.0 / (weight0 + weight1);
         for (int i = 0; i < length; i++) {
-            out2[offset2 + i] = (in0[offset0 + i] * weight0 + in1[offset1 + i] * weight1) * totalInverseWeight;
+//            out2[offset2 + i] = (in0[offset0 + i] * weight0 + in1[offset1 + i] * weight1) * totalInverseWeight;
+            out2[offset2 + i] = (in0[offset0 + i] * weight0 + in1[offset1 + i] * weight1) / (weight0 + weight1);
         }
     }
 
@@ -699,6 +966,82 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     }
 
 
+    protected void computeCorrectedOUWeightedAverage(int offset0, double weight0, NodeRef childNode0,
+                                                     int offset1, double weight1, NodeRef childNode1,
+                                                     int offset2,
+                                                     int length, NodeRef thisNode) {
+
+        final double totalInverseWeight = 1.0 / (weight0 + weight1);
+
+        double[] optimal;
+        double selection;
+
+        if (!treeModel.isRoot(thisNode)) {
+            optimal = getOptimalValue(thisNode);
+            selection = getTimeScaledSelection(thisNode);
+        } else {
+            optimal = null;
+            selection = 1;
+        }
+        double[] optimalChild0 = getOptimalValue(childNode0);
+        double[] optimalChild1 = getOptimalValue(childNode1);
+        double selectionChild0 = getTimeScaledSelection(childNode0);
+        double selectionChild1 = getTimeScaledSelection(childNode1);
+
+        /*
+
+        if (treeModel.isExternal(childNode0)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset0 + i] = (meanCache[offset0 + i] - selectionChild0 * optimalChild0[i]) / (1 - selectionChild0);
+            }
+        }
+
+        if (treeModel.isExternal(childNode1)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset1 + i] = (meanCache[offset1 + i] - selectionChild1 * optimalChild1[i]) / (1 - selectionChild1);
+            }
+        }
+
+        for (int i = 0; i < length; i++) {
+
+            // meanCache[offset2 + i] = ((meanCache[offset0 + i] -  shiftChild0[i]) * weight0 + (meanCache[offset1 + i] - shiftChild1[i]) * weight1) * totalInverseWeight;
+            meanCache[offset2 + i] = (correctedMeanCache[offset0 + i] * weight0 + correctedMeanCache[offset1 + i] * weight1) * totalInverseWeight;
+            if (!treeModel.isRoot(thisNode)) {
+                correctedMeanCache[offset2 + i] = (meanCache[offset2 + i] - selection * optimal[i]) / (1 - selection);
+            } else {
+                correctedMeanCache[offset2 + i] = meanCache[offset2 + i];
+            }
+        }
+
+        */
+
+        if (treeModel.isExternal(childNode0)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset0 + i] = Math.exp(selectionChild0) * meanCache[offset0 + i] - (Math.exp(selectionChild0) - 1) * optimalChild0[i];
+            }
+        }
+
+        if (treeModel.isExternal(childNode1)) {
+            for (int i = 0; i < length; i++) {
+                correctedMeanCache[offset1 + i] = Math.exp(selectionChild1) * meanCache[offset1 + i] - (Math.exp(selectionChild1) - 1) * optimalChild1[i];
+            }
+        }
+
+        for (int i = 0; i < length; i++) {
+
+            // meanCache[offset2 + i] = ((meanCache[offset0 + i] -  shiftChild0[i]) * weight0 + (meanCache[offset1 + i] - shiftChild1[i]) * weight1) * totalInverseWeight;
+            meanCache[offset2 + i] = (correctedMeanCache[offset0 + i] * weight0 + correctedMeanCache[offset1 + i] * weight1) * totalInverseWeight;
+            if (!treeModel.isRoot(thisNode)) {
+                correctedMeanCache[offset2 + i] = Math.exp(selection) * meanCache[offset2 + i] - (Math.exp(selection) - 1) * optimal[i];
+            } else {
+                correctedMeanCache[offset2 + i] = meanCache[offset2 + i];
+            }
+        }
+
+
+    }
+
+
     protected abstract double[][] computeMarginalRootMeanAndVariance(double[] conditionalRootMean,
                                                                      double[][] treePrecisionMatrix,
                                                                      double[][] treeVarianceMatrix,
@@ -707,7 +1050,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     private void preOrderTraverseSample(MultivariateTraitTree treeModel, NodeRef node, int parentIndex, double[][] treePrecision,
                                         double[][] treeVariance) {
-
+        //  System.err.println("preOrderTraverseSample got called!!");
+        //  System.exit(-1);
         final int thisIndex = node.getNumber();
 
         if (treeModel.isRoot(node)) {
@@ -731,7 +1075,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                 }
 
                 System.arraycopy(draw, 0, drawnStates, rootIndex * dim + datum * dimTrait, dimTrait);
-
+                //                  DEBUG=true;
                 if (DEBUG) {
                     System.err.println("Root mean: " + new Vector(rootMean));
                     System.err.println("Root var : " + new Matrix(variance));
@@ -745,9 +1089,13 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
                 //System.arraycopy(meanCache, thisIndex * dim, drawnStates, thisIndex * dim, dim);
                 System.arraycopy(cacheHelper.getMeanCache(), thisIndex * dim, drawnStates, thisIndex * dim, dim);
-
+                //  System.err.println("I got here");
+                //  System.exit(-1);
             } else {
 
+                //        System.err.println("I got here");
+                //     System.exit(-1);
+
                 if (missingTraits.isPartiallyMissing(thisIndex)) {
                     throw new RuntimeException("Partially missing values are not yet implemented");
                 }
@@ -765,7 +1113,7 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
                     int parentOffset = parentIndex * dim + datum * dimTrait;
                     int thisOffset = thisIndex * dim + datum * dimTrait;
-
+                    //                   DEBUG=true;
                     if (DEBUG) {
                         double[] parentValue = new double[dimTrait];
                         System.arraycopy(drawnStates, parentOffset, parentValue, 0, dimTrait);
@@ -776,7 +1124,8 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
                     }
 
                     for (int i = 0; i < dimTrait; i++) {
-                        mean[i] = (drawnStates[parentOffset + i] * precisionToParent
+                        mean[i] = ((drawnStates[parentOffset + i] + cacheHelper.getShift(node)[i]) * precisionToParent
+                                //mean[i] = (drawnStates[parentOffset + i] * precisionToParent
                                 //  + meanCache[thisOffset + i] * precisionOfNode) / totalPrecision;
                                 + cacheHelper.getMeanCache()[thisOffset + i] * precisionOfNode) / totalPrecision;
                         for (int j = 0; j < dimTrait; j++) {
@@ -803,6 +1152,19 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
         }
     }
 
+    protected void handleModelChangedEvent(Model model, Object object, int index) {
+
+        if (driftModels != null && driftModels.contains(model)) {
+            if (cacheBranches) {
+                updateAllNodes();
+            } else {
+                likelihoodKnown = false;
+            }
+        } else {
+            super.handleModelChangedEvent(model, object, index);
+        }
+    }
+
     protected boolean peel() {
         return true;
     }
@@ -831,6 +1193,15 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
         }
 
+        public double[] getShift(NodeRef node) {
+
+            double[] shift = new double[dimTrait];
+            for (int i = 0; i < dim; ++i) {
+                shift[i] = 0;
+            }
+            return shift;
+        }
+
         public double[] getMeanCache() {
             return meanCache;
         }
@@ -853,8 +1224,12 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             // }
         }
 
-        public void doFunkyStuff(NodeRef node) {
+        public double getOUFactor(NodeRef node) {
+            return 1;
+        }
 
+        public double getUpperPrecFactor(NodeRef node) {
+            return 1.0 / getRescaledBranchLengthForPrecision(node);
         }
 
         protected boolean cacheBranches;
@@ -901,8 +1276,42 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
 
     }
 
-    ;
+    class StandarizedCacheHelper extends CacheHelper {
 
+        public StandarizedCacheHelper(int dim, int nodeCount, boolean cacheBranches) {
+            super(dim * nodeCount, cacheBranches);
+            this.dim = dim;
+            this.nodeCount = nodeCount;
+
+
+        }
+
+        public void setTipMeans(double[] traitValue, int dim, int index, NodeRef node) {
+            //System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+            for (int i = 0; i < dim; ++i) {
+                setMeanCache(dim * index + i, traitValue[i]);
+            }
+        }
+
+        public void setTipMeans(double[] traitValue, int dim, int index) {
+//            System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+            for (int i = 0; i < dim; ++i) {
+                setMeanCache(dim * index + i, traitValue[i]);
+            }
+        }
+
+//        public void copyToMeanCache(double[] src, int destPos, int length) {
+//            System.arraycopy(src, 0, meanCache, destPos, length);
+//        }
+
+        public void setMeanCache(int index, double value) {
+            final int thisDim = index % dim;
+            meanCache[index] = value;
+        }
+
+        private final int dim;
+        private final int nodeCount;
+    }
 
     class DriftCacheHelper extends CacheHelper {
 
@@ -911,12 +1320,21 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
             correctedMeanCache = new double[cacheLength];
         }
 
+        public double[] getShift(NodeRef node) {
+            return getShiftForBranchLength(node);
+        }
+
         public double[] getCorrectedMeanCache() {
             return correctedMeanCache;
         }
 
-        public void doFunkyStuff(NodeRef node) {
-            getShiftForBranchLength(node);
+
+        public double getOUFactor(NodeRef node) {
+            return 1;
+        }
+
+        public double getUpperPrecFactor(NodeRef node) {
+            return 1.0 / getRescaledBranchLengthForPrecision(node);
         }
 
         public void setTipMeans(double[] traitValue, int dim, int index, NodeRef node) {
@@ -950,6 +1368,75 @@ public abstract class IntegratedMultivariateTraitLikelihood extends AbstractMult
     }
 
 
+    class OUCacheHelper extends CacheHelper {
+
+        public OUCacheHelper(int cacheLength, boolean cacheBranches) {
+            super(cacheLength, cacheBranches);
+            correctedMeanCache = new double[cacheLength];
+        }
+
+        public double[] getCorrectedMeanCache() {
+            return correctedMeanCache;
+        }
+
+        public double getOUFactor(NodeRef node) {
+            // return 1 - getTimeScaledSelection(node);
+            return Math.exp(-getTimeScaledSelection(node));
+        }
+
+        public double getUpperPrecFactor(NodeRef node) {
+            return (2 * strengthOfSelection.getBranchRate(treeModel, node)) / (1 - Math.exp(-2 * getTimeScaledSelection(node)));
+        }
+
+
+        public void setTipMeans(double[] traitValue, int dim, int index, NodeRef node) {
+            System.arraycopy(traitValue, 0, meanCache, dim * index, dim);
+        }
+
+
+        public void computeMeanCaches(int meanThisOffset, int meanOffset0, int meanOffset1,
+                                      double totalPrecision, double precision0, double precision1, MissingTraits missingTraits,
+                                      NodeRef thisNode, NodeRef node0, NodeRef node1) {
+            if (totalPrecision == 0) {
+                System.arraycopy(zeroDimVector, 0, meanCache, meanThisOffset, dim);
+            } else {
+                // Delegate in case either child is partially missing
+                // computeCorrectedWeightedAverage
+                computeCorrectedOUWeightedAverage(
+                        meanOffset0, precision0, node0,
+                        meanOffset1, precision1, node1,
+                        meanThisOffset, dim, thisNode);
+            }
+        }
+
+    }
+
+    private CacheHelper createCacheHelper(IntegratedDiffusionType type, int cacheLength, boolean cacheBranches) {
+        CacheHelper helper = null;
+        switch (type) {
+            case PLAIN:
+                helper = new CacheHelper(cacheLength, cacheBranches);
+                break;
+            case SCALED:
+                helper = new CacheHelper(cacheLength, cacheBranches);
+                break;
+            case DRIFT:
+                helper = new DriftCacheHelper(cacheLength, cacheBranches);
+                break;
+            case OU:
+                helper = new OUCacheHelper(cacheLength, cacheBranches);
+                break;
+        }
+        return helper;
+    }
+
+    public enum IntegratedDiffusionType {
+        PLAIN,
+        SCALED,
+        DRIFT,
+        OU,
+    }
+
     // protected boolean hasDrift;
 
     protected double[] upperPrecisionCache;
