@@ -1,11 +1,13 @@
 package dr.evomodel.epidemiology.casetocase;
 
+import dr.app.beagle.tools.Partition;
 import dr.evolution.util.Date;
 import dr.evolution.util.Taxa;
 import dr.inference.model.AbstractModel;
+import dr.inference.model.Parameter;
 
 /**
- * Abstract class for cases; best implemented as an inner class in implementations of AbstractOutbreak
+ * Abstract class for outbreak; best implemented as an inner class in implementations of AbstractOutbreak
  *
  * User: Matthew Hall
  * Date: 15/04/13
@@ -20,8 +22,10 @@ public abstract class AbstractCase extends AbstractModel {
 
     protected String caseID;
     protected Taxa associatedTaxa;
-    protected Date examDate;
-    protected Date endOfInfectiousDate;
+    protected double examTime;
+    protected double endOfInfectiousTime;
+    protected boolean wasEverInfected;
+    protected Parameter infectionBranchPosition;
 
     public String getName(){
         return caseID;
@@ -31,20 +35,12 @@ public abstract class AbstractCase extends AbstractModel {
         return associatedTaxa;
     }
 
-    public Date getExamDate() {
-        return examDate;
-    }
-
     public double getExamTime() {
-        return examDate.getTimeValue();
-    }
-
-    public Date getCullDate(){
-        return endOfInfectiousDate;
+        return examTime;
     }
 
     public double getCullTime(){
-        return endOfInfectiousDate.getTimeValue();
+        return endOfInfectiousTime;
     }
 
     public abstract boolean culledYet(double time);
@@ -55,6 +51,23 @@ public abstract class AbstractCase extends AbstractModel {
 
     public abstract double[] getCoords();
 
+    public boolean wasEverInfected(){
+        return wasEverInfected;
+    }
+
+    public Parameter getInfectionBranchPosition(){
+        return infectionBranchPosition;
+    }
+
+
+    public void setInfectionBranchPosition(double value){
+        infectionBranchPosition.setParameterValue(0, value);
+    }
+
+    public void setEverInfected(boolean value){
+        wasEverInfected = value;
+    }
+
 
 }
 
