@@ -3,14 +3,18 @@ package dr.evomodel.epidemiology.casetocase.operators;
 import dr.evolution.tree.NodeRef;
 import dr.evolution.tree.Tree;
 import dr.evomodel.epidemiology.casetocase.AbstractCase;
+import dr.evomodel.epidemiology.casetocase.AbstractOutbreak;
+import dr.evomodel.epidemiology.casetocase.BranchMapModel;
 import dr.evomodel.epidemiology.casetocase.CaseToCaseTreeLikelihood;
 import dr.evomodel.operators.AbstractTreeOperator;
 import dr.evomodel.tree.TreeModel;
+import dr.inference.model.Parameter;
 import dr.inference.operators.*;
 import dr.math.MathUtils;
 import dr.xml.*;
 
 import java.util.ArrayList;
+import java.util.HashMap;
 import java.util.List;
 
 /**
@@ -27,17 +31,19 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
     private boolean gaussian = false;
     private final boolean swapInRandomRate;
     private final boolean swapInRandomTrait;
+    private final boolean resampleInfectionTimes;
     private CoercionMode mode = CoercionMode.DEFAULT;
-    private boolean debug = true;
+    private static final boolean DEBUG = false;
     public static final String TRANSMISSION_SUBTREE_SLIDE_B = "transmissionSubtreeSlideB";
     public static final String SWAP_RATES = "swapInRandomRate";
     public static final String SWAP_TRAITS = "swapInRandomTrait";
     public static final String TARGET_ACCEPTANCE = "targetAcceptance";
 
     public TransmissionSubtreeSlideB(CaseToCaseTreeLikelihood c2cLikelihood, double weight, double size,
-                                     boolean gaussian, boolean swapRates, boolean swapTraits,  CoercionMode mode) {
+                                     boolean gaussian, boolean swapRates, boolean swapTraits,  CoercionMode mode,
+                                     boolean resampleInfectionTimes) {
         this.c2cLikelihood = c2cLikelihood;
-        tree = c2cLikelihood.getTree();
+        tree = c2cLikelihood.getTreeModel();
         setWeight(weight);
 
         if (size == 0.0) {
@@ -53,6 +59,8 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
         this.swapInRandomRate = swapRates;
         this.swapInRandomTrait = swapTraits;
 
+        this.resampleInfectionTimes = resampleInfectionTimes;
+
         this.mode = mode;
     }
 
@@ -63,7 +71,13 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
      */
     public double doOperation() throws OperatorFailedException {
 
-        AbstractCase[] branchMap = c2cLikelihood.getBranchMap();
+
+        if(DEBUG){
+            c2cLikelihood.debugOutputTree("beforeTSSB.nex", false);
+        }
+
+
+        BranchMapModel branchMap = c2cLikelihood.getBranchMap();
 
         double logq;
 
@@ -75,7 +89,7 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
         // 1. choose a random eligible node avoiding root
         do {
             i = tree.getNode(MathUtils.nextInt(tree.getNodeCount()));
-        } while (root == i || !eligibleForMove(i, tree, branchMap));
+        } while (!eligibleForMove(i, tree, branchMap));
 
         final NodeRef iP = tree.getParent(i);
         final NodeRef CiP = getOtherChild(tree, iP, i);
@@ -86,9 +100,33 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
         final double oldHeight = tree.getNodeHeight(iP);
         final double newHeight = oldHeight + delta;
 
+        AbstractCase iCase = branchMap.get(i.getNumber());
+        AbstractCase iPCase = branchMap.get(iP.getNumber());
+        AbstractCase CiPCase = branchMap.get(CiP.getNumber());
+        AbstractCase PiPCase = null;
+        if(PiP!=null){
+            PiPCase = branchMap.get(PiP.getNumber());
+        }
+
+        if(resampleInfectionTimes) {
+            // what happens on i's branch (there has always been a change)
+
+            iCase.setInfectionBranchPosition(MathUtils.nextDouble());
+
+            // what happens between PiP and CiP
+
+            if (PiPCase == null || CiPCase != PiPCase) {
+
+                CiPCase.setInfectionBranchPosition(MathUtils.nextDouble());
+
+            }
+        }
+
+
         // 3. if the move is down
         if (delta > 0) {
 
+
             // 3.1 if the topology will change
             if (PiP != null && tree.getNodeHeight(PiP) < newHeight) {
                 // find new parent
@@ -152,23 +190,52 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
                 // Randomly assign iP the partition of either its parent or the child that is not i, and adjust q
                 // appropriately
 
-                if(branchMap[PiP.getNumber()]!=branchMap[CiP.getNumber()]){
+                if(branchMap.get(PiP.getNumber())!=branchMap.get(CiP.getNumber())){
                     logq += Math.log(0.5);
                 }
 
-                if(newParent != null && branchMap[newParent.getNumber()]!=branchMap[newChild.getNumber()]){
+                AbstractCase newiPCase;
+
+
+                AbstractCase newChildCase = branchMap.get(newChild.getNumber());
+
+                if(newParent != null && branchMap.get(newParent.getNumber())!=branchMap.get(newChild.getNumber())){
                     if(MathUtils.nextInt(2)==0){
-                        c2cLikelihood.changeMap(iP.getNumber(), branchMap[newParent.getNumber()]);
+                        newiPCase = branchMap.get(newParent.getNumber());
                     } else {
-                        c2cLikelihood.changeMap(iP.getNumber(), branchMap[newChild.getNumber()]);
+                        newiPCase = newChildCase;
                     }
+
+                    if(resampleInfectionTimes) {
+                        //whichever we picked for iP, it's the new child's case whose infection branch is modified
+                        // (even if this infection branch is iP's branch)
+
+                        newChildCase.setInfectionBranchPosition(MathUtils.nextDouble());
+
+                    }
+
                     logq += Math.log(2);
+
                 } else {
-                    c2cLikelihood.changeMap(iP.getNumber(), branchMap[newChild.getNumber()]);
+                    newiPCase = newChildCase;
+
+                    // if iP is now the root its case infection time has changed. If not, then there is no infection
+                    // on this branch
+
+                    if(resampleInfectionTimes) {
+                        if (newParent == null) {
+                            newChildCase.setInfectionBranchPosition(MathUtils.nextDouble());
+                        }
+                    }
                 }
 
+                branchMap.set(iP.getNumber(), newiPCase, true);
+
             } else {
                 // just change the node height
+
+                // todo you could actually randomise whether the subtree containing iP is changed here
+
                 tree.setNodeHeight(iP, newHeight);
                 logq = 0.0;
             }
@@ -245,22 +312,42 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
                 // Randomly assign iP the partition of either its parent or the child that is not i, and adjust q
                 // appropriately
 
-                if(PiP!=null && branchMap[PiP.getNumber()]!=branchMap[CiP.getNumber()]){
+                if(PiP!=null && branchMap.get(PiP.getNumber())!=branchMap.get(CiP.getNumber())){
                     logq += Math.log(0.5);
                 }
 
-                if(newParent!=null && branchMap[newParent.getNumber()]!=branchMap[newChild.getNumber()]){
+                AbstractCase newiPCase;
+
+                AbstractCase newChildCase = branchMap.get(newChild.getNumber());
+
+                if(branchMap.get(newParent.getNumber())!=branchMap.get(newChild.getNumber())){
                     if(MathUtils.nextInt(2)==0){
-                        c2cLikelihood.changeMap(iP.getNumber(), branchMap[newParent.getNumber()]);
+                        newiPCase = branchMap.get(newParent.getNumber());
                     } else {
-                        c2cLikelihood.changeMap(iP.getNumber(), branchMap[newChild.getNumber()]);
+                        newiPCase = newChildCase;
                     }
+
+
+                    if(resampleInfectionTimes) {
+                        //whichever we picked for iP, it's the new child's case whose infection branch is modified
+                        // (even if this infection branch is iP's branch)
+
+                        newChildCase.setInfectionBranchPosition(MathUtils.nextDouble());
+
+                    }
+
                     logq += Math.log(2);
                 } else {
-                    c2cLikelihood.changeMap(iP.getNumber(), branchMap[newChild.getNumber()]);
+                    //upward, so don't have to worry about newParent being the root if the topology changed
+
+                    newiPCase = newChildCase;
+
                 }
 
+                branchMap.set(iP.getNumber(), newiPCase, true);
+
             } else {
+
                 tree.setNodeHeight(iP, newHeight);
                 logq = 0.0;
             }
@@ -291,7 +378,11 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
 
         if (logq == Double.NEGATIVE_INFINITY) throw new OperatorFailedException("invalid slide");
 
-        if (debug) c2cLikelihood.checkPartitions();
+        if (DEBUG) {
+            c2cLikelihood.checkPartitions();
+            c2cLikelihood.debugOutputTree("afterTSSB.nex", false);
+        }
+
 
         return logq;
     }
@@ -304,17 +395,12 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
         }
     }
 
-    private boolean eligibleForMove(NodeRef node, TreeModel tree, AbstractCase[] branchMap){
-        // to be eligible for this move, the node's parent and grandparent (if it has one), or parent and other child,
-        // must be in the same partition (so removing the parent has no effect on the remaining links of the TT), and
-        // the node and its parent must be in different partitions (so the move does not disconnect anything)
+    private boolean eligibleForMove(NodeRef node, TreeModel tree, BranchMapModel branchMap){
+        // to be eligible for this move, the node's parent must exist and be in a different partition to itself. This
+        // forces the parent to be in the same partition as either its grandchild or its child.
 
-        return ((tree.getParent(tree.getParent(node))!=null
-                && branchMap[tree.getParent(node).getNumber()]
-                ==branchMap[tree.getParent(tree.getParent(node)).getNumber()])
-                || branchMap[tree.getParent(node).getNumber()]
-                ==branchMap[getOtherChild(tree,tree.getParent(node), node).getNumber()])
-                && branchMap[tree.getParent(node).getNumber()]!=branchMap[node.getNumber()];
+        return (!tree.isRoot(node) && branchMap.get(tree.getParent(node).getNumber())
+                !=branchMap.get(node.getNumber()));
     }
 
     //intersectingEdges is the same as in normal STS, since there's no additional restriction in this case on where
@@ -373,6 +459,8 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
 
     public static XMLObjectParser PARSER = new AbstractXMLObjectParser() {
 
+        public static final String RESAMPLE_INFECTION_TIMES = "resampleInfectionTimes";
+
         public Object parseXMLObject(XMLObject xo) throws XMLParseException {
 
             boolean swapRates = xo.getAttribute(SWAP_RATES, false);
@@ -396,12 +484,18 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
 
             if (Double.isInfinite(size) || size <= 0.0) {
                 throw new XMLParseException("size attribute must be positive and not infinite. was " + size +
-                        " for tree " + c2cL.getTree().getId() );
+                        " for tree " + c2cL.getTreeModel().getId() );
+            }
+
+            boolean resampleInfectionTimes = false;
+
+            if(xo.hasAttribute(RESAMPLE_INFECTION_TIMES)) {
+                resampleInfectionTimes = xo.getBooleanAttribute(RESAMPLE_INFECTION_TIMES);
             }
 
             final boolean gaussian = xo.getBooleanAttribute("gaussian");
             TransmissionSubtreeSlideB operator = new TransmissionSubtreeSlideB(c2cL, weight, size, gaussian,
-                    swapRates, swapTraits, mode);
+                    swapRates, swapTraits, mode, resampleInfectionTimes);
             operator.setTargetAcceptanceProbability(targetAcceptance);
 
             return operator;
@@ -432,6 +526,7 @@ public class TransmissionSubtreeSlideB extends AbstractTreeOperator implements C
                 AttributeRule.newBooleanRule(SWAP_RATES, true),
                 AttributeRule.newBooleanRule(SWAP_TRAITS, true),
                 AttributeRule.newBooleanRule(CoercableMCMCOperator.AUTO_OPTIMIZE, true),
+                AttributeRule.newBooleanRule(RESAMPLE_INFECTION_TIMES, true),
                 new ElementRule(CaseToCaseTreeLikelihood.class)
         };
     };
