@@ -1,3 +1,28 @@
+/*
+ * AlloppDiploidHistory.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.speciation;
 
 import java.util.ArrayList;
@@ -13,7 +38,7 @@ import dr.evolution.util.Taxon;
 import dr.math.MathUtils;
 import dr.util.AlloppMisc;
 import jebl.util.FixedBitSet;
-import test.dr.evomodel.speciation.AlloppSpeciesNetworkModelTEST;
+
 
 /**
  * AlloppDiploidHistory represents part of the network before hybridizations.
@@ -31,7 +56,7 @@ import test.dr.evomodel.speciation.AlloppSpeciesNetworkModelTEST;
  * tetraploid) occurs. The latter are in pairs and have times before present,
  * and I call them `hydridization tips' or `hyb-tips'. From the point of
  * view of a tetraploid tree these hyb-tips are feet at the ends of legs.
- * 
+ *
  * The purpose of this class is to represent the part of the network before
  * hybridizations (=`diploid history') in a form (the array of DipHistNode's)
  * that can be subjected to Mau-type moves.
@@ -308,7 +333,7 @@ public class AlloppDiploidHistory implements SlidableTree {
             dhnodes[nextn].height = tettipsheight;
             tojoin.add(new JoiningNode(nextn, false));
             nextn ++;
-            }
+        }
 
         // build the tree from tips, taking care not to join the last diploids before all tets used up
         for (int i = 0; i < ntips-1; i++) {
@@ -359,8 +384,7 @@ public class AlloppDiploidHistory implements SlidableTree {
 
     //  constructor for testing.
     public AlloppDiploidHistory(SimpleNode[] snodes, int sroot, ArrayList<AlloppLeggedTree> tettrees,
-                                boolean diprootisroot, AlloppSpeciesBindings apsp,
-                                AlloppSpeciesNetworkModelTEST.NetworkToMultreeTEST nmltTEST) {
+                                boolean diprootisroot, AlloppSpeciesBindings apsp) {
         this.apsp = apsp;
         // Make array of dud nodes
         dhnodes = new DipHistNode[snodes.length];
