@@ -1,3 +1,28 @@
+/*
+ * AlloppSpeciesNetworkModel.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.speciation;
 
 
@@ -23,8 +48,6 @@ import jebl.util.FixedBitSet;
 import java.util.*;
 import java.util.logging.Logger;
 
-import test.dr.evomodel.speciation.AlloppSpeciesNetworkModelTEST;
-
 
 /**
  *
@@ -37,29 +60,29 @@ import test.dr.evomodel.speciation.AlloppSpeciesNetworkModelTEST;
 
 /*
  * class AlloppSpeciesNetworkModel
- * 
+ *
  * Implements the species network as a collection of `trees with legs'.
- * and converts this representation into a multiply labelled 
+ * and converts this representation into a multiply labelled
  * binary tree.
- * 
+ *
  * General idea is that the network is easiest to change (eg detach
  * and re-attach tetraploid subtrees) while likelihood calculations
  * are easiest to do in the multiply labelled tree.
- * 
- * The individual `trees with legs' are implemented by AlloppLeggedTree's. 
+ *
+ * The individual `trees with legs' are implemented by AlloppLeggedTree's.
  * The multiply labelled binary tree is implemented by AlloppMulLabTree.
- * 
+ *
  * *********************
- * 
- * apsp is a reference to the AlloppSpeciesBindings which knows how 
+ *
+ * apsp is a reference to the AlloppSpeciesBindings which knows how
  * species are made of individuals and individuals are made of taxa,
  * and which contains the list of gene trees.
- * 
+ *
  * trees[][] represents the network as a set of homoploid trees
- * 
+ *
  * mullabtree represents the network as single tree with tips that
- * can be multiply labelled with species. 
- * 
+ * can be multiply labelled with species.
+ *
  */
 //  grjtodo-oneday JH's SpeciesTreeModel implements
 // MutableTree, TreeTraitProvider, TreeLogger.LogUpon, Scalable
@@ -234,8 +257,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
       * This (partial) constructor is for testing conversion network to multree.
       * Real work done by testExampleNetworkToMulLabTree()
       */
-    public AlloppSpeciesNetworkModel(AlloppSpeciesBindings apsp,
-                                     AlloppSpeciesNetworkModelTEST.NetworkToMultreeTEST nmltTEST) {
+    public AlloppSpeciesNetworkModel(AlloppSpeciesBindings apsp) {
         super(AlloppSpeciesNetworkModelParser.ALLOPPSPECIESNETWORK);
         this.apsp = apsp;
         tippopvalues = null;
@@ -705,7 +727,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
 
 
     /*
-	 * Make a random initial starting network. 
+	 * Make a random initial starting network.
 	 */
     private void makeInitialNDipsNTetsNetwork(Taxon[] dipspp, Taxon[] tetspp) {
         //
@@ -1039,8 +1061,10 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
     // it just signals that this is indeed a test.
 
 
-    public String testExampleNetworkToMulLabTree(
-            AlloppSpeciesNetworkModelTEST.NetworkToMultreeTEST nmltTEST) {
+
+    //AR - removing this as it creates a dependency to test.dr.* which is bad...
+
+    public String testExampleNetworkToMulLabTree(int testcase) {
 
         int ntaxa = apsp.numberOfSpecies();
         Taxon[] spp = new Taxon[ntaxa];
@@ -1060,7 +1084,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
         // case 5. three tettrees, 1+1+1, one of each type of feet, as in cases 1-3
 
         int ntettrees = 0;
-        switch (nmltTEST.testcase) {
+        switch (testcase) {
             case 1:
             case 2:
             case 3:
@@ -1088,33 +1112,33 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
         Taxon[] tets2 = {spp[2]};
         Taxon[] tets3 = {spp[3]};
         Taxon[] dips = new Taxon[0];
-        switch (nmltTEST.testcase) {
+        switch (testcase) {
             case 1:
-                tettrees.add(new AlloppLeggedTree(tets123, nmltTEST));
+                tettrees.add(new AlloppLeggedTree(tets123));
                 tetheight0 = tettrees.get(0).getRootHeight();
                 dips = new Taxon[] {spp[0], l0, r0, spp[4]};
                 break;
             case 2:
-                tettrees.add(new AlloppLeggedTree(tets123, nmltTEST));
+                tettrees.add(new AlloppLeggedTree(tets123));
                 tetheight0 = tettrees.get(0).getRootHeight();
                 dips = new Taxon[] {spp[0], l0, r0, spp[4]};
                 break;
             case 3:
-                tettrees.add(new AlloppLeggedTree(tets123, nmltTEST));
+                tettrees.add(new AlloppLeggedTree(tets123));
                 tetheight0 = tettrees.get(0).getRootHeight();
                 dips = new Taxon[] {spp[0], l0, r0, spp[4]};
                 break;
             case 4:
-                tettrees.add(new AlloppLeggedTree(tets12, nmltTEST));
-                tettrees.add(new AlloppLeggedTree(tets3, nmltTEST));
+                tettrees.add(new AlloppLeggedTree(tets12));
+                tettrees.add(new AlloppLeggedTree(tets3));
                 tetheight0 = tettrees.get(0).getRootHeight();
                 tetheight1 = tettrees.get(1).getRootHeight();
                 dips = new Taxon[] {spp[0], l0, r0, l1, r1, spp[4]};
                 break;
             case 5:
-                tettrees.add(new AlloppLeggedTree(tets1, nmltTEST));
-                tettrees.add(new AlloppLeggedTree(tets2, nmltTEST));
-                tettrees.add(new AlloppLeggedTree(tets3, nmltTEST));
+                tettrees.add(new AlloppLeggedTree(tets1));
+                tettrees.add(new AlloppLeggedTree(tets2));
+                tettrees.add(new AlloppLeggedTree(tets3));
                 tetheight0 = tettrees.get(0).getRootHeight();
                 tetheight1 = tettrees.get(1).getRootHeight();
                 tetheight2 = tettrees.get(2).getRootHeight();
@@ -1133,7 +1157,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
             }
         }
         int dhroot = -1;
-        switch (nmltTEST.testcase) {
+        switch (testcase) {
             case 1:
                 dhnodes[1].setHeight(tetheight0 + 1.0);
                 dhnodes[2].setHeight(tetheight0 + 1.0);
@@ -1187,7 +1211,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
                 dhroot = 14;
                 break;
         }
-        AlloppDiploidHistory adhist = new AlloppDiploidHistory(dhnodes, dhroot, tettrees, true, apsp, nmltTEST);
+        AlloppDiploidHistory adhist = new AlloppDiploidHistory(dhnodes, dhroot, tettrees, true, apsp);
         int ntippopparams = numberOfTipPopParameters();
         int nrootpopparams = numberOfRootPopParameters();
         int maxnhybpopparams = maxNumberOfHybPopParameters();
@@ -1204,7 +1228,7 @@ public class AlloppSpeciesNetworkModel extends AbstractModel implements
             testhybpopvalues[pp] = 3000+pp;
         }
         AlloppMulLabTree testmullabtree = new AlloppMulLabTree(adhist, tettrees, apsp,
-                testtippopvalues, testrootpopvalues, testhybpopvalues, nmltTEST);
+                testtippopvalues, testrootpopvalues, testhybpopvalues);
         System.out.println(testmullabtree.asText());
         String newick = testmullabtree.mullabTreeAsNewick();
         return newick;
