@@ -1,3 +1,28 @@
+/*
+ * HypermutantErrorModel.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodel.treelikelihood;
 
 import dr.evolution.alignment.HypermutantAlignment;
@@ -32,9 +57,9 @@ public class HypermutantErrorModel extends TipStatesModel {
         this.hypermutationRateParameter = hypermutationRateParameter;
         addVariable(this.hypermutationRateParameter);
 
-        this.hypermuationIndicatorParameter = hypermuationIndicatorParameter;
+        this.hypermutationIndicatorParameter = hypermuationIndicatorParameter;
 
-        addVariable(this.hypermuationIndicatorParameter);
+        addVariable(this.hypermutationIndicatorParameter);
 
         addStatistic(new TaxonHypermutatedStatistic());
         addStatistic(new TaxonHypermutationRateStatistic());
@@ -42,8 +67,8 @@ public class HypermutantErrorModel extends TipStatesModel {
     }
 
     protected void taxaChanged() {
-        if (hypermuationIndicatorParameter.getDimension() <= 1) {
-            this.hypermuationIndicatorParameter.setDimension(tree.getExternalNodeCount());
+        if (hypermutationIndicatorParameter.getDimension() <= 1) {
+            this.hypermutationIndicatorParameter.setDimension(tree.getExternalNodeCount());
         }
         if (unlinkedRates && hypermutationRateParameter.getDimension() <= 1) {
             this.hypermutationRateParameter.setDimension(tree.getExternalNodeCount());
@@ -63,7 +88,7 @@ public class HypermutantErrorModel extends TipStatesModel {
     @Override
     public void getTipPartials(int nodeIndex, double[] partials) {
         int[] states = this.states[nodeIndex];
-        boolean isHypermutated = hypermuationIndicatorParameter.getParameterValue(nodeIndex) > 0.0;
+        boolean isHypermutated = hypermutationIndicatorParameter.getParameterValue(nodeIndex) > 0.0;
 
         double rate = (unlinkedRates ? hypermutationRateParameter.getParameterValue(nodeIndex) : hypermutationRateParameter.getParameterValue(0));
 
@@ -123,12 +148,14 @@ public class HypermutantErrorModel extends TipStatesModel {
 
     @Override
     protected final void handleVariableChangedEvent(Variable variable, int index, Parameter.ChangeType type) {
-        if (variable == hypermuationIndicatorParameter) {
-            fireModelChanged();
+        if (variable == hypermutationIndicatorParameter) {
+            fireModelChanged(tree.getTaxon(index));
         } else if (variable == hypermutationRateParameter) {
-            if (!unlinkedRates || hypermuationIndicatorParameter.getValue(index)  > 0.5) {
-                // only fire an update if the indicator is on....
+            if (!unlinkedRates) {
                 fireModelChanged();
+            } else if (hypermutationIndicatorParameter.getValue(index)  > 0.5) {
+                // only fire an update if the indicator is on....
+                fireModelChanged(tree.getTaxon(index));
             }
         } else {
             throw new RuntimeException("Unknown parameter has changed in HypermutantErrorModel.handleVariableChangedEvent");
@@ -194,7 +221,7 @@ public class HypermutantErrorModel extends TipStatesModel {
         }
 
         public int getDimension() {
-            return hypermuationIndicatorParameter.getDimension();
+            return hypermutationIndicatorParameter.getDimension();
         }
 
         public String getDimensionName(int dim) {
@@ -202,7 +229,7 @@ public class HypermutantErrorModel extends TipStatesModel {
         }
 
         public double getStatisticValue(int dim) {
-            return hypermuationIndicatorParameter.getParameterValue(dim);
+            return hypermutationIndicatorParameter.getParameterValue(dim);
         }
 
     }
@@ -222,7 +249,7 @@ public class HypermutantErrorModel extends TipStatesModel {
         }
 
         public double getStatisticValue(int dim) {
-            return hypermutationRateParameter.getParameterValue(dim) * hypermuationIndicatorParameter.getParameterValue(dim);                           
+            return hypermutationRateParameter.getParameterValue(dim) * hypermutationIndicatorParameter.getParameterValue(dim);
         }
 
     }
@@ -256,8 +283,8 @@ public class HypermutantErrorModel extends TipStatesModel {
 
             double mutatedCount = 0;
             double totalCount = 0;
-            for (int i = 0; i < hypermuationIndicatorParameter.getDimension(); i++) {
-                if (hypermuationIndicatorParameter.getParameterValue(i) > 0.5) {
+            for (int i = 0; i < hypermutationIndicatorParameter.getDimension(); i++) {
+                if (hypermutationIndicatorParameter.getParameterValue(i) > 0.5) {
                     mutatedCount += mutatedContextCounts.get(i);
                     totalCount += mutatedCount + unmutatedContextCounts.get(i);
 
@@ -275,6 +302,6 @@ public class HypermutantErrorModel extends TipStatesModel {
 
     private final HypermutantAlignment hypermutantAlignment;
     private final Parameter hypermutationRateParameter;
-    private final Parameter hypermuationIndicatorParameter;
+    private final Parameter hypermutationIndicatorParameter;
     private final boolean unlinkedRates;
 }
\ No newline at end of file
