@@ -1,7 +1,7 @@
 /*
  * EmpiricalTreeDistributionModelParser.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -44,6 +44,7 @@ import java.util.logging.Logger;
 public class EmpiricalTreeDistributionModelParser extends AbstractXMLObjectParser {
 
     public static final String RATE_ATTRIBUTE_NAME = "rateAttribute";
+    public static final String STARTING_TREE = "startingTree";
 
     public String getParserName() {
         return EmpiricalTreeDistributionModel.EMPIRICAL_TREE_DISTRIBUTION_MODEL;
@@ -82,7 +83,9 @@ public class EmpiricalTreeDistributionModelParser extends AbstractXMLObjectParse
         
         Logger.getLogger("dr.evomodel").info("    Read " + trees.length + " trees from file, " + fileName);
 
-        return new EmpiricalTreeDistributionModel(trees);
+        int startingTree = xo.getAttribute(STARTING_TREE, -1); // default is random tree
+
+        return new EmpiricalTreeDistributionModel(trees, startingTree);
     }
 
     public static final String FILE_NAME = "fileName";
@@ -90,6 +93,7 @@ public class EmpiricalTreeDistributionModelParser extends AbstractXMLObjectParse
 
     public XMLSyntaxRule[] getSyntaxRules() {
         return new XMLSyntaxRule[]{
+                AttributeRule.newIntegerRule(STARTING_TREE, true),
                 new StringAttributeRule(FILE_NAME,
                         "The name of a NEXUS tree file"),
 //                AttributeRule.newIntegerRule(BURNIN, true,
