@@ -1,3 +1,28 @@
+/*
+ * DiscretizedBranchRatesParser.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodelxml.branchratemodel;
 
 import dr.evomodel.branchratemodel.DiscretizedBranchRates;
@@ -20,6 +45,8 @@ public class DiscretizedBranchRatesParser extends AbstractXMLObjectParser {
     public static final String OVERSAMPLING = "overSampling";
     public static final String NORMALIZE = "normalize";
     public static final String NORMALIZE_BRANCH_RATE_TO = "normalizeBranchRateTo";
+    public static final String RANDOMIZE_RATES = "randomizeRates";
+    public static final String KEEP_RATES = "keepRates";
     //public static final String NORMALIZED_MEAN = "normalizedMean";
 
 
@@ -57,11 +84,19 @@ public class DiscretizedBranchRatesParser extends AbstractXMLObjectParser {
             Logger.getLogger("dr.evomodel").warning("   WARNING: single root rate is not implemented!");
         }
 
+        final boolean randomizeRates = xo.getAttribute(RANDOMIZE_RATES, false);
+        final boolean keepRates = xo.getAttribute(KEEP_RATES, false);
+
+        if (randomizeRates && keepRates) {
+            throw new XMLParseException("Unable to both randomize and keep current rate categories");
+        }
+
         /* if (xo.hasAttribute(NORMALIZED_MEAN)) {
             dbr.setNormalizedMean(xo.getDoubleAttribute(NORMALIZED_MEAN));
         }*/
 
-        return new DiscretizedBranchRates(tree, rateCategoryParameter, distributionModel, overSampling, normalize, normalizeBranchRateTo);
+        return new DiscretizedBranchRates(tree, rateCategoryParameter, distributionModel, overSampling, normalize,
+                normalizeBranchRateTo, randomizeRates, keepRates);
     }
 
     //************************************************************************
@@ -88,6 +123,8 @@ public class DiscretizedBranchRatesParser extends AbstractXMLObjectParser {
             AttributeRule.newIntegerRule(OVERSAMPLING, true, "The integer factor for oversampling the distribution model (1 means no oversampling)"),
             AttributeRule.newBooleanRule(NORMALIZE, true, "Whether the mean rate has to be normalized to a particular value"),
             AttributeRule.newDoubleRule(NORMALIZE_BRANCH_RATE_TO, true, "The mean rate to normalize to, if normalizing"),
+            AttributeRule.newBooleanRule(RANDOMIZE_RATES, true, "Randomize initial categories"),
+            AttributeRule.newBooleanRule(KEEP_RATES, true, "Keep current rate category specification"),
             new ElementRule(TreeModel.class),
             new ElementRule(DISTRIBUTION, ParametricDistributionModel.class, "The distribution model for rates among branches", false),
             new ElementRule(RATE_CATEGORIES, Parameter.class, "The rate categories parameter", false),
