@@ -1,3 +1,28 @@
+/*
+ * LocalClockModelParser.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.evomodelxml.branchratemodel;
 
 import dr.evolution.tree.Tree;
@@ -18,6 +43,7 @@ public class LocalClockModelParser extends AbstractXMLObjectParser {
     public static final String RELATIVE = "relative";
     public static final String CLADE = "clade";
     public static final String INCLUDE_STEM = "includeStem";
+    public static final String STEM_PROPORTION = "stemProportion";
     public static final String EXCLUDE_CLADE = "excludeClade";
     public static final String EXTERNAL_BRANCHES = "externalBranches";
     public static final String TRUNK = "trunk";
@@ -52,9 +78,19 @@ public class LocalClockModelParser extends AbstractXMLObjectParser {
 
                     boolean includeStem = false;
                     boolean excludeClade = false;
+                    double stemProportion = 0.0;
 
                     if (xoc.hasAttribute(INCLUDE_STEM)) {
                         includeStem = xoc.getBooleanAttribute(INCLUDE_STEM);
+                        // if includeStem=true then assume it is the whole stem
+                        stemProportion = 1.0;
+                    }
+
+                    if (xoc.hasAttribute(STEM_PROPORTION)) {
+                        stemProportion = xoc.getDoubleAttribute(STEM_PROPORTION);
+                        if (stemProportion < 0.0 || stemProportion > 1.0) {
+                            throw new XMLParseException("A stem proportion should be between 0, 1");
+                        }
                     }
 
                     if (xoc.hasAttribute(EXCLUDE_CLADE)) {
@@ -62,7 +98,7 @@ public class LocalClockModelParser extends AbstractXMLObjectParser {
                     }
 
                     try {
-                        localClockModel.addCladeClock(taxonList, rateParameter, relative, includeStem, excludeClade);
+                        localClockModel.addCladeClock(taxonList, rateParameter, relative, stemProportion, excludeClade);
 
                     } catch (Tree.MissingTaxonException mte) {
                         throw new XMLParseException("Taxon, " + mte + ", in " + getParserName() + " was not found in the tree.");
@@ -85,7 +121,10 @@ public class LocalClockModelParser extends AbstractXMLObjectParser {
 
                     boolean relative = xoc.getAttribute(RELATIVE, false);
 
-                    Parameter indexParameter = (Parameter) xoc.getElementFirstChild(INDEX);
+                    Parameter indexParameter = null;
+                    if (xoc.hasChildNamed(INDEX)) {
+                        indexParameter = (Parameter) xoc.getElementFirstChild(INDEX);
+                    }
                     Parameter rateParameter = (Parameter) xoc.getChild(Parameter.class);
                     TaxonList taxonList = (TaxonList) xoc.getChild(TaxonList.class);
 
@@ -136,6 +175,7 @@ public class LocalClockModelParser extends AbstractXMLObjectParser {
                     new XMLSyntaxRule[]{
                             AttributeRule.newBooleanRule(RELATIVE, true),
                             AttributeRule.newBooleanRule(INCLUDE_STEM, true, "determines whether or not the stem branch above this clade is included in the siteModel (default false)."),
+                            AttributeRule.newDoubleRule(STEM_PROPORTION, true, "proportion of stem to include in clade rate (default 0)."),
                             AttributeRule.newBooleanRule(EXCLUDE_CLADE, true, "determines whether to exclude actual branches of the clade from the siteModel (default false)."),
                             new ElementRule(Taxa.class, "A set of taxa which defines a clade to apply a different site model to"),
                             new ElementRule(Parameter.class, "The rate parameter")
