@@ -1,7 +1,7 @@
 /*
  * TreeTraitParserUtilities.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2014 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -27,6 +27,7 @@ package dr.evomodelxml.treelikelihood;
 
 import dr.evolution.tree.MultivariateTraitTree;
 import dr.inference.model.CompoundParameter;
+import dr.inference.model.MatrixParameter;
 import dr.inference.model.Parameter;
 import dr.inference.model.ParameterParser;
 import dr.math.MathUtils;
@@ -51,6 +52,7 @@ public class TreeTraitParserUtilities {
     public static final String RANDOMIZE_LOWER = "lower";
     public static final String RANDOMIZE_UPPER = "upper";
 
+    public static final String ALLOW_IDENTICAL = "allowIdentical";
     public static final String JITTER = "jitter";
     public static final String WINDOW = "window";
     public static final String DUPLICATES = "duplicatesOnly";
@@ -139,6 +141,27 @@ public class TreeTraitParserUtilities {
         }
     }
 
+    public boolean hasIdenticalTraits(Parameter trait, int dim) {
+        int numTraits = trait.getDimension() / dim;
+
+        DoubleArray[] traitArray = new DoubleArray[numTraits];
+        for (int i = 0; i < numTraits; i++) {
+            double[] x = new double[dim];
+            for (int j = 0; j < dim; j++) {
+                x[j] = trait.getParameterValue(i * dim + j);
+            }
+            traitArray[i] = new DoubleArray(x, i);
+        }
+        Arrays.sort(traitArray);
+        // Mark duplicates
+        for (int i = 1; i < numTraits; i++) {
+            if (traitArray[i].compareTo(traitArray[i - 1]) == 0) {
+                return true;
+            }
+        }
+        return false;
+    }
+
     public void jitter(Parameter trait, int dim, List<Integer> missingIndices, double[] window, boolean duplicates, boolean verbose) {
         int numTraits = trait.getDimension() / dim;
         boolean[] update = new boolean[numTraits];
@@ -224,6 +247,13 @@ public class TreeTraitParserUtilities {
         CompoundParameter traitParameter;
         List<Integer> missingIndices = null;
 
+        boolean isMatrixParameter = false;
+        if (parameter instanceof MatrixParameter) {
+            traitParameter = (CompoundParameter) parameter;
+            isMatrixParameter = true;
+        } else
+
+
         if (parameter instanceof CompoundParameter) {
             // if we have been passed a CompoundParameter, this will be a leaf trait
             // parameter from a tree model so use this to allow for individual sampling
@@ -268,9 +298,14 @@ public class TreeTraitParserUtilities {
                             }
                         }
                     } else {
-                        // Make multidimensional, in earlier revisions only first dimension was stored
-                        traitParam = new Parameter.Default(paramName, count);
-                        traitParameter.addParameter(traitParam);
+                        if (isMatrixParameter) {
+                            traitParam = traitParameter.getParameter(i);
+                            traitParam.setId(paramName);
+                        } else {
+                            // Make multidimensional, in earlier revisions only first dimension was stored
+                            traitParam = new Parameter.Default(paramName, count);
+                            traitParameter.addParameter(traitParam);
+                        }
                     }
 
 
