@@ -1,7 +1,7 @@
 /*
  * MergePatternsParser.java
  *
- * Copyright (C) 2002-2006 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -28,6 +28,8 @@ package dr.evoxml;
 import dr.evolution.alignment.*;
 import dr.xml.*;
 
+import java.util.logging.Logger;
+
 /**
  * @author Alexei Drummond
  * @author Andrew Rambaut
@@ -51,6 +53,12 @@ public class MergePatternsParser extends AbstractXMLObjectParser {
 		    patterns.addPatterns((PatternList)xo.getChild(i));
 	    }
 
+        if (xo.hasAttribute(XMLParser.ID)) {
+            final Logger logger = Logger.getLogger("dr.evoxml");
+            logger.info("Site patterns '" + xo.getId() + "' created by merging " + xo.getChildCount() + " pattern lists");
+            logger.info("  pattern count = " + patterns.getPatternCount());
+        }
+
         return patterns;
     }
 
