@@ -1,7 +1,7 @@
 /*
  * DistributionLikelihood.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -40,6 +40,8 @@ import org.w3c.dom.Element;
 
 public class DistributionLikelihood extends AbstractDistributionLikelihood {
 
+    public final static boolean DEBUG = false;
+
     public static final String DISTRIBUTION_LIKELIHOOD = "distributionLikelihood";
 
     private int from = -1;
@@ -47,22 +49,27 @@ public class DistributionLikelihood extends AbstractDistributionLikelihood {
     private final boolean evaluateEarly;
 
     public DistributionLikelihood(Distribution distribution) {
-        this(distribution, 0.0, false);
+        this(distribution, 0.0, false, 1.0);
     }
 
     public DistributionLikelihood(Distribution distribution, double offset) {
-        this(distribution, offset, offset > 0.0);
+        this(distribution, offset, offset > 0.0, 1.0);
+    }
+
+    public DistributionLikelihood(Distribution distribution, double offset, double scale){
+        this(distribution, offset, offset>0.0, scale);
     }
 
     public DistributionLikelihood(Distribution distribution, boolean evaluateEarly) {
-        this(distribution, 0.0, evaluateEarly);
+        this(distribution, 0.0, evaluateEarly, 1.0);
     }
 
-    public DistributionLikelihood(Distribution distribution, double offset, boolean evaluateEarly) {
+    public DistributionLikelihood(Distribution distribution, double offset, boolean evaluateEarly, double scale) {
         super(null);
         this.distribution = distribution;
         this.offset = offset;
         this.evaluateEarly = evaluateEarly;
+        this.scale=scale;
     }
 
     public DistributionLikelihood(ParametricDistributionModel distributionModel) {
@@ -70,6 +77,7 @@ public class DistributionLikelihood extends AbstractDistributionLikelihood {
         this.distribution = distributionModel;
         this.offset = 0.0;
         this.evaluateEarly = false;
+        this.scale=1.0;
     }
 
     public Distribution getDistribution() {
@@ -92,6 +100,12 @@ public class DistributionLikelihood extends AbstractDistributionLikelihood {
      */
     public double calculateLogLikelihood() {
 
+        if (DEBUG) {
+            System.out.println("Calling DistributionLikelihood.calculateLogLikelihood()");
+            System.out.println(distribution.toString());
+            System.out.println(dataList.toString() + "\n");
+        }
+
         double logL = 0.0;
 
         for( Attribute<double[]> data : dataList ) {
@@ -112,7 +126,7 @@ public class DistributionLikelihood extends AbstractDistributionLikelihood {
                     return Double.NEGATIVE_INFINITY;
                 }
 
-                logL += distribution.logPdf(value);
+                logL += distribution.logPdf(value/scale)/scale;
             }
 
         }
@@ -155,5 +169,6 @@ public class DistributionLikelihood extends AbstractDistributionLikelihood {
 
     protected Distribution distribution;
     private final double offset;
+    private final double scale;
 }
 
