@@ -1,7 +1,7 @@
 /*
  * MarkovChain.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -25,12 +25,15 @@
 
 package dr.inference.markovchain;
 
+import dr.evomodel.continuous.GibbsIndependentCoalescentOperator;
 import dr.inference.model.CompoundLikelihood;
 import dr.inference.model.Likelihood;
 import dr.inference.model.Model;
+import dr.inference.model.PathLikelihood;
 import dr.inference.operators.*;
 import dr.inference.prior.Prior;
 
+import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.logging.Logger;
 
@@ -43,7 +46,10 @@ import java.util.logging.Logger;
  * @author Andrew Rambaut
  * @version $Id: MarkovChain.java,v 1.10 2006/06/21 13:34:42 rambaut Exp $
  */
-public final class MarkovChain {
+public final class MarkovChain implements Serializable {
+    private static final long serialVersionUID = 181L;
+
+
     private final static boolean DEBUG = false;
     private final static boolean PROFILE = true;
 
@@ -102,10 +108,8 @@ public final class MarkovChain {
      * Run the chain for a given number of states.
      *
      * @param length number of states to run the chain.
-     *               <p/>
-     *               param onTheFlyOperatorWeights
      */
-    public long runChain(long length, boolean disableCoerce /*,int onTheFlyOperatorWeights*/) {
+    public long runChain(long length, boolean disableCoerce) {
 
         likelihood.makeDirty();
         currentScore = evaluate(likelihood, prior);
@@ -135,6 +139,9 @@ public final class MarkovChain {
             String message = "The initial likelihood is zero";
             if (likelihood instanceof CompoundLikelihood) {
                 message += ": " + ((CompoundLikelihood) likelihood).getDiagnosis();
+            } else if (likelihood instanceof PathLikelihood) {
+                message += ": " + ((CompoundLikelihood)((PathLikelihood) likelihood).getSourceLikelihood()).getDiagnosis();
+                message += ": " + ((CompoundLikelihood)((PathLikelihood) likelihood).getDestinationLikelihood()).getDiagnosis();
             } else {
                 message += ".";
             }
@@ -152,8 +159,6 @@ public final class MarkovChain {
         pleaseStop = false;
         isStopped = false;
 
-        String diagnostic = "";
-
         //int otfcounter = onTheFlyOperatorWeights > 0 ? onTheFlyOperatorWeights : 0;
 
         double[] logr = {0.0};
@@ -166,6 +171,8 @@ public final class MarkovChain {
 
         while (!pleaseStop && (currentState < (currentLength + length))) {
 
+            String diagnosticStart = "";
+
             // periodically log states
             fireCurrentModel(currentState, currentModel);
 
@@ -179,6 +186,10 @@ public final class MarkovChain {
             final MCMCOperator mcmcOperator = schedule.getOperator(op);
 
             double oldScore = currentScore;
+            if (usingFullEvaluation) {
+                diagnosticStart = likelihood instanceof CompoundLikelihood ?
+                        ((CompoundLikelihood) likelihood).getDiagnosis() : "";
+            }
 
             // assert Profiler.startProfile("Store");
 
@@ -239,14 +250,25 @@ public final class MarkovChain {
                     mcmcOperator.addEvaluationTime(System.currentTimeMillis() - elapsedTime);
                 }
 
-                // assert Profiler.stopProfile("Evaluate");
+                String diagnosticOperator = "";
+                if (usingFullEvaluation) {
+                    diagnosticOperator = likelihood instanceof CompoundLikelihood ?
+                            ((CompoundLikelihood) likelihood).getDiagnosis() : "";
+                }
 
-                if (score == Double.POSITIVE_INFINITY || Double.isNaN(score)) {
+                if (score == Double.NEGATIVE_INFINITY && mcmcOperator instanceof GibbsOperator) {
+                    if (!(mcmcOperator instanceof GibbsIndependentNormalDistributionOperator) && !(mcmcOperator instanceof GibbsIndependentGammaOperator) && !(mcmcOperator instanceof GibbsIndependentCoalescentOperator) && !(mcmcOperator instanceof GibbsIndependentJointNormalGammaOperator)) {
+                        Logger.getLogger("error").severe("State " + currentState + ": A Gibbs operator, " + mcmcOperator.getOperatorName() + ", returned a state with zero likelihood.");
+                    }
+                }
+
+                if (score == Double.POSITIVE_INFINITY ||
+                        Double.isNaN(score) ) {
                     if (likelihood instanceof CompoundLikelihood) {
-                        Logger.getLogger("error").severe("A likelihood returned with a numerical error:\n" +
+                        Logger.getLogger("error").severe("State "+currentState+": A likelihood returned with a numerical error:\n" +
                                 ((CompoundLikelihood)likelihood).getDiagnosis());
                     } else {
-                        Logger.getLogger("error").severe("A likelihood returned with a numerical error.");
+                        Logger.getLogger("error").severe("State "+currentState+": A likelihood returned with a numerical error.");
                     }
 
                     // If the user has chosen to ignore this error then we transform it
@@ -256,19 +278,27 @@ public final class MarkovChain {
 
                 if (usingFullEvaluation) {
 
-                    // This is a test that the state is correctly restored. The
-                    // restored state is fully evaluated and the likelihood compared with
-                    // that before the operation was made.
+                    // This is a test that the state was correctly evaluated. The
+                    // likelihood of all components of the model are flagged as
+                    // needing recalculation, then the full likelihood is calculated
+                    // again and compared to the first result. This checks that the
+                    // BEAST is aware of all changes that the operator induced.
+
                     likelihood.makeDirty();
                     final double testScore = evaluate(likelihood, prior);
 
+                    final String d2 = likelihood instanceof CompoundLikelihood ?
+                            ((CompoundLikelihood) likelihood).getDiagnosis() : "";
+
                     if (Math.abs(testScore - score) > evaluationTestThreshold) {
                         Logger.getLogger("error").severe(
-                                "State was not correctly calculated after an operator move.\n"
+                                "State "+currentState+": State was not correctly calculated after an operator move.\n"
                                         + "Likelihood evaluation: " + score
                                         + "\nFull Likelihood evaluation: " + testScore
                                         + "\n" + "Operator: " + mcmcOperator
-                                        + " " + mcmcOperator.getOperatorName());
+                                        + " " + mcmcOperator.getOperatorName()
+                                        + (diagnosticOperator.length() > 0 ? "\n\nDetails\nBefore: " + diagnosticOperator + "\nAfter: " + d2 : "")
+                                        + "\n\n");
                         fullEvaluationError = true;
                     }
                 }
@@ -294,19 +324,6 @@ public final class MarkovChain {
                 currentModel.acceptModelState();
                 currentScore = score;
 
-//                if( otfcounter > 0 ) {
-//                    --otfcounter;
-//                    if( otfcounter == 0 ) {
-//                        adjustOpWeights(currentState);
-//                        otfcounter = onTheFlyOperatorWeights;
-//                    }
-//                }
-
-                if (usingFullEvaluation) {
-                    oldScore = score; // for the usingFullEvaluation test
-                    diagnostic = likelihood instanceof CompoundLikelihood ?
-                            ((CompoundLikelihood) likelihood).getDiagnosis() : "";
-                }
             } else {
                 if (DEBUG) {
                     System.out.println("** Move rejected: new score = " + score
@@ -318,34 +335,35 @@ public final class MarkovChain {
                 // assert Profiler.startProfile("Restore");
 
                 currentModel.restoreModelState();
+
+                if (usingFullEvaluation) {
+                    // This is a test that the state is correctly restored. The
+                    // restored state is fully evaluated and the likelihood compared with
+                    // that before the operation was made.
+
+                    likelihood.makeDirty();
+                    final double testScore = evaluate(likelihood, prior);
+
+                    final String d2 = likelihood instanceof CompoundLikelihood ?
+                            ((CompoundLikelihood) likelihood).getDiagnosis() : "";
+
+                    if (Math.abs(testScore - oldScore) > evaluationTestThreshold) {
+
+
+                        final Logger logger = Logger.getLogger("error");
+                        logger.severe("State "+currentState+": State was not correctly restored after reject step.\n"
+                                + "Likelihood before: " + oldScore
+                                + " Likelihood after: " + testScore
+                                + "\n" + "Operator: " + mcmcOperator
+                                + " " + mcmcOperator.getOperatorName()
+                                + (diagnosticStart.length() > 0 ? "\n\nDetails\nBefore: " + diagnosticStart + "\nAfter: " + d2 : "")
+                                + "\n\n");
+                        fullEvaluationError = true;
+                    }
+                }
             }
             // assert Profiler.stopProfile("Restore");
 
-            if (usingFullEvaluation) {
-                // This is a test that the state is correctly restored. The
-                // restored state is fully evaluated and the likelihood compared with
-                // that before the operation was made.
-
-                likelihood.makeDirty();
-                final double testScore = evaluate(likelihood, prior);
-
-                final String d2 = likelihood instanceof CompoundLikelihood ?
-                        ((CompoundLikelihood) likelihood).getDiagnosis() : "";
-
-                if (Math.abs(testScore - oldScore) > evaluationTestThreshold) {
-
-
-                    final Logger logger = Logger.getLogger("error");
-                    logger.severe("State was not correctly restored after reject step.\n"
-                            + "Likelihood before: " + oldScore
-                            + " Likelihood after: " + testScore
-                            + "\n" + "Operator: " + mcmcOperator
-                            + " " + mcmcOperator.getOperatorName()
-                            + (diagnostic.length() > 0 ? "\n\nDetails\nBefore: " + diagnostic + "\nAfter: " + d2 : "")
-                    );
-                    fullEvaluationError = true;
-                }
-            }
 
             if (!disableCoerce && mcmcOperator instanceof CoercableMCMCOperator) {
                 coerceAcceptanceProbability((CoercableMCMCOperator) mcmcOperator, logr[0]);
@@ -384,55 +402,6 @@ public final class MarkovChain {
 
         // Profiler.report();
     }
-//
-//    private void adjustOpWeights(int currentState) {
-//        final int count = schedule.getOperatorCount();
-//        double[] s = new double[count];
-//        final double factor = 100;
-//        final double limitSpan = 1000;
-//        System.err.println("start cycle " + currentState);
-//
-//        double sHas = 0.0/* , sNot = 0.0 */, nHas = 0.0;
-//        for(int no = 0; no < count; ++no) {
-//            final MCMCOperator op = schedule.getOperator(no);
-//            final double v = op.getSpan(true);
-//
-//            if( v == 0 ) {
-//                // sNot += op.getWeight();
-//                s[no] = 0;
-//            } else {
-//                sHas += op.getWeight();
-//                s[no] = Math.max(factor * Math.min(v, limitSpan), 1);
-//                nHas += s[no];
-//            }
-//        }
-//
-//        // for(int no = 0; no < count; ++no) {
-//        // final MCMCOperator op = schedule.getOperator(no);
-//        // final double v = op.getSpan(false);
-//        // if( v == 0 ) {
-//        // System.err.println(op.getOperatorName() + " blocks");
-//        // return;
-//        // }
-//        // }
-//
-//        // keep sum of changed parts unchanged
-//        final double scaleHas = sHas / nHas;
-//
-//        for(int no = 0; no < count; ++no) {
-//            final MCMCOperator op = schedule.getOperator(no);
-//            if( s[no] > 0 ) {
-//                final double val = s[no] * scaleHas;
-//                op.setWeight(val);
-//                System.err.println("set " + op.getOperatorName() + " " + val);
-//            } else {
-//                System.err.println("** " + op.getOperatorName() + " = "
-//                        + op.getWeight());
-//            }
-//        }
-//        schedule.operatorsHasBeenUpdated();
-//    }
-
 
     public Prior getPrior() {
         return prior;
@@ -482,6 +451,10 @@ public final class MarkovChain {
         return isStopped;
     }
 
+    public double evaluate() {
+        return evaluate(likelihood, prior);
+    }
+
     protected double evaluate(Likelihood likelihood, Prior prior) {
 
         double logPosterior = 0.0;
