@@ -1,7 +1,7 @@
 /*
  * MarginalLikelihoodEstimator.java
  *
- * Copyright (c) 2002-2012 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -49,7 +49,7 @@ import java.util.List;
  */
 public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
 
-    public MarginalLikelihoodEstimator(String id, int chainLength, int burninLength, int pathSteps, double fixedRunValue,
+    public MarginalLikelihoodEstimator(String id, int chainLength, int burninLength, int pathSteps, double[] fixedRunValues,
 //                                       boolean linear, boolean lacing,
                                        PathScheme scheme,
                                        PathLikelihood pathLikelihood,
@@ -61,7 +61,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
         this.pathSteps = pathSteps;
         this.scheme = scheme;
         this.schedule = schedule;
-        this.fixedRunValue = fixedRunValue;
+        this.fixedRunValues = fixedRunValues;
         // deprecated
         // this.linear = (scheme == PathScheme.LINEAR);
         // this.lacing = false; // Was not such a good idea
@@ -111,9 +111,9 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
             mc.setCurrentLength(cl);
             mc.runChain(chainLength, false);
 
-            // AR - I have commented this out. Was it being used for debugging or did it have
-            // actual usefulness?
-            // (new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
+            if (SHOW_OPERATOR_ANALYSIS) {
+            	(new OperatorAnalysisPrinter(schedule)).showOperatorAnalysis(System.out);
+            }
             ((CombinedOperatorSchedule) schedule).reset();
         }
     }
@@ -134,17 +134,17 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
     }
 
     public class FixedThetaRun extends Integrator {
-        private double value;
+        private double[] value;
 
-        public FixedThetaRun(double value) {
-            super(1);
+        public FixedThetaRun(double[] value) {
+            super(value.length);
             this.value = value;
         }
 
         double nextPathParameter() {
-            if (step == 0) {
+            if (step < value.length) {
                 step++;
-                return value;
+                return value[step-1];
             } else {
                 return -1.0;
             }
@@ -302,7 +302,11 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
     }*/
 
     private void reportIteration(double pathParameter, long chainLength, long burnin, long totalSteps, long steps) {
-        System.out.println("Attempting theta ("+steps+"/" + (totalSteps+1) +") = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
+    	if (scheme == PathScheme.FIXED) {
+    		System.out.println("Attempting fixed theta ("+steps+"/" + (totalSteps) +") = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
+    	} else {
+    		System.out.println("Attempting theta ("+steps+"/" + (totalSteps+1) +") = " + pathParameter + " for " + chainLength + " iterations + " + burnin + " burnin.");
+    	}
     }
 
     public void run() {
@@ -331,7 +335,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
 
         switch (scheme) {
             case FIXED:
-                integrate(new FixedThetaRun(fixedRunValue));
+                integrate(new FixedThetaRun(fixedRunValues));
                 break;
             case LINEAR:
                 integrate(new LinearIntegrator(pathSteps));
@@ -449,10 +453,23 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
                 }
             }
 
-//            MCLogger logger = (MCLogger) xo.getChild(MCLogger.class);
+            //MCLogger logger = (MCLogger) xo.getChild(MCLogger.class);
 
             int chainLength = xo.getIntegerAttribute(CHAIN_LENGTH);
-            int pathSteps = xo.getIntegerAttribute(PATH_STEPS);
+            int pathSteps = -1;
+            double[] fixedRunValues = null;
+            if (xo.hasAttribute(PATH_STEPS)) {
+            	pathSteps = xo.getIntegerAttribute(PATH_STEPS);
+            } else if (xo.hasAttribute(FIXED_VALUE)) {
+                fixedRunValues = xo.getDoubleArrayAttribute(FIXED_VALUE);
+            } else {
+            	throw new RuntimeException("Either a number of path steps or predefined beta values need to be provided.");
+            }
+            
+            if (xo.hasAttribute(PRINT_OPERATOR_ANALYSIS)) {
+            	SHOW_OPERATOR_ANALYSIS = xo.getBooleanAttribute(PRINT_OPERATOR_ANALYSIS);
+            }
+            
             int burninLength = -1;
             if (xo.hasAttribute(BURNIN)) {
                 burninLength = xo.getIntegerAttribute(BURNIN);
@@ -462,11 +479,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
             if (xo.hasAttribute(PRERUN)) {
                 prerunLength = xo.getIntegerAttribute(PRERUN);
             }
-            double fixedRunValue = -1.0;
-            if (xo.hasAttribute(FIXED_VALUE)) {
-                fixedRunValue = xo.getDoubleAttribute(FIXED_VALUE);
-            }
-
+            
             // deprecated
             boolean linear = xo.getAttribute(LINEAR, true);
             // boolean lacing = xo.getAttribute(LACING,false);
@@ -501,7 +514,11 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
                         }
                         mcmc.getMarkovChain().runChain(prerunLength, false);
                     }
-                    os.addOperatorSchedule(mcmc.getOperatorSchedule());
+                    if (xo.getChild(OperatorSchedule.class) != null) {
+                    	os.addOperatorSchedule((OperatorSchedule)xo.getChild(OperatorSchedule.class));
+                    } else {
+                    	os.addOperatorSchedule(mcmc.getOperatorSchedule());
+                    }
                 }
             }
 
@@ -510,7 +527,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
             }
 
             MarginalLikelihoodEstimator mle = new MarginalLikelihoodEstimator(MARGINAL_LIKELIHOOD_ESTIMATOR, chainLength,
-                    burninLength, pathSteps, fixedRunValue, scheme, pathLikelihood, os, loggerList);
+                    burninLength, pathSteps, fixedRunValues, scheme, pathLikelihood, os, loggerList);
 
             if (!xo.getAttribute(SPAWN, true))
                 mle.setSpawnable(false);
@@ -542,7 +559,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
                     "\n    2012. Improving the accuracy of demographic and molecular clock model comparison while accommodating " +
                     "\n          phylogenetic uncertainty. Mol. Biol. Evol. 29(9):2157-2167." +
                     "\n    and " +
-                    "\n    Guy Baele, Wai Lok Sibon Li, Alexei J. Drummond, Marc A. Suchard, and Philippe Lemey. 2013" +
+                    "\n    Guy Baele, Wai Lok Sibon Li, Alexei J. Drummond, Marc A. Suchard, and Philippe Lemey. 2013." +
                     "\n    Accurate model selection of relaxed molecular clocks in Bayesian phylogenetics. Mol. Biol. Evol. 30(2):239-243.\n");
             return mle;
         }
@@ -565,14 +582,15 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
 
         private final XMLSyntaxRule[] rules = {
                 AttributeRule.newIntegerRule(CHAIN_LENGTH),
-                AttributeRule.newIntegerRule(PATH_STEPS),
+                AttributeRule.newIntegerRule(PATH_STEPS, true),
                 AttributeRule.newIntegerRule(BURNIN, true),
                 AttributeRule.newIntegerRule(PRERUN, true),
                 AttributeRule.newBooleanRule(LINEAR, true),
                 AttributeRule.newBooleanRule(LACING, true),
                 AttributeRule.newBooleanRule(SPAWN, true),
+                AttributeRule.newBooleanRule(PRINT_OPERATOR_ANALYSIS, true),
                 AttributeRule.newStringRule(PATH_SCHEME, true),
-                AttributeRule.newDoubleRule(FIXED_VALUE, true),
+                AttributeRule.newDoubleArrayRule(FIXED_VALUE, true),
                 AttributeRule.newDoubleRule(ALPHA, true),
                 AttributeRule.newDoubleRule(BETA, true),
                 new ElementRule(MCMC,
@@ -642,7 +660,7 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
     private final PathScheme scheme;
     private double alphaFactor = 0.5;
     private double betaFactor = 0.5;
-    private double fixedRunValue = -1.0;
+    private double[] fixedRunValues;
     private final double pathDelta;
     private double pathParameter;
 
@@ -660,8 +678,11 @@ public class MarginalLikelihoodEstimator implements Runnable, Identifiable {
     public static final String BURNIN = "burnin";
     public static final String MCMC = "samplers";
     public static final String PATH_SCHEME = "pathScheme";
-    public static final String FIXED_VALUE = "fixedValue";
+    public static final String FIXED_VALUE = "fixedValues";
     public static final String ALPHA = "alpha";
     public static final String BETA = "beta";
     public static final String PRERUN = "prerun";
+    public static final String PRINT_OPERATOR_ANALYSIS = "printOperatorAnalysis";
+    
+    private static boolean SHOW_OPERATOR_ANALYSIS = false;
 }
\ No newline at end of file
