@@ -50,10 +50,12 @@ import java.util.List;
  * An MCMC analysis that estimates parameters of a probabilistic model.
  *
  * @author Andrew Rambaut
- * @version $Id: MCMCMC.java,v 1.12 2005/01/10 10:56:59 rambaut Exp $
+ * @version $Id: ParallelMCMC.java,v 1.12 2005/01/10 10:56:59 rambaut Exp $
  */
 public class MCMCMC implements Runnable {
 
+    public final static boolean DEBUG = false;
+
     public MCMCMC(MCMC[] mcmcs, MCMCMCOptions mcmcmcOptions) {
 
         this.mcmcmcOptions = mcmcmcOptions;
@@ -104,17 +106,17 @@ public class MCMCMC implements Runnable {
 
         timer.start();
 
-        if (isPreBurninNeeded()) {
-            long preBurnin = mcmcOptions.getCoercionDelay();
-            if (preBurnin > 0) {
-                MarkovChainListener burninListener = new BurninListener(preBurnin);
-
-                chains[coldChain].addMarkovChainListener(burninListener);
-                runChains(preBurnin, true);
-                chains[coldChain].removeMarkovChainListener(burninListener);
-                resetChains();
-            }
-        }
+//        if (isPreBurninNeeded()) {
+//            long preBurnin = mcmcOptions.getCoercionDelay();
+//            if (preBurnin > 0) {
+//                MarkovChainListener burninListener = new BurninListener(preBurnin);
+//
+//                chains[coldChain].addMarkovChainListener(burninListener);
+//                runChains(preBurnin, true);
+//                chains[coldChain].removeMarkovChainListener(burninListener);
+//                resetChains();
+//            }
+//        }
 
         MCLogger[] coldChainLoggers = mcLoggers[coldChain];
         List<LogFormatter>[] logFormatters = new List[coldChainLoggers.length];
@@ -226,6 +228,21 @@ public class MCMCMC implements Runnable {
 
     private int swapChainTemperatures() {
 
+        if(DEBUG){
+            System.out.print("Current scores: ");
+            for(int i=0; i<chains.length; i++){
+                System.out.print("\t");
+                if(i==coldChain){
+                    System.out.print("[");
+                }
+                System.out.print(chains[i].getCurrentScore());
+                if(i==coldChain){
+                    System.out.print("]");
+                }
+            }
+            System.out.println();
+        }
+
         int newColdChain = coldChain;
 
         int index1 = MathUtils.nextInt(chains.length);
@@ -245,6 +262,10 @@ public class MCMCMC implements Runnable {
         boolean swap = (Math.log(MathUtils.nextDouble()) < logRatio);
 
         if (swap) {
+            if(DEBUG){
+                System.out.println("Swapping chain "+index1+" and chain "+index2);
+            }
+
             acceptor1.setTemperature(temperature2);
             acceptor2.setTemperature(temperature1);
 
