@@ -415,17 +415,17 @@ public interface Parameter extends Statistic, Variable<Double> {
                 //
             }
             final String id = getId();
-            if (id != null) buffer.append(id);
+            if (id != null) buffer.append(", ").append(id);
             if (bounds != null) {
                 buffer.append("=[").append(String.valueOf(bounds.getLowerLimit(0)));
-                buffer.append(",").append(String.valueOf(bounds.getUpperLimit(0))).append("]");
+                buffer.append(", ").append(String.valueOf(bounds.getUpperLimit(0))).append("]");
             }
 
             for (int i = 1; i < getDimension(); i++) {
                 buffer.append(", ").append(String.valueOf(getParameterValue(i)));
                 if (bounds != null) {
                     buffer.append("[").append(String.valueOf(bounds.getLowerLimit(i)));
-                    buffer.append(",").append(String.valueOf(bounds.getUpperLimit(i))).append("]");
+                    buffer.append(", ").append(String.valueOf(bounds.getUpperLimit(i))).append("]");
                 }
             }
             return buffer.toString();
@@ -643,7 +643,6 @@ public interface Parameter extends Statistic, Variable<Double> {
             System.arraycopy(values, 0, newValues, 0, index);
             System.arraycopy(values, index, newValues, index - 1, n - index);
             values = newValues;
-
             fireParameterChangedEvent(index, Parameter.ChangeType.REMOVED);
             return value;
         }
@@ -679,7 +678,7 @@ public interface Parameter extends Statistic, Variable<Double> {
         protected final void storeValues() {
             // no need to pay a price in a very common call for one-time rare usage
             //hasBeenStored = true;
-            if (storedValues == null) {
+            if (storedValues == null || storedValues.length != values.length) {
                 storedValues = new double[values.length];
             }
             System.arraycopy(values, 0, storedValues, 0, storedValues.length);
