@@ -1,7 +1,7 @@
 /*
  * Variable.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -78,7 +78,7 @@ public interface Variable<V> extends Identifiable {
     void removeVariableListener(VariableListener listener);
 
     /**
-     * stores the state of this parameter for subsquent restore
+     * stores the state of this parameter for subsequent restore
      */
     void storeVariableValues();
 
@@ -265,7 +265,7 @@ public interface Variable<V> extends Identifiable {
                     }
 
                     public Double getLowerLimit(int dimension) {
-                        return Double.MIN_VALUE;
+                        return -Double.MAX_VALUE;
                     }
 
                     public int getBoundsDimension() {
@@ -340,7 +340,7 @@ public interface Variable<V> extends Identifiable {
             values = new double[v.length][v[0].length];
             for (int i = 0; i < v.length; i++) {
                 System.arraycopy(v[i], 0, values[i], 0, v[i].length);
-                lower[i] = Double.MIN_VALUE;
+                lower[i] = -Double.MAX_VALUE;
                 upper[i] = Double.MAX_VALUE;
             }
             storedValues = new double[values.length][values[0].length];
