@@ -1,7 +1,7 @@
 /*
  * LoggerParser.java
  *
- * Copyright (C) 2002-2009 Alexei Drummond and Andrew Rambaut
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -12,10 +12,10 @@
  * published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
- * BEAST is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with BEAST; if not, write to the
@@ -47,6 +47,7 @@ public class LoggerParser extends AbstractXMLObjectParser {
     public static final String ECHO = "echo";
     public static final String ECHO_EVERY = "echoEvery";
     public static final String TITLE = "title";
+    public static final String HEADER = "header";
     public static final String FILE_NAME = FileHelpers.FILE_NAME;
     public static final String FORMAT = "format";
     public static final String TAB = "tab";
@@ -88,17 +89,30 @@ public class LoggerParser extends AbstractXMLObjectParser {
         // added a performance measurement delay to avoid the full evaluation period.
         final MCLogger logger = new MCLogger(formatter, logEvery, performanceReport, 10000);
 
+        String title = null;
         if (xo.hasAttribute(TITLE)) {
-            logger.setTitle(xo.getStringAttribute(TITLE));
-        } else {
+            title = xo.getStringAttribute(TITLE);
+        }
 
+        String header = null;
+        if (xo.hasAttribute(HEADER)) {
+            header = xo.getStringAttribute(HEADER);
+        }
+
+        if (title == null) {
             final BeastVersion version = new BeastVersion();
 
-            final String title = "BEAST " + version.getVersionString() + "\n" +
+            title = "BEAST " + version.getVersionString() + "\n" +
+                    (header != null ? header + "\n" : "") +
                     "Generated " + (new Date()).toString() + " [seed=" + MathUtils.getSeed() + "]";
-            logger.setTitle(title);
+        } else {
+            if (header != null) {
+                title += "\n" + header;
+            }
         }
 
+        logger.setTitle(title);
+
         for (int i = 0; i < xo.getChildCount(); i++) {
 
             final Object child = xo.getChild(i);
@@ -146,6 +160,8 @@ public class LoggerParser extends AbstractXMLObjectParser {
                             "If no file name is specified then log is sent to standard output", true),
             new StringAttributeRule(TITLE,
                     "The title of the log", true),
+            new StringAttributeRule(HEADER,
+                    "The subtitle of the log", true),
             new OrRule(
                     new XMLSyntaxRule[]{
                             new ElementRule(Columns.class, 1, Integer.MAX_VALUE),
