@@ -57,7 +57,7 @@ public class MathUtils {
 	// Chooses one category if a cumulative probability distribution is given
 	public static int randomChoice(double[] cf) {
 
-		double U = random.nextDouble();
+		double U = MathUtils.nextDouble();
 
 		int s;
 		if (U <= cf[0]) {
@@ -80,7 +80,7 @@ public class MathUtils {
 	 */
 	public static int randomChoicePDF(double[] pdf) {
 
-		double U = random.nextDouble() * getTotal(pdf);
+		double U = MathUtils.nextDouble() * getTotal(pdf);
 		for (int i = 0; i < pdf.length; i++) {
 
 			U -= pdf[i];
@@ -92,11 +92,45 @@ public class MathUtils {
 		for (int i = 0; i < pdf.length; i++) {
 			System.out.println(i + "\t" + pdf[i]);
 		}
-		throw new Error("randomChoiceUnnormalized falls through -- negative components in input distribution?");
+		throw new Error("randomChoicePDF falls through -- negative, infinite or NaN components in input " +
+                "distribution, or all zeroes?");
 	}
 
+    /**
+     * @param logpdf array of unnormalised log probabilities
+     * @return a sample according to an unnormalised probability distribution
+     *
+     * Use this if probabilities are rounding to zero when converted to real space
+     */
+    public static int randomChoiceLogPDF(double[] logpdf) {
 
-	/**
+        double scalingFactor=Double.NEGATIVE_INFINITY;
+
+        for (double aLogpdf : logpdf) {
+            if (aLogpdf > scalingFactor) {
+                scalingFactor = aLogpdf;
+            }
+        }
+
+        if(scalingFactor == Double.NEGATIVE_INFINITY){
+            throw new Error("randomChoiceLogPDF falls through -- all -INF components in input distribution");
+        }
+
+        for(int j=0; j<logpdf.length; j++){
+            logpdf[j] = logpdf[j] - scalingFactor;
+        }
+
+        double[] pdf = new double[logpdf.length];
+
+        for(int j=0; j<logpdf.length; j++){
+            pdf[j] = Math.exp(logpdf[j]);
+        }
+
+        return randomChoicePDF(pdf);
+
+    }
+
+    /**
 	 * @param array to normalize
 	 * @return a new double array where all the values sum to 1.
 	 *         Relative ratios are preserved.
@@ -208,6 +242,16 @@ public class MathUtils {
 		}
 	}
 
+    //Mean = alpha/(alpha+beta)
+    //Variance = (alpha*beta)/(alpha+beta)^2*(alpha+beta+1)
+
+    public static double nextBeta(double alpha, double beta){
+        double x = nextGamma(alpha, 1);
+        double y = nextGamma(beta, 1);
+        return x/(x+y);
+    }
+
+
 	/**
 	 * Access a default instance of this class, access is synchronized
 	 *
