@@ -1,7 +1,34 @@
+/*
+ * GammaKDEDistribution.java
+ *
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ *
+ * This file is part of BEAST.
+ * See the NOTICE file distributed with this work for additional
+ * information regarding copyright ownership and licensing.
+ *
+ * BEAST is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ *  BEAST is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with BEAST; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ */
+
 package dr.math.distributions;
 
 import dr.stats.DiscreteStatistics;
 
+import java.util.Random;
+
 
 /**
  * @author Jennifer Tom
@@ -69,6 +96,28 @@ public class GammaKDEDistribution extends KernelDensityEstimatorDistribution {
 
     }
 
+    public static void main(String[] args) {
+
+        long start = System.currentTimeMillis();
+
+        Random random = new Random(1234);
+
+        Double[] samples = new Double[10000000];
+        for (int i = 0; i < samples.length; i++) {
+            samples[i] = random.nextDouble();
+        }
+        GammaKDEDistribution nKDE = new GammaKDEDistribution(samples);
+
+        for (int i = 0; i < 100; i++) {
+            nKDE.evaluateKernel(random.nextDouble());
+        }
+
+        long end = System.currentTimeMillis();
+
+        System.out.println("Time: " + (end-start));
+
+    }
+
 //    private double sampleMean() {return DiscreteStatistics.mean(sample);}
 
 
