@@ -2,10 +2,11 @@ package dr.math.distributions;
 
 /**
  * @author Marc Suchard
+ * @author Guy Baele
  */
 public class MultivariateGammaDistribution implements MultivariateDistribution {
 
-    // todo Currently this implements a product of independent Gammas, need to re-code as true multivariate distribution
+    //TODO: Currently this implements a product of independent Gammas, need to re-code as true multivariate distribution
 
     public static final String TYPE = "multivariateGamma";
 
@@ -18,6 +19,25 @@ public class MultivariateGammaDistribution implements MultivariateDistribution {
 
         this.shape = shape;
         this.scale = scale;
+        
+        this.flags = new boolean[dim];
+        for (int i = 0; i < dim; i++) {
+        	flags[i] = true;
+        }
+
+    }
+    
+    public MultivariateGammaDistribution(double[] shape, double[] scale, boolean[] flags) {
+
+        if (shape.length != scale.length)
+            throw new RuntimeException("Creation error in MultivariateGammaDistribution");
+
+        dim = shape.length;
+
+        this.shape = shape;
+        this.scale = scale;
+        
+        this.flags = flags;
 
     }
 
@@ -29,9 +49,11 @@ public class MultivariateGammaDistribution implements MultivariateDistribution {
             throw new IllegalArgumentException("data array is of the wrong dimension");
         }
 
-        for (int i = 0; i < dim; i++)
-            logPdf += GammaDistribution.logPdf(x[i],
-                    shape[i], scale[i]);
+        for (int i = 0; i < dim; i++) {
+        	if (flags[i]) { 
+        		logPdf += GammaDistribution.logPdf(x[i], shape[i], scale[i]);
+        	}
+        }
 
         return logPdf;
     }
@@ -51,4 +73,8 @@ public class MultivariateGammaDistribution implements MultivariateDistribution {
     private double[] shape;
     private double[] scale;
     private int dim;
+    
+    //for each flag that is true, add the logPdf of that gamma distribution to the overall logPdf
+    private boolean[] flags;
+    
 }
