@@ -113,7 +113,7 @@ public class Matrix {
      */
     public Matrix add(Matrix a) throws IllegalDimension {
         if (a.rows() != rows() || a.columns() != columns())
-            throw new IllegalDimension("Operation error: cannot add a"
+            throw new IllegalDimension("Operation error: cannot add a "
                     + a.rows() + " by " + a.columns()
                     + " matrix to a " + rows() + " by "
                     + columns() + " matrix");
@@ -273,7 +273,7 @@ public class Matrix {
     public Matrix product(Matrix a) throws IllegalDimension {
         if (a.rows() != columns())
             throw new IllegalDimension(
-                    "Operation error: cannot multiply a"
+                    "Operation error: cannot multiply a "
                             + rows() + " by " + columns()
                             + " matrix with a " + a.rows()
                             + " by " + a.columns()
@@ -338,6 +338,14 @@ public class Matrix {
         return new Matrix(productWithTransposedComponents(a));
     }
 
+    public static Matrix buildIdentityTimesElementMatrix(int dim, double element){
+        double[][] idTemp=new double[dim][dim];
+        for (int i = 0; i < dim; i++) {
+            idTemp[i][i]=element;
+        }
+        return new Matrix(idTemp);
+    }
+
     /**
      * @param a MatrixAlgebra.Matrix
      * @return double[][]    the components of the product of the receiver
@@ -431,7 +439,7 @@ public class Matrix {
         int m = this.columns();
         double[][] newComponents = new double[n][m];
         for (int i = 0; i < n; i++) {
-            for (int j = 0; j < n; j++)
+            for (int j = 0; j < m; j++)
                 newComponents[i][j] = components[i][j] - a.components[i][j];
         }
         return newComponents;
