@@ -1,7 +1,7 @@
 /*
  * Transform.java
  *
- * Copyright (c) 2002-2013 Alexei Drummond, Andrew Rambaut and Marc Suchard
+ * Copyright (c) 2002-2015 Alexei Drummond, Andrew Rambaut and Marc Suchard
  *
  * This file is part of BEAST.
  * See the NOTICE file distributed with this work for additional
@@ -82,12 +82,8 @@ public interface Transform {
             return "log";
         }
 
-//        public double getLogJacobian(double oldValue, double newValue) {
-//            return Math.log(newValue * (1.0 / oldValue));
-//        }
-
         public double getLogJacobian(double value) {
-            return Math.log(value);
+            return -Math.log(value);
         }
 
         public List<Citation> getCitations() {
@@ -99,7 +95,7 @@ public interface Transform {
                             new Author("P", "Lemey"),
                     },
                     Citation.Status.IN_PREPARATION
-            ));
+                    ));
             return citations;
         }
     }
@@ -121,13 +117,30 @@ public interface Transform {
             return "logit";
         }
 
-//        public double getLogJacobian(double oldValue, double newValue) {
-//            // TODO Code review; this seems wrong, as it is lacking symmetric in old/newValue
-//            return Math.log((1.0 - newValue) * (1.0 / oldValue - 1.0 / (1.0 - oldValue)));
-//        }
+        public double getLogJacobian(double value) {
+            return -Math.log(1.0 - value) - Math.log(value);
+        }
+    }
+
+    public static class FisherZTransform implements Transform {
+
+        public FisherZTransform() {
+        }
+
+        public double transform(double value) {
+            return 0.5 * (Math.log(1.0 + value) - Math.log(1.0 - value));
+        }
+
+        public double inverse(double value) {
+            return (Math.exp(2 * value) - 1) / (Math.exp(2 * value) + 1);
+        }
+
+        public String getTransformName() {
+            return "fisherz";
+        }
 
         public double getLogJacobian(double value) {
-            return Math.log(1.0 - value) - Math.log(value);
+            return -Math.log(1 - value) - Math.log(1 + value);
         }
     }
 
@@ -218,8 +231,9 @@ public interface Transform {
     public static final LogTransform LOG = new LogTransform();
     public static final LogitTransform LOGIT = new LogitTransform();
     public static final NoTransform NONE = new NoTransform();
+    public static final FisherZTransform FISHER_Z = new FisherZTransform();
 
-    public static final Transform[] transformList = {LOG, LOGIT, NONE};
+    public static final Transform[] transformList = {LOG, LOGIT, NONE, FISHER_Z};
 
     public static final String TRANSFORM = "transform";
     public static final String TYPE = "type";
