@@ -1,5 +1,7 @@
 package test.dr.app.beagle;
 
+import dr.app.beagle.evomodel.branchmodel.BranchModel;
+import dr.app.beagle.evomodel.branchmodel.HomogeneousBranchModel;
 import dr.app.beagle.evomodel.sitemodel.HomogenousBranchSubstitutionModel;
 import test.dr.inference.trace.TraceCorrelationAssert;
 import dr.evolution.datatype.Nucleotides;
@@ -57,16 +59,15 @@ public class MarkovJumpsTest extends TraceCorrelationAssert {
         //treeLikelihood
         SitePatterns patterns = new SitePatterns(alignment, null, 0, -1, 1, true);
 
-        BranchSubstitutionModel branchSubstitutionModel = new HomogenousBranchSubstitutionModel(
-                siteRateModel.getSubstitutionModel(),
-                siteRateModel.getSubstitutionModel().getFrequencyModel());
+        BranchModel branchModel = new HomogeneousBranchModel(
+                siteRateModel.getSubstitutionModel());
 
         BranchRateModel branchRateModel = null;
 
         MarkovJumpsBeagleTreeLikelihood mjTreeLikelihood = new MarkovJumpsBeagleTreeLikelihood(
                 patterns,
                 treeModel,
-                branchSubstitutionModel,
+                branchModel,
                 siteRateModel,
                 branchRateModel,
                 null,
@@ -75,7 +76,6 @@ public class MarkovJumpsTest extends TraceCorrelationAssert {
                 null,
                 hky.getDataType(),
                 "stateTag",
-                hky,
                 false, // use MAP
                 true, // return ML
                 false, // use uniformization
