@@ -11,6 +11,7 @@ public class EmpiricalBayesPoissonSmootherTest extends MathTestCase {
 
     public void testSmootherMean() {
 
+        MathUtils.setSeed(666);
         int length = 100;
 
         double[] in = new double[length];
@@ -18,6 +19,16 @@ public class EmpiricalBayesPoissonSmootherTest extends MathTestCase {
             in[i] = MathUtils.nextDouble() * 100.0;
         }
 
+        doSmoothing(in);
+    }
+
+    public void testUnstableMean() {
+        double[] in = new double[] {0, 1};
+        doSmoothing(in);
+    }
+
+    private void doSmoothing(double[] in) {
+
         double meanX = DiscreteStatistics.mean(in);
         double varX = DiscreteStatistics.variance(in);
         System.err.println("Original mean: " + meanX);
@@ -29,9 +40,10 @@ public class EmpiricalBayesPoissonSmootherTest extends MathTestCase {
         double varY = DiscreteStatistics.variance(out);
         System.err.println("Smoothed mean: " + meanY);
         System.err.println("Smoothed var : " + varY);
+        System.err.println("");
 
         assertEquals(meanX, meanY, tolerance);
-        assertTrue(varY < varX);
+        assertTrue(varY <= varX);
     }
 
     private static final double tolerance = 10E-6;
